/**
 * CommonClientAPI.java
 * @author Yi Huang (yihuan@cs.indiana.edu)
 * Created:Dec 8, 2005 10:01:04 AM
 */

package wsmg;

import java.io.StringReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.Vector;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;

import wsmg.msg_box.MsgBoxClient;
import wsmg.pull.MessagePuller;
import wsmg.stub.NotificationProducerStub;
import wsmg.stub.WseEventSourceStub;
import wsmg.util.WsmgUtil;
import wsmg.wsrm.axis2.DedicatedSimpleSandeshaServer;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.soap11_util.Soap11Util;
import xsul.ws_addressing.WsAddressing;
import xsul.ws_addressing.WsaEndpointReference;
import xsul.ws_addressing.WsaInvoker;

public abstract class CommonClientProcessing {
    protected final static MLogger logger = MLogger.getLogger();

    protected final static XmlInfosetBuilder builder = XmlConstants.BUILDER;




    protected static WSMGService service;

 
    protected static XmlConsumer xc;

    protected static DedicatedSimpleSandeshaServer wsrmServer;

    public final static XmlNamespace WSNT_NS = builder.newNamespace("wsnt",
            "http://www.ibm.com/xmlns/stdwip/web-services/WS-BaseNotification");

    private static String SANDESHA2_HOME = "F:\\Sandesha2-0.9-src"; // Change
                                                                    // this to
                                                                    // ur path.

    private static String AXIS2_SERVER_PATH = SANDESHA2_HOME
            + "\\target\\repos\\server\\";

    protected MsgBoxClient msgBoxService = null;

    // private WsaEndpointReference msgBoxAddr=null;

    public CommonClientProcessing() {
        WsmgVersion.requireVersionOrExit(WsmgVersion.getSpecVersion());

    }

    /**
     * 
     * @param carrier
     *            specify the carrier. narada or openJms or null
     * @param port
     *            specify port number of the BrokerService
     * @return
     */
    public int startBrokerService(int port, String carrier,
            String carrierLocation, boolean wsrmEnabled, int rmport) {

        // Build the WSMGConfig which represents the configuration of the
        // brocker
        WsmgConfig wsmgConfig = new WsmgConfig(carrier, carrierLocation);
        if (wsrmEnabled) {
            wsmgConfig.setWsrmEnabled(true);
            wsmgConfig.setRmport(rmport);
        }
        wsmgConfig.setPort(port);
        wsmgConfig
                .setBrokerWebServiceStack(WsmgConstants.BROKER_WEB_SERVICE_STACK_STANDALONE_XSUL);

        service = new WSMGService(wsmgConfig);

        service.start();
        //		

        return 0;
    }

    /**
     * shutdown Broker service
     */
    public void shutdownBrokerService() {
        if (service != null) {
            System.out
                    .println("WS-Eventing/Notification Broker Service shutdown requested");
            service.shutdown();
        }
    }

    /**
     * 
     * @param port
     *            specify the port number of the WS-Eventing/Notification
     *            eventSink service Use 0 as port number to let xsul
     *            automatically select a port from available ports pass this
     *            option to JVM: -Dhttp_transport.server.port.range=START-END to
     *            specify a range of port to choose
     */
    public int startConsumerService(int port, NotificationHandler handler) {
        xc = new XmlConsumer(port, handler);
        int listenPort = 0;
        xc.start();
        listenPort = xc.getListenPort();
        return listenPort;
    }

    /**
     * shutdown the eventSink service
     */
    public void shutdownConsumerService() {
        if (xc != null) {
            System.out.println("Consumer Service shutdown requested");
            xc.shutdown();
        }
    }

    /**
     * 
     * @param eventSourceLoc
     *            specify the location that the subscription is going to send
     *            to, e.g. localhost:19999
     * @param eventSinkLoc
     *            specify the location that will get the notification, e.g.
     *            localhost:12345
     * @param topic
     *            specify the topic string
     * @return
     */
    public String subscribe(String eventSourceLoc, String eventSinkLoc,
            String topic) {
        return subscribe(eventSourceLoc, eventSinkLoc, topic,
                null, null, null);
    }
    
    public String subscribe(String eventSourceLocIn, String eventSinkLocIn,
            String topicExpression, String xpathExpression) {
        return subscribe(eventSourceLocIn, eventSinkLocIn, topicExpression,xpathExpression, null, null);
    }
    

    public abstract String subscribe(String eventSourceLoc, String eventSinkLoc,
            String topic, String xpathExpression, String eventSinkEndpointReferenceNS,
            String eventSinkEndpointReference); 
    
    public String subscribe(String eventSourceLoc, String eventSinkLoc,
            String topic, String eventSinkEndpointReferenceNS,
            String eventSinkEndpointReference) {
        return subscribe(eventSourceLoc, eventSinkLoc, topic,
                null, eventSinkEndpointReferenceNS, eventSinkEndpointReference);
    }

//    protected String subscribe(String eventSourceLoc, String eventSinkLoc,
//            String subscriptionExpression, int subscriptionType) {
//        return subscribe(eventSourceLoc, eventSinkLoc, subscriptionExpression,
//                subscriptionType, null, null);
//    }

//    /**
//     * 
//     * @param eventSourceLoc
//     *            specify the location that the subscription is going to send
//     *            to, e.g. localhost:19999
//     * @param eventSinkLoc
//     *            specify the location that will get the notification, e.g.
//     *            localhost:12345
//     * @param subscriptionExpression
//     *            specify the topic string //Used in WS-Topics filter
//     * @return
//     */
//    protected String subscribe(String eventSourceLocIn, String eventSinkLocIn,
//            String topicExpression, int subscriptionType,
//            String eventSinkEndpointReferenceNS,
//            String eventSinkEndpointReference) {
//        return subscribe(eventSourceLocIn, eventSinkLocIn, topicExpression,null,
//                subscriptionType, null, null);
//    }
    

    
    
//    protected String subscribe(String eventSourceLocIn, String eventSinkLocIn,
//            String topicExpression, String xpathExpression, int subscriptionType,
//            String eventSinkEndpointReferenceNS,
//            String eventSinkEndpointReference) {
//             return null;
//
//    }

    /**
     * 
     * @param eventSourceLoc
     *            specify the location that the subscription is going to send
     *            to, e.g. localhost:19999
     * @param eventSinkLoc
     *            specify the location that will get the notification, e.g.
     *            localhost:12345
     * @param topic
     *            specify the topic string
     * @return
     */
    public Vector getAllSubscription(String subscriptionManagerLocIn) {
        String eventSourceLoc = WsmgUtil
                .formatURLString(subscriptionManagerLocIn);
        Vector allSubscriptions = new Vector();
        WsaEndpointReference eventSourceReference = null;

        try {
            eventSourceReference = new WsaEndpointReference(new URI(
                    eventSourceLoc));
        } catch (URISyntaxException e) {
            throw new WsmgRuntimeException("Illegal subscriptionManager URI", e);
        }
        ;

        NotificationProducerStub np = new NotificationProducerStub(
                eventSourceReference);

        WsaInvoker invoker = new WsaInvoker();
        invoker.setTargetEPR(eventSourceReference);
        invoker.setSoapFragrance(Soap11Util.getInstance());
        invoker.setIndent("  ");
        // to improve interop do not use KA
        invoker.setUseHttpKeepAlive(false);
        XmlElement message = builder.newFragment(WsmgConstants.WIDGET_NS,
                "getAllSubscriptions");
        // invoker.setDefaultAction(URI.create(message.getNamespace().getNamespaceName()+"/"+message.getName()));
        invoker.setDefaultAction(URI.create(message.getNamespace()
                .getNamespaceName()
                + "/getAllSubscriptions"));
        XmlElement responseMessage = invoker.invokeMessage(message);
        logger.fine("\n\nRESPONSE="
                + xsul.util.XsulUtil.safeXmlToString(responseMessage)
                + "\n\n\n");

        for (Iterator it = responseMessage.elements(WsmgConstants.WIDGET_NS,
                "Subscription").iterator(); it.hasNext();) {
            XmlElement subscriptionEl = (XmlElement) it.next();
            String subscriptionId = subscriptionEl.requiredElement(
                    WsmgConstants.WIDGET_NS, "SubscriptionId")
                    .requiredTextContent();
            String eventSinkAddress = subscriptionEl.requiredElement(
                    WsmgConstants.WIDGET_NS, "ConsumerAddress")
                    .requiredTextContent();

            XmlElement referencePropertyEl = subscriptionEl.element(
                    WsmgConstants.WIDGET_NS, "ReferenceProperty");
            String referenceProperty = null;
            if (referencePropertyEl == null) {
                referenceProperty = "";
            } else {
                referenceProperty = referencePropertyEl.requiredTextContent();
            }
            String topic = subscriptionEl.requiredElement(
                    WsmgConstants.WIDGET_NS, "Topic").requiredTextContent();
            SubscriptionEntry sub = new SubscriptionEntry();
            sub.setSubscriptionId(subscriptionId);
            sub.setConsumerAddress(eventSinkAddress);
            sub.setReferenceProperty(referenceProperty);
            sub.setTopic(topic);
            allSubscriptions.add(sub);
        }

        return allSubscriptions;
    }

    public int unSubscribe(String subscriptionManagerLocIn, String subId,
            XmlElement message) {
        String subscriptionManagerLoc = WsmgUtil
                .formatURLString(subscriptionManagerLocIn);
        WsaEndpointReference eventSourceReference = null;
        XmlNamespace eventSourceEPR_NS = null;
        try {
            eventSourceReference = new WsaEndpointReference(
                    WsAddressing.NS_2004_08, new URI(subscriptionManagerLoc));
        } catch (URISyntaxException e) {
            throw new WsmgRuntimeException("Illegal SubscriptionManager URI", e);
        }
        // If the subId start with <, then it is the whole element of
        // referenceProperties. It should be used for interoperability purpose
        if (subId.startsWith("<")) {
            eventSourceReference.getReferenceProperties().addElement(
                    builder.parseFragmentFromReader(new StringReader(subId)));
        } else { // If the user just provides a subId as a string, assume it
            // is generated by WS-Messenger and put it in the
            // WS-Messenger's default "Identifier" element
            eventSourceReference.getReferenceProperties().addElement(
                    WsmgConstants.WSE_NS, "Identifier").addChild(subId);
        }

        WsaInvoker invoker = new WsaInvoker();
        invoker.setMessageId(null);
        invoker.setTargetEPR(eventSourceReference);
        invoker.setDefaultAction(URI.create(message.getNamespace()
                .getNamespaceName()
                + "/" + message.getName()));
        XmlElement responseMessage = invoker.invokeMessage(message);
        // System.out.println("\n\nRESPONSE=" +
        // xsul.util.XsulUtil.safeXmlToString(responseMessage.getRoot()) +
        // "\n\n\n");
        return 0;

    }

 
    // /**
    // *
    // * @param eventSinkLoc location of eventSink service that is listening to
    // the notification, eg. localhost:19999
    // * @param publisherLoc location of this publisher, e.g. localhost:54321
    // * @param topicString the topic of the message
    // * @param message the content of the message
    // * @return
    // */
    // public int publish(String eventSinkLoc,String publisherLoc,
    // String topicString, String message) {
    // //TODO: currently ignore publisherLoc since it is not in SOAP body as
    // WSNT. Should be able to add to the header wsa:From
    // return publish(eventSinkLoc, topicString, message);
    // }

    // //publish without publisherLocation
    // public abstract int publish(String eventSinkLocIn,
    // String topicString, String message);

    /*
     * create a message box for the client to pull message from.
     * msgBoxServerLoc: the location of the messagebox service return: EPR of
     * the messagebox
     */
    public WsaEndpointReference createPullMsgBox(String msgBoxServerLoc) {
        URI messageBoxLocation = null;
        messageBoxLocation = URI.create(WsmgUtil.formatURLString(msgBoxServerLoc));

        // create mailbox
        msgBoxService = new MsgBoxClient(messageBoxLocation);
        WsaEndpointReference msgBoxAddr = msgBoxService.createMsgBox();
        System.out.println("msgBoxAddr=" + msgBoxAddr);
        return msgBoxAddr;
    }

    public MessagePuller startPullingEventsFromMsgBox(
            WsaEndpointReference msgBoxAddr, NotificationHandler handler,
            long interval) {
        XmlElement msg[] = null;
        if (msgBoxService != null) {
            msg = msgBoxService.takeMessages(msgBoxAddr);
        } else {
            throw new WsmgRuntimeException("Unable start pulling, the messagebox client was not initialized");
        }
        MessagePuller messagePuller = new MessagePuller(msgBoxService,
                msgBoxAddr, handler, interval);
        messagePuller.startPulling();
        return messagePuller;
    }
    
    public MessagePuller startPullingEventsFromAlreadyCreatedMsgBox(String msgBoxLocation,
            WsaEndpointReference msgBoxAddr, NotificationHandler handler,
            long interval) {
        XmlElement msg[] = null;
        try {
            this.msgBoxService = new MsgBoxClient(new URI(WsmgUtil.formatURLString(msgBoxLocation)));
        } catch (URISyntaxException e) {
           throw new WsmgRuntimeException(e);
        }
//        if (msgBoxService != null) {
//            msg = msgBoxService.takeMessages(msgBoxAddr);
//        } else {
//           throw new WsmgRuntimeException("Unable start pulling, the messagebox client was not initialized");
//        }
        MessagePuller messagePuller = new MessagePuller(msgBoxService,
                msgBoxAddr, handler, interval);
        
        messagePuller.startPulling();
        return messagePuller;
    }

    public void stopPullingEventsFromMsgBox(MessagePuller msgPuller) {
        msgPuller.stopPulling();
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2002-2004 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
