package wsmg;

import java.io.StringReader;
import java.net.URI;
import java.util.Iterator;
import java.util.List;

import org.apache.axis2.AxisFault;
import org.xmlpull.v1.builder.XmlBuilderException;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import wsmg.config.WSMGParameter;
import wsmg.db.MessageBoxDB;
import wsmg.jms.JMSToWsntConverter;
import wsmg.measure.profile.RunTimeStatistics;
import wsmg.processors.ConsumerInfo;
import wsmg.processors.OutGoingMessage;
import wsmg.processors.AdditionalMessageContent;
import wsmg.util.WsmgUtil;
import wsmg.wsrm.RMNotificationSender;
import wsmg.wsrm.WSRMAdaptor;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.invoker.DynamicInfosetInvokerException;
import xsul.secconv.UUID;
import xsul.soap11_util.Soap11Util;
import xsul.ws_addressing.WsaEndpointReference;
import xsul.ws_addressing.WsaInvoker;

/**
 * <p>
 * Title:
 * </p>
 * <p>
 * Description:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author not attributable
 * @version 1.0
 */

public class NotificationProducerForBroker {
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    private final static MLogger logger = MLogger.getLogger();

    private static final boolean IS_FINEST = logger.isFinestEnabled();

    // private SoapUtil soapUtil = Soap12Util.getInstance();

    protected JMSToWsntConverter converter;

    protected WsaInvoker invoker = null;

    private RMNotificationSender wsrmAdaptor = null;
    
    private MessageBoxDB messageBoxDB;

    public NotificationProducerForBroker() {
        invoker = new WsaInvoker();
        // invoker.setTargetEPR(resourceEpr);
        invoker.setSoapFragrance(Soap11Util.getInstance());
        // invoker.setIndent(" ");
        invoker.setIndent(null);
        // to improve interop do not use KA
        // invoker.setUseHttpKeepAlive(false);
        invoker.setUseHttpKeepAlive(false);

        wsrmAdaptor = new WSRMAdaptor();
    }

    public synchronized void sendNotification(OutGoingMessage outGoingMessage) {
        // XmlElement fullNotif=null;
        if (outGoingMessage == null) {
            System.out.println("ERROR:  Got a null outGoing Message");
            return;
        }
        String messageString = outGoingMessage.getTextMessage();

        List consumerInfoList = outGoingMessage.getConsumerInfoList();
        AdditionalMessageContent soapHeader = outGoingMessage.getAdditionalMessageContent();
        deliverMessage(consumerInfoList, messageString, soapHeader);
    }

    public void deliverMessage(List consumerInfoList, String messageString,
            AdditionalMessageContent additionalMessageContent) {
        XmlElement rawNotificationEl = null;
        try {
            rawNotificationEl = builder
                    .parseFragmentFromReader(new StringReader(messageString));
        } catch (XmlBuilderException e) { // catch Runtime exception to
            // prevent messenger from crashing
            // by one mal-formated message
            e.printStackTrace();
        }
        if (rawNotificationEl == null) {
            System.out.println("ERROR:  Got a null after paring Message");
            return;
        }
        deliverMessage(consumerInfoList, rawNotificationEl,
                additionalMessageContent);
    }

    public void deliverMessage(List consumerInfoList, XmlElement rawNotif,
            AdditionalMessageContent additionalMessageContent) {
        Iterator iter = consumerInfoList.iterator();
        boolean successfullyDBTransferedTOMessageBox = false;
//        int count=1;
        while (iter.hasNext()) {
//        	System.out.println("*****Iter "+count);
//        	count++;
            ConsumerInfo consumerInfo = (ConsumerInfo) iter.next();
            XmlElement messageEl = builder
                    .parseFragmentFromReader(new StringReader(consumerInfo
                            .getConsumerEprStr()));
            XmlElement address = messageEl.element(null, "Address");
            String consumerAddress = null;
            if (address.children().hasNext()) {
                consumerAddress = (String) address.children().next();
            }
             
            if (WSMGParameter.allowSaveToLocalMsgBoxDB) {
				if (this.isMessageBoxAssociatedEPR(consumerAddress)) {
					try {
						String messageboxID = consumerAddress
								.substring(consumerAddress.indexOf("/MsgBox/")
										+ "/MsgBox/".length());

						XmlElement envelop = WsmgUtil.createSoapEnvelop(
								rawNotif, consumerAddress,
								additionalMessageContent.getTopicElement(),
								additionalMessageContent.getMessageID(),
								additionalMessageContent.getAction());
						String envelopString = builder
								.serializeToString(envelop);
						if (this.messageBoxDB == null) {
							this.messageBoxDB = new MessageBoxDB();
						}
						this.messageBoxDB.addMessage(messageboxID,
								envelopString);
						successfullyDBTransferedTOMessageBox = true;
					} catch (Exception e) {
						successfullyDBTransferedTOMessageBox = false;
					}
					if (successfullyDBTransferedTOMessageBox && this
							.isMessageBoxAssociatedEPR(consumerInfo
									.getConsumerEprStr())) {
						return;
					}
				}
            }
//				if (!(successfullyDBTransferedTOMessageBox && this
//						.isMessageBoxAssociatedEPR(consumerInfo
//								.getConsumerEprStr()))) {
					deliverMessageToOneConsumer(consumerInfo, rawNotif,
							additionalMessageContent);
//				}
        }
    }

    public void deliverMessageToOneConsumer(ConsumerInfo consumerInfo,
            XmlElement rawNotif,
            AdditionalMessageContent additionalMessageContent) {
        boolean paused = consumerInfo.isPaused();
        if (paused) {
            return;
        }

        String type = consumerInfo.getType();
        boolean useNotify = consumerInfo.isUseNotify();
        WsaEndpointReference consumerReference = new WsaEndpointReference(
                builder.parseFragmentFromReader(new StringReader(consumerInfo
                        .getConsumerEprStr())));

        if (rawNotif == null) {
            logger.info("IGNORED");
            return;
        }

        // The xmlElement sent to the invoker cannot have parent,
        // otherwise, WsaInvoker will check the parent and don't add SOAP
        // envelop.
        // maybe a bug in xsul?
        XmlElement message = null;
        String actionString = null;
        //Measure customization start time 
//        long startTime = IS_FINEST ? System.nanoTime() : 0;
        if (type.compareTo("wsnt") == 0) {
            if (useNotify) {
                message = wrapRawMessageToWsntWrappedFormat(rawNotif,
                        additionalMessageContent);
            } else {
                // message = (XmlElement) notificationMessage.clone();
                message = rawNotif;

            }
            actionString = WsmgConstants.WSN_ActionPrefix + "/Notify";
        } else { // wse
            message = rawNotif;
            actionString = additionalMessageContent.getAction(); // Not
            // action defined in WSE, keep action in the original message

            // add topic to the consumer reference properties, so that it will
            // be shown in the WSE message header.
            String topicElString = additionalMessageContent.getTopicElement();
            if (topicElString != null) {
//            	System.out.println("topicElString="+topicElString);
                XmlElement topicEl = builder
                        .parseFragmentFromReader(new StringReader(topicElString));
                consumerReference.getReferenceProperties().addChild(topicEl);
            }
        }
//      Measure customization End time
//        if (IS_FINEST) {
//            long endTime = System.nanoTime();
//            logger.finest(" " + (endTime - startTime));
//        }

        URI messageId = null;
        String messageIdString = additionalMessageContent.getMessageID();
        if (messageIdString != null) {
            messageId = URI.create(messageIdString);
        }
        invoker.setMessageId(messageId);

        invoker.setDefaultAction(URI.create(actionString));
        invoker.setTargetEPR(consumerReference);
//        System.out.println("consumerReference="+builder.serializeToString(consumerReference));
//        System.out.println("InvokerEPR="+builder.serializeToString(invoker.getTargetEPR()));
//        System.out.println("Before calling Message="+builder.serializeToString(message));
        long timeTried=-1;
        long startTime1=-1;
        try {
            if (consumerInfo.isWsrmEnabled()) {

                wsrmAdaptor.send(message, consumerReference.getAddress()
                        .toString());
            } else {
            	startTime1=System.currentTimeMillis();
            	//It's important to set null here since 
//            	When I call invoker.invokeMessage(message) more than once for a message, xsul will keep adding previous EPR to the SOAP header. I got messages like the one at the end with more than one topic elements (EPR) when sending a message to multiple consumers. 
//            	After debugging, I found the problem happens at public XmlDocument wrapBodyContent(XmlElement bodyContent) in the SOAP12Util.java. Original, my message does not have parent since I created by 
//            	builder.parseFragmentFromReader(new StringReader(messageString)), after calling wrapBodyContent(), xsul adds parent element (Envelope/Body) to my original message. Then the next time I call invoke() method and it calls wrapBodyContent() to find the previously created parent of my message element. Then xsul will explode EPR again to the previous envelope. In this way, the EPRs are accumulated in the message.
            	message.setParent(null);
                XmlElement responseMessage = invoker.invokeMessage(message);
//                System.out.println("After calling Message="+builder.serializeToString(message));
                long finishTime=System.currentTimeMillis();
                timeTried=finishTime-startTime1;
                if(WSMGParameter.showTrackId)
                System.out.println(additionalMessageContent.getTrackId()+": delivered to: "+consumerReference.getAddress().toString()+" in "+timeTried+"ms");

            }
            RunTimeStatistics.addNewSuccessfulDeliverTime(timeTried);
        } catch (DynamicInfosetInvokerException ex) {
            // Catch the RunTime exception. If the broker cannot connect to a
            // client, just ignore it.
            // TODO: Not reliable!!!!

             long finishTime=System.currentTimeMillis();
             timeTried=finishTime-startTime1;
             RunTimeStatistics.addNewFailedDeliverTime(timeTried);
           //  System.out.println((finishTime-startTime)+" failed");
        	System.out.println(additionalMessageContent.getTrackId()+": Failed after trying "+timeTried+" ms to deliver to: "+consumerReference.getAddress().toString());
//            logger.severe("Cannot connect to the client "
//                    + consumerReference.getAddress().toString() + "!!!!"
//                    + ex.getMessage(), ex);
//            System.out.println("Missed Message="
//                    + builder.serializeToString(message));
         //   ex.printStackTrace();

        } catch (AxisFault ex) {
            // Catch the RunTime exception. If the broker cannot connect to a
            // client, just ignore it.
            // TODO: Not reliable!!!!

            // long finishTime=System.currentTimeMillis();
            // System.out.println((finishTime-startTime)+" failed");
            logger.severe("Cannot connect to the client "
                    + consumerReference.getAddress().toString() + "!!!!"
                    + ex.getMessage(), ex);
//            ex.printStackTrace();
            // ex.printStackTrace();
            // System.out.println("Cannot connect to the client
            // "+consumerReference.getAddress().toString()+"!!!!");
            // ex.printStackTrace();
            // System.out.println("Cannot connect to the
            // client"+getConsumerAddress()+" in
            // "+(finishTime-startTime)+"!!!!");

        }
    }

    public XmlElement wrapRawMessageToWsntWrappedFormat(XmlElement rawNotif,
            AdditionalMessageContent additionalMessageContent) {

        XmlElement notificationMessageEl = WsmgConstants.builder.newFragment(
                WsmgConstants.WSNT_NS, "NotificationMessage");
        String topicElString = additionalMessageContent.getTopicElement();
        if (topicElString != null) {
            XmlElement topicEl = builder
                    .parseFragmentFromReader(new StringReader(topicElString));
            notificationMessageEl.addElement(topicEl);
        }
        String producerReferenceElString = additionalMessageContent
                .getProducerReference();
        if (producerReferenceElString != null) {
            XmlElement producerReferenceEl = builder
                    .parseFragmentFromReader(new StringReader(
                            producerReferenceElString));
            notificationMessageEl.addElement(producerReferenceEl);
        }

        notificationMessageEl.addElement(WsmgConstants.WSNT_NS, "Message")
                .addChild(rawNotif);
        XmlElement fullNotif = WsmgConstants.builder.newFragment(
                WsmgConstants.WSNT_NS, "Notify");
        fullNotif.addElement(notificationMessageEl);

        return fullNotif;

    }

    private boolean isMessageBoxAssociatedEPR(String epr) {
        if (epr != null && epr.indexOf("/MsgBox/") != -1) {
            return true;
        } else {
            return false;
        }       
    }

}
