package wsmg;

import java.io.ByteArrayInputStream;
import java.net.URI;
import java.net.URISyntaxException;

import javax.xml.namespace.QName;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.impl.llom.factory.OMXMLBuilderFactory;
import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axiom.soap.SOAPHeaderBlock;
import org.apache.axiom.soap.impl.llom.soap11.SOAP11Factory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.util.StreamWrapper;
import org.apache.axis2.util.UUIDGenerator;
import org.apache.sandesha2.client.SandeshaClientConstants;
import org.apache.xmlbeans.XmlObject;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlNamespace;

import wsmg.stub.NotificationProducerStub;
import wsmg.stub.SubscriptionStub;
import wsmg.util.WsmgUtil;
import wsmg.wsrm.WSRMAdaptor;
import wsmg.wsrm.axis2.DedicatedSimpleSandeshaServer;
import wsmg.wsrm.axis2.SandeshaConfiguration;
import xsul.ws_addressing.WsaEndpointReference;

/**
 * @author Chathura Herath (cherath@cs.indiana.edu)
 */

public class ReliableWSNTClientAPI extends WsntClientAPI implements
        WsmgClientAPI, XMLConstants {

   

    public void shutdownConsumerService() {
        if (wsrmServer != null) {
            wsrmServer.shutDown();
        }

    }

    
    public int publish(String eventSinkLoc, String publisherLoc,
            String topicString, String message) {
        try {
//            SOAPEnvelope env = createMessage(eventSinkLoc, publisherLoc,
//                    topicString, message);

            ServiceClient sender = this.getConfiguredMessageSender(eventSinkLoc);
//            
            SOAP11Factory fac = new SOAP11Factory();
            OMNamespace wsntNamespace = fac.createOMNamespace(
                    ReliableWSNTClientAPI.WEB_SERVICES_WS_BASE_NOTIFICATION,
                    ReliableWSNTClientAPI.XMLNS_WSNT);
            SOAPHeaderBlock topicBlock = fac.createSOAPHeaderBlock("Topic",
                    wsntNamespace);

            topicBlock
                    .addAttribute(fac
                            .createOMAttribute("Dialect", null,
                                    "http://www.ibm.com/xmlns/stdwip/web-services/WS-Topics/TopicExpression/simple"));
            topicBlock.declareNamespace(fac.createOMNamespace(
                    ReliableWSNTClientAPI.HTTP_WIDGETS_COM,
                    ReliableWSNTClientAPI.XMLNS_WIDGET));
            topicBlock.setText(XMLNS_WIDGET + ":" + topicString);
            sender.addHeader(topicBlock);

            
            sender.fireAndForget(this.getPayload(message));
        } catch (Exception e) {
            throw new WsmgRuntimeException(e);
        }
        return 0;
    }
    public int publish(String eventSinkLoc, String topicString, String message) {
        String publisherIP = WsmgUtil.getHostIP();
        this.publish(eventSinkLoc, publisherIP, topicString, message);
        // FIXME @author Chathura Is there a reason why we can't remove all
        // these C++ syntaxed return 0 s.
        return 0;
    }

    public void publish(String eventSinkLoc, String topicString,
            XmlObject message) {
        String publisherIP = WsmgUtil.getHostIP();
        this.publish(eventSinkLoc, publisherIP, topicString, message);
    }

    public void publish(String eventSinkLoc, String publisherLoc,
            String topicString, XmlObject message) {
        try {
            SOAPEnvelope env = createMessage(eventSinkLoc, publisherLoc,
                    topicString, message);
            ServiceClient sender = this.getConfiguredMessageSender(eventSinkLoc);
//            sender.addHeader()
            sender.fireAndForget(env);
        } catch (Exception e) {
            throw new WsmgRuntimeException(e);
        }
    }

    public int startConsumerService(int port, NotificationHandler handler) {
        SandeshaConfiguration sandeshaConfiguration = new SandeshaConfiguration(
                port, handler);
        wsrmServer = new DedicatedSimpleSandeshaServer(sandeshaConfiguration);
        wsrmServer.start();
        wsrmServer.deployClientService();

        return port;
    }

    
    

    public String subscribe(String eventSourceLocIn, String eventSinkLocIn, String subscriptionExpression, int subscriptionType, String eventSinkEndpointReferenceNS, String eventSinkEndpointReference) {
        throw new UnsupportedOperationException("to be implemented if necessary");
    }


 
    public String subscribe(String eventSourceLoc, String eventSinkLoc, String subscriptionExpression, int subscriptionType) {
        throw new UnsupportedOperationException("to be implemented if necessary");
    }



    public String subscribe(String eventSourceLoc, String eventSinkLoc, String topic) {
        throw new UnsupportedOperationException("to be implemented if necessary");
    }


    /**
     * Would subscribe the eventSink as a Reliable consume endpoint. THis
     * particular consumer endpoint should support WSRM. It is recomended that
     * the particular Consumer be started using this API's
     * <code>startConsumerService()</code> method.
     * 
     * @param producerLoc
     * 
     * @param consumerLoc
     *            Epr of the Notification Consumer
     * @param topic
     *            Topic that the consumer is interested in.
     * @param consumerEndpointReferenceNS
     * @param consumerEndpointReference
     * @return SubscriptionID
     * @throws XMLStreamException
     */
    public String subscribe(String producerLoc, String consumerLoc,
            String topic, String consumerEndpointReferenceNS,
            String consumerEndpointReference) {
        producerLoc = WsmgUtil.formatURLString(producerLoc);
        consumerLoc = WsmgUtil.formatURLString(consumerLoc);
        String subscriptionId = null;
        if (consumerLoc == null) {
            throw new IllegalStateException("consumer location required.");
        }
        WsaEndpointReference producerReference = null;
        try {
            producerReference = new WsaEndpointReference(new URI(producerLoc));
        } catch (URISyntaxException e) {
            throw new WsmgRuntimeException("Illegal producer URI", e);
        }

        NotificationProducerStub np = new NotificationProducerStub(
                producerReference);

        WsaEndpointReference consumerReference = null;
        try {
            consumerReference = new WsaEndpointReference(new URI(consumerLoc));
        } catch (URISyntaxException e) {
            throw new WsmgRuntimeException("Illegal consumer URI", e);
        }
        if (consumerEndpointReferenceNS != null) {
            XmlNamespace consumerEPR_NS = builder.newNamespace("ns1",
                    consumerEndpointReferenceNS);
            // consumerReference.getReferenceProperties().addElement(consumerEPR_NS,"resourceId").addChild(consumerEndpointReference);\
            consumerReference.getReferenceProperties().addElement(
                    consumerEPR_NS, "NotificationConsumerKey").addChild(
                    consumerEndpointReference);
        }

        XmlElement topicExpEl = builder.newFragment(WsmgConstants.WSNT_NS,
                "TopicExpression");
        topicExpEl.addAttribute("Dialect",
                WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);

        // FIXME:@author Chathura Get rid of the GT4 specific hardcoding.
        XmlNamespace auctionEPR_NS = builder.newNamespace("ns2",
                "http://tutorial.globus.org/auction");
        topicExpEl.declareNamespace(auctionEPR_NS);
        topicExpEl.addChild(auctionEPR_NS.getPrefix() + ":" + topic);

        SubscriptionStub subscriptionRes = np.subscribe(consumerReference,
                topicExpEl, true, true);

        WsaEndpointReference subscriptionManagerEpr = subscriptionRes
                .getResourceEpr();
        XmlElement referencePropertiesEl = ((XmlElement) subscriptionManagerEpr)
                .element(null, "ReferenceProperties");

        subscriptionId = builder
                .serializeToString((XmlElement) referencePropertiesEl
                        .requiredElementContent().iterator().next());
        // subscriptionId=subscriptionRes.getResourceEpr().getReferenceProperty(WidgetService.WIDGET_NS,
        // WidgetService.SUBSCRIPTION_ID).requiredTextContent();
        // System.out.println(builder.serializeToString((XmlElement)subscriptionManagerEpr));
        return subscriptionId;
    }

    public SOAPEnvelope createMessage(String eventSinkLoc, String publisherLoc,
            String topicString, String message) throws XMLStreamException {
        SOAPEnvelope env = createMessageHeaders(eventSinkLoc, publisherLoc,
                topicString);
        SOAPBody body = env.getBody();
        java.io.ByteArrayInputStream in = new java.io.ByteArrayInputStream(
                message.getBytes());

        StAXOMBuilder builder = new StAXOMBuilder(in);

        OMElement doc = builder.getDocumentElement();
        // doc.build();
        body.setFirstChild(doc);
        return env;
    }

    public SOAPEnvelope createMessage(String eventSinkLoc, String publisherLoc,
            String topicString, XmlObject message) throws XMLStreamException {
        SOAPEnvelope env = createMessageHeaders(eventSinkLoc, publisherLoc,
                topicString);
        SOAPBody body = env.getBody();
        StAXOMBuilder builder = new StAXOMBuilder(OMAbstractFactory
                .getOMFactory(),
                new StreamWrapper(message.newXMLStreamReader()));
        OMElement documentElement = builder.getDocumentElement();
        // /////Building the element is needed to avoid certain stream errors!
        documentElement.build();
        // doc.build();
        body.setFirstChild(documentElement);
        return env;
    }

    public SOAPEnvelope createMessageHeaders(String eventSinkLoc,
            String publisherLoc, String topicString) throws XMLStreamException {

        SOAP11Factory fac = new SOAP11Factory();

        SOAPEnvelope env = fac.getDefaultEnvelope();

        env.declareNamespace(fac.createOMNamespace(
                ReliableWSNTClientAPI.HTTP_WWW_W3_ORG_2001_XMLSCHEMA,
                ReliableWSNTClientAPI.XMLNS_XSD));
        env.declareNamespace(fac.createOMNamespace(
                ReliableWSNTClientAPI.HTTP_WWW_W3_ORG_2001_XMLSCHEMA_INSTANCE,
                ReliableWSNTClientAPI.XMLNS_XSI));
        OMNamespace addressingNamespace = fac
                .createOMNamespace(
                        ReliableWSNTClientAPI.HTTP_SCHEMAS_XMLSOAP_ORG_WS_2004_08_ADDRESSING,
                        ReliableWSNTClientAPI.XMLNS_WA);
        env.declareNamespace(addressingNamespace);
        env
                .declareNamespace(fac
                        .createOMNamespace(
                                ReliableWSNTClientAPI.HTTP_SCHEMAS_XMLSOAP_ORG_WS_2002_12_POLICY,
                                ReliableWSNTClientAPI.XMLNS_WSP));
        env.setNamespace(fac.createOMNamespace(
                ReliableWSNTClientAPI.HTTP_SCHEMAS_XMLSOAP_ORG_SOAP_ENVELOPE,
                ReliableWSNTClientAPI.XMLNS_S));

        SOAPHeader header = env.getHeader();
        SOAPHeaderBlock toBlock = header.addHeaderBlock("To",
                addressingNamespace);
        toBlock.setText(eventSinkLoc);
        OMNamespace wsntNamespace = fac.createOMNamespace(
                ReliableWSNTClientAPI.WEB_SERVICES_WS_BASE_NOTIFICATION,
                ReliableWSNTClientAPI.XMLNS_WSNT);
        SOAPHeaderBlock topicBlock = header.addHeaderBlock("Topic",
                wsntNamespace);

        topicBlock
                .addAttribute(fac
                        .createOMAttribute("Dialect", null,
                                "http://www.ibm.com/xmlns/stdwip/web-services/WS-Topics/TopicExpression/simple"));
        topicBlock.declareNamespace(fac.createOMNamespace(
                ReliableWSNTClientAPI.HTTP_WIDGETS_COM,
                ReliableWSNTClientAPI.XMLNS_WIDGET));
        topicBlock.setText(XMLNS_WIDGET + ":" + topicString);

        SOAPHeaderBlock msgID = header.addHeaderBlock("MessageID",
                addressingNamespace);
        msgID.setText(UUIDGenerator.getUUID());

        SOAPHeaderBlock action = header.addHeaderBlock("Action",
                addressingNamespace);
        action.setText(ReliableWSNTClientAPI.WSNT_PUBLISH_ACTION);

        return env;
    }

    public ServiceClient getConfiguredMessageSender(String eventSinkLoc)
            throws AxisFault {
        ConfigurationContext configContext = ConfigurationContextFactory
                .createConfigurationContextFromFileSystem(
                        WsmgConstants.AXIS2_MAR_LOCATION,
                        WsmgConstants.AXIS2_MAR_LOCATION + "/axis2.xml");
        
        ServiceClient sender = new ServiceClient(configContext, null);
        Options clientOptions = new Options();
        clientOptions.setProperty(Options.COPY_PROPERTIES, new Boolean(true));
        sender.setOverrideOptions(clientOptions);
        clientOptions
                .setSoapVersionURI(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI);
        // clientOptions.setProperty(Sandesha2ClientAPI.AcksTo,acksToEPR);
        // sender.engageModule(new
        // QName(WsmgConstants.SANDESHA_VERSIONED_MODULE_NAME));
        clientOptions.setTo(new EndpointReference(eventSinkLoc));
        clientOptions.setProperty(SandeshaClientConstants.SEQUENCE_KEY,
                "sequence1");
        clientOptions.setAction("publish");
        clientOptions.setProperty(SandeshaClientConstants.LAST_MESSAGE, "true");
        return sender;

    }
    
    
    protected OMElement getPayload(String message){
        OMElement omElement = null;
        XMLStreamReader parser = null;
        try {
            
            
            parser = XMLInputFactory.newInstance().createXMLStreamReader(
                    new ByteArrayInputStream(message.getBytes()));
        } catch (XMLStreamException e) {
            throw new NotificationException(
                    "Cannot create message for sandesha", e);
        } catch (FactoryConfigurationError e) {
            throw new NotificationException(e);
        }
        StAXOMBuilder builder = OMXMLBuilderFactory.createStAXOMBuilder(
                OMAbstractFactory.getOMFactory(), parser);
        omElement = builder.getDocumentElement();
        return omElement;
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

