package wsmg;

import java.net.URI;
import java.net.URISyntaxException;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlNamespace;

import wsmg.stub.WseEventSourceStub;
import wsmg.util.WsmgUtil;
import xsul.ws_addressing.WsAddressing;
import xsul.ws_addressing.WsaEndpointReference;

/**
 * <p>
 * Title: WS-Eventing/Notification
 * </p>
 * 
 * @author yi Huang
 * @version 1.0
 */

public class WseClientAPI extends CommonClientProcessing implements
        WsmgClientAPI {

    protected NotificationProducer notifProducer = new NotificationProducer();

    protected final static String WIDGET_NS_PREFIX = WsmgConstants.WIDGET_NS
            .getPrefix()
            + ":";
    
    protected WsaEndpointReference brokerEpr;

    public WseClientAPI() {
        super();
    }
    
    public WseClientAPI(WsaEndpointReference epr){
        this.brokerEpr = epr;
    }
    
    public void publish(String message){
        if(this.brokerEpr != null){
            this.publish(this.brokerEpr, message);
        }else{
            throw new WsmgRuntimeException("The Endpoint Reference is not set: Use the *WseClientAPI(WsaEndpointReference epr)* constructor if this method is called for publishing");
        }
    }

    public int unSubscribe(String subscriptionManagerLocIn, String subId) {
        XmlElement message = builder.newFragment(WsmgConstants.WSE_NS,
                "Unsubscribe");
        return super.unSubscribe(subscriptionManagerLocIn, subId, message);
    }

    public static WsaEndpointReference createEndpointReference(
            String brokerURL, String topic) {
        if (brokerURL == null) {
            throw new IllegalArgumentException("Broker URL is null.");
        }
        if (topic == null) {
            throw new IllegalArgumentException("Topic is null.");
        }
        
        URI sinkLocation = URI.create(brokerURL + "topic/" +topic);
        WsaEndpointReference eventSinkReference = new WsaEndpointReference(sinkLocation);
        return eventSinkReference;
        
//        eventSinkLoc = WsmgUtil.formatURLString(eventSinkLoc);
//        WsaEndpointReference eventSinkReference = null;
//        try {
//            eventSinkReference = new WsaEndpointReference(
//                    WsAddressing.NS_2004_08, new URI(eventSinkLoc));
//            // System.out.println("########Sink
//            // Reference="+builder.serializeToString(eventSinkReference));
//        } catch (URISyntaxException e) {
//            throw new WsmgRuntimeException("Illegal eventSink URI: "
//                    + eventSinkLoc, e);
//        }
//        if (topic != null) {
//            XmlElement topicExpEl = builder.newFragment(WsmgConstants.WSNT_NS,
//                    "Topic");
//            topicExpEl.addAttribute("Dialect",
//                    WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);
//            topicExpEl.declareNamespace(WsmgConstants.WIDGET_NS);
//            topicExpEl.addChild(WIDGET_NS_PREFIX + topic);
//            eventSinkReference.getReferenceProperties().addChild(topicExpEl);
//        }
//        return eventSinkReference;
    }

    public String subscribe(String eventSourceLocIn, String eventSinkLocIn,
            String topicExpression, String xpathExpression, 
            String eventSinkEndpointReferenceNS,
            String eventSinkEndpointReference) {
        String eventSourceLoc = WsmgUtil.formatURLString(eventSourceLocIn);
        String eventSinkLoc = WsmgUtil.formatURLString(eventSinkLocIn);
        XmlElement filterEl = null;
        if (eventSinkLoc == null) {
            throw new IllegalStateException("eventSink location required.");
        }
        WsaEndpointReference eventSourceReference = null;

        try {
            eventSourceReference = new WsaEndpointReference(
                    WsAddressing.NS_2004_08, new URI(eventSourceLoc));
        } catch (URISyntaxException e) {
            throw new WsmgRuntimeException("Illegal eventSource URI", e);
        };

        WseEventSourceStub eventSourceStub = new WseEventSourceStub(
                eventSourceReference);

        WsaEndpointReference eventSinkReference = null;
        try {
            eventSinkReference = new WsaEndpointReference(
                    WsAddressing.NS_2004_08, new URI(eventSinkLoc));
        } catch (URISyntaxException e) {
            // TODO Auto-generated catch block
            throw new WsmgRuntimeException("Illegal eventSink URI", e);
        }
        if (eventSinkEndpointReferenceNS != null) {
            XmlNamespace eventSinkEPR_NS = builder.newNamespace("ncex",
                    eventSinkEndpointReferenceNS);
            eventSinkReference.getReferenceProperties().addElement(
                    eventSinkEPR_NS, "resourceId").addChild(
                    eventSinkEndpointReference);
        }

        logger
                .info("NotifyTo:"
                        + builder.serializeToString(eventSinkReference));
        boolean hasTopicExpression= (topicExpression!=null && topicExpression.length()!=0);
        boolean hasXPathExpression= (xpathExpression!=null && xpathExpression.length()!=0); 
//        if (topicExpression != null || xpathExpression!=null) {
        if (hasTopicExpression ||hasXPathExpression){
            filterEl = builder.newFragment(WsmgConstants.WSE_NS, "Filter");
//            if (subscriptionType == WsmgConstants.SIMPLE_TOPIC_SUBSCRIPTION) {
            if (!hasXPathExpression) {  //simple topic
                filterEl.addAttribute("Dialect",
                        WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);
                filterEl.declareNamespace(WsmgConstants.WIDGET_NS);
                filterEl.addChild(WsmgConstants.WIDGET_NS.getPrefix() + ":"
                        + topicExpression);
//            } else if (subscriptionType == WsmgConstants.XPATH_SUBSCRIPTION) {
            } else if (!hasTopicExpression) {  //Xpath only
                filterEl.addAttribute("Dialect", WsmgConstants.XPATH_DIALECT);
                filterEl.addChild(xpathExpression);
//            } else if (subscriptionType == WsmgConstants.TOPIC_AND_XPATH_SUBSCRIPTION) {
            } else { //both topic and XPath
                filterEl.addAttribute("Dialect", WsmgConstants.TOPIC_AND_XPATH_DIALECT);
//                if(topicExpression!=null && topicExpression.length()!=0){
                XmlElement topicEl = builder.newFragment(WsmgConstants.WSNT_NS, "TopicExpression");
                topicEl.addAttribute("Dialect",
                        WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);
                topicEl.declareNamespace(WsmgConstants.WIDGET_NS);
                topicEl.addChild(WsmgConstants.WIDGET_NS.getPrefix() + ":"
                        + topicExpression);
                filterEl.addChild(topicEl);
//                }
//                if(xpathExpression!=null && xpathExpression.length()!=0){
                XmlElement xpathEl = builder.newFragment(WsmgConstants.WSNT_NS, "MessageContent");
                xpathEl.addAttribute("Dialect",
                        WsmgConstants.XPATH_DIALECT);
                xpathEl.addChild(xpathExpression);                 
                filterEl.addChild(xpathEl);
//                }
            }
        }
        XmlElement subscriptionManager = eventSourceStub.subscribe(
                eventSinkReference, filterEl, true);
        XmlElement referencePropertiesEl = ((XmlElement) subscriptionManager)
                .element(null, "ReferenceProperties");
        if (referencePropertiesEl == null) {
            referencePropertiesEl = ((XmlElement) subscriptionManager).element(
                    null, "ReferenceParameters");
        }
        // It's not good to assume the subID is in the ReferenceProperty's
        // next/next element. Just save whatever in the referenceProperty as
        // SubID
        // String subscriptionId=
        // ((XmlElement)referencePropertiesEl.requiredElementContent().iterator().next()).requiredTextContent();
        String subscriptionId = builder
                .serializeToString((XmlElement) referencePropertiesEl
                        .requiredElementContent().iterator().next());
        System.out.println("SubscriptionID received=" + subscriptionId);
        return subscriptionId;
    }
    
    public void publish(WsaEndpointReference eventSinkReference, String message) {

        XmlElement messageToNotify = WsmgUtil
        .convertStringMessageToXmlElement(message);
        notifProducer.sendNotification(messageToNotify, "wse",
                eventSinkReference);
    }

    // /**
    // *
    // * @param eventSinkLoc location of eventSink service that is listening to
    // the notification, eg. localhost:19999
    // * @param publisherLoc location of this publisher, e.g. localhost:54321
    // * @param topicString the topic of the message
    // * @param message the content of the message
    // * @return
    // */
    public int publish(String eventSinkLoc, String publisherLoc,
            String topicString, String message) {
        // TODO: currently ignore publisherLoc since it is not in SOAP body as
        // WSNT. Should be able to add to the header wsa:From
        return publish(eventSinkLoc, topicString, message);
    }

    public int publish(String eventSinkLocIn, String topicString, String message) {

        // ysimmhan: we should require the user to provide a URL and not just a
        // host:port.
        // the way you do the check and prepend 'http://' is kludgy. I'm not
        // changing it
        // since it will change the API behavior

        String eventSinkLoc = WsmgUtil.formatURLString(eventSinkLocIn);
        XmlElement messageToNotify = WsmgUtil
                .convertStringMessageToXmlElement(message);
        WsaEndpointReference eventSinkReference = null;
        try {
            eventSinkReference = new WsaEndpointReference(
                    WsAddressing.NS_2004_08, new URI(eventSinkLoc));
            // System.out.println("########Sink
            // Reference="+builder.serializeToString(eventSinkReference));
        } catch (URISyntaxException e) {
            throw new WsmgRuntimeException("Illegal eventSink URI: "
                    + eventSinkLocIn, e);
        }

        if (topicString != null) {
            XmlElement topicExpEl = builder.newFragment(WsmgConstants.WSNT_NS,
                    "Topic");
            topicExpEl.addAttribute("Dialect",
                    WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);
            topicExpEl.declareNamespace(WsmgConstants.WIDGET_NS);
            topicExpEl.addChild(WIDGET_NS_PREFIX + topicString);
            eventSinkReference.getReferenceProperties().addChild(topicExpEl);
        }

        notifProducer.sendNotification(messageToNotify, "wse",
                eventSinkReference);
        return 0;
    }
}
