/**
 * WsmgClientAPI.java
 * @author Yi Huang (yihuan@cs.indiana.edu)
 * Created:Jan 3, 2006 12:03:08 AM
 */

package wsmg;

import java.util.Vector;

import org.xmlpull.v1.builder.XmlElement;

import wsmg.pull.MessagePuller;
import xsul.ws_addressing.WsaEndpointReference;

public interface WsmgClientAPI {
	/**
	 * 
	 * @param carrier
	 *            specify the carrier. narada or openJms or null
	 * @param port
	 *            specify port number of the BrokerService
	 * @return
	 */
	public int startBrokerService(int port, String carrier,
			String carrierLocation, boolean wsrmEnabled, int rmport);
	
	
	

	/**
	 * shutdown Broker service
	 */
	public void shutdownBrokerService();

	/**
	 * 
	 * @param port
	 *            specify the port number of the WS-Eventing/Notification
	 *            eventSink service Use 0 as port number to let xsul
	 *            automatically select a port from available ports pass this
	 *            option to JVM: -Dhttp_transport.server.port.range=START-END to
	 *            specify a range of port to choose
	 */
	public int startConsumerService(int port, NotificationHandler handler);

	/**
	 * shutdown the eventSink service
	 */
	public void shutdownConsumerService();

	/**
	 * 
	 * @param eventSourceLoc
	 *            specify the location that the subscription is going to send
	 *            to, e.g. localhost:19999
	 * @param eventSinkLoc
	 *            specify the location that will get the notification, e.g.
	 *            localhost:12345
	 * @param topic
	 *            specify the topic string
	 * @return
	 */
	public String subscribe(String eventSourceLoc, String eventSinkLoc,
			String topic);

	/**
	 * 
	 * @param eventSourceLoc
	 *            specify the location that the subscription is going to send
	 *            to, e.g. localhost:19999
	 * @param eventSinkLoc
	 *            specify the location that will get the notification, e.g.
	 *            localhost:12345
	 * @param topic
	 *            specify the topic string //Used in WS-Topics filter
	 * @return
	 */
	public String subscribe(String eventSourceLocIn, String eventSinkLocIn,
			String topic, String eventSinkEndpointReferenceNS,
			String eventSinkEndpointReference);
	
	public String subscribe(String eventSourceLoc, String eventSinkLoc,
			String subscriptionExpression, String xpathExpression);
	
	public String subscribe(String eventSourceLocIn, String eventSinkLocIn,
			String subscriptionExpression, String xpathExpression, String eventSinkEndpointReferenceNS,
			String eventSinkEndpointReference);

	/**
	 * 
	 * @param eventSourceLoc
	 *            specify the location that the subscription is going to send
	 *            to, e.g. localhost:19999
	 * @param eventSinkLoc
	 *            specify the location that will get the notification, e.g.
	 *            localhost:12345
	 * @param topic
	 *            specify the topic string
	 * @return
	 */
	public Vector getAllSubscription(String subscriptionManagerLocIn);

	public int unSubscribe(String subscriptionManagerLocIn, String subId,
			XmlElement message);

	/**
	 * 
	 * @param eventSinkLoc
	 *            location of eventSink service that is listening to the
	 *            notification, eg. localhost:19999
	 * @param publisherLoc
	 *            location of this publisher, e.g. localhost:54321
	 * @param topicString
	 *            the topic of the message
	 * @param message
	 *            the content of the message
	 * @return
	 */
	public int publish(String eventSinkLoc, String publisherLoc,
			String topicString, String message);

	// publish without publisherLocation
	public int publish(String eventSinkLocIn, String topicString, String message);

	/*
	 * create a message box for the client to pull message from.
	 * msgBoxServerLoc: the location of the messagebox service return: EPR of
	 * the messagebox
	 */
	public WsaEndpointReference createPullMsgBox(String msgBoxServerLoc);

	public MessagePuller startPullingEventsFromMsgBox(
			WsaEndpointReference msgBoxAddr, NotificationHandler handler,
			long interval);

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2002-2004 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
