package wsmg;

import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlNamespace;

import wsmg.stub.NotificationProducerStub;
import wsmg.stub.SubscriptionStub;
import wsmg.util.WsmgUtil;
import xsul.ws_addressing.WsaEndpointReference;


/**
 * <p>
 * Title: Ws-Notification
 * </p>
 * 
 * @author yi Huang
 * @version 1.0
 */


public class WsntClientAPI extends CommonClientProcessing implements WsmgClientAPI{
  public final static XmlNamespace WIDGET_NS = builder.newNamespace(
      "widget", "http://widgets.com");

  public WsntClientAPI() {
    super();
  }

  /**
 *
 * @param producerLoc specify the location that the subscription is going to send to, e.g. localhost:19999
 * @param consumerLoc specify the location that will get the notification, e.g. localhost:12345
 * @param topic specify the topic string
 * @return
 * @throws URISyntaxException
 */
 	    public String subscribe(String producerLoc, String consumerLoc,
	            String topicExpression, String xpathExpression, 
	            String consumerEndpointReferenceNS,
	            String consumerEndpointReference){
	  String subscriptionId=null;
	  if (consumerLoc == null) {
	    throw new IllegalStateException("consumer location required.");
	  }
	  consumerLoc = WsmgUtil.formatURLString(consumerLoc);
	  producerLoc = WsmgUtil.formatURLString(producerLoc);
	  WsaEndpointReference producerReference = null;
	    try {
			producerReference = new WsaEndpointReference(new URI(
			    producerLoc));
		} catch (URISyntaxException e) {
			throw new WsmgRuntimeException("Illegal producer URI",e);
		} 
	    boolean gt4=true;
	   if(gt4){
	    XmlNamespace producerEPR_NS = builder.newNamespace(
	    	      "ns1", "http://auction.com");
	    	  //    consumerReference.getReferenceProperties().addElement(consumerEPR_NS,"resourceId").addChild(consumerEndpointReference);\
	    producerReference.getReferenceProperties().addElement(producerEPR_NS,"BidKey").addChild("6581243");    
	    }

	  NotificationProducerStub np = new NotificationProducerStub(
	      producerReference);
	  
	  WsaEndpointReference consumerReference = null;
	    try {
			consumerReference = new WsaEndpointReference(new URI(
			    consumerLoc));
		} catch (URISyntaxException e) {
			throw new WsmgRuntimeException("Illegal consumer URI",e);
		}
	    if(consumerEndpointReferenceNS!=null){
	    	XmlNamespace consumerEPR_NS = builder.newNamespace(
	      "ns1", consumerEndpointReferenceNS);
	  //    consumerReference.getReferenceProperties().addElement(consumerEPR_NS,"resourceId").addChild(consumerEndpointReference);\
	      consumerReference.getReferenceProperties().addElement(consumerEPR_NS,"NotificationConsumerKey").addChild(consumerEndpointReference);
	    }

	  //System.out.println("ConsumerReference:"+builder.serializeToString(consumerReference));
		  XmlElement topicExpEl=null;
	    if(topicExpression!=null){  
	   topicExpEl = builder.newFragment(WsmgConstants.WSNT_NS,
	                                              "TopicExpression");
	  topicExpEl.addAttribute("Dialect",
	  		WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);
	 // topicExpEl.declareNamespace(WsmgConstants.WIDGET_NS);
	  XmlNamespace auctionEPR_NS = builder.newNamespace(
	        "ns2", "http://tutorial.globus.org/auction");
	  topicExpEl.declareNamespace(auctionEPR_NS);
	  topicExpEl.addChild(auctionEPR_NS.getPrefix() + ":" + topicExpression);
	  }

	    XmlElement xpathExpEl=null;
		    if(xpathExpression!=null){  
		    	xpathExpEl = builder.newFragment(WsmgConstants.WSNT_NS,
		                                              "Selector");
		    	xpathExpEl.addAttribute("Dialect",
		  		WsmgConstants.XPATH_DIALECT);
		  xpathExpEl.addChild(xpathExpression);
		  }  

	  SubscriptionStub subscriptionRes = np.subscribe(consumerReference,
	      topicExpEl, xpathExpEl,true);

	  WsaEndpointReference subscriptionManagerEpr=subscriptionRes.getResourceEpr();
	  XmlElement referencePropertiesEl=((XmlElement)subscriptionManagerEpr).element(null,"ReferenceProperties");
//	  subscriptionId= ((XmlElement)referencePropertiesEl.requiredElementContent().iterator().next()).requiredTextContent();
	   subscriptionId= builder.serializeToString((XmlElement)referencePropertiesEl.requiredElementContent().iterator().next());  
	  //subscriptionId=subscriptionRes.getResourceEpr().getReferenceProperty(WidgetService.WIDGET_NS, WidgetService.SUBSCRIPTION_ID).requiredTextContent();
	  //System.out.println(builder.serializeToString((XmlElement)subscriptionManagerEpr));
	  return subscriptionId;
	}


	public int unSubscribe(String subscriptionManagerLocIn, String subId)
			throws URISyntaxException {
		XmlElement message = builder.newFragment(WsmgConstants.WSRL_NS,
				"Destroy");
		return super.unSubscribe(subscriptionManagerLocIn, subId, message);
	}

	public int publish(String consumerLocIn, String topicString, String message) {
		String publisherIP = WsmgUtil.getHostIP();
		return publish(consumerLocIn, publisherIP, topicString, message);

	}

	/**
	 * 
	 * @param consumerLoc
	 *            location of consumer service that is listening to the
	 *            notification, eg. localhost:19999
	 * @param publisherLoc
	 *            location of this publisher, e.g. localhost:54321
	 * @param topicString
	 *            the topic of the message
	 * @param message
	 *            the content of the message
	 * @return
	 * @throws URISyntaxException
	 */
	public int publish(String consumerLocIn, String publisherLocIn,
			String topicString, String message) {
		String publisherLoc = WsmgUtil.formatURLString(publisherLocIn);
		String consumerLoc = WsmgUtil.formatURLString(consumerLocIn);
		WsaEndpointReference consumeReference;
		boolean useNotify = true;
		XmlElement topic;
		WsaEndpointReference producerRef;
		/*
		 * final XmlElement messageToNotify = builder.newFragment(WIDGET_NS,
		 * "TestNotification"); messageToNotify.declareNamespace(WIDGET_NS);
		 */
		// System.out.println("Message: "+message);
		// System.out.println("TEST message:"+"<);
		// messageToNotify.addChild(message);
		/*
		 * if(message.trim().startsWith("<")){ //Message is an XML Element
		 * XmlElement messageEl = builder.parseFragmentFromReader(new
		 * StringReader( message)); messageToNotify.addChild(messageEl); }else{
		 * messageToNotify.addChild(message); }
		 */
		// System.out.println("MessageToNotify:
		// "+Util.safeXmlToString(messageToNotify));
		WsaEndpointReference consumerReference = null;
		try {
			consumerReference = new WsaEndpointReference(new URI(consumerLoc));
		} catch (URISyntaxException e) {
			throw new WsmgRuntimeException("Illegal consumer URI", e);
		}

		WsaEndpointReference producerReference = null;
		if (publisherLoc == null) {
			InetAddress localAddress = null;
			try {
				localAddress = InetAddress.getLocalHost();
			} catch (UnknownHostException ex) {
				System.out.println("Error - unable to resolve localhost");
			}
			publisherLoc = localAddress.getHostAddress();
		}
		try {
			producerReference = new WsaEndpointReference(new URI(publisherLoc));
		} catch (URISyntaxException e) {
			throw new WsmgRuntimeException("Illegal publisher URI", e);
		}

		producerReference.setNamespace(WsmgConstants.WSNT_NS);
		producerReference.setName("ProducerReference");
		XmlElement topicExpEl = builder.newFragment(WsmgConstants.WSNT_NS,
				"Topic");
		topicExpEl.addAttribute("Dialect",
				WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);
		XmlNamespace auctionEPR_NS = builder.newNamespace("ns2",
				"http://tutorial.globus.org/auction");
		topicExpEl.declareNamespace(auctionEPR_NS);
		topicExpEl.addChild(auctionEPR_NS.getPrefix() + ":" + topicString);
		// topicExpEl.declareNamespace(WsmgConstants.WIDGET_NS);
		// topicExpEl.addChild(WsmgConstants.WIDGET_NS.getPrefix() + ":" +
		// topicString);

		// Use SubscriptionState class here to save efforts on creating
		// messages. Using NotificationConsumer class seems to be intuitive.
		// SubscriptionState ss = new SubscriptionState(consumerReference,
		// useNotify,
		// topicExpEl, producerReference,"wsnt");
		// ss.sendNotification(message);
		XmlElement messageEl = WsmgUtil
				.convertStringMessageToXmlElement(message);
		XmlElement messageToNotify = null;

		if (useNotify) {
			messageToNotify = builder.newFragment(WsmgConstants.WSNT_NS,
					"Notify");
			XmlElement notificationMesssageEl = messageToNotify.addElement(
					WsmgConstants.WSNT_NS, "NotificationMessage");
			notificationMesssageEl.addChild(producerReference);
			notificationMesssageEl.addChild(topicExpEl);
			notificationMesssageEl.addElement(WsmgConstants.WSNT_NS, "Message")
					.addChild(messageEl);
		} else {
			messageToNotify = messageEl;
		}
		NotificationProducer notifProducer = new NotificationProducer();
		notifProducer.sendNotification(messageToNotify, "wsnt",
				consumerReference);
		return 0;
	}

}
