/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: XmlConsumer.java,v 1.9 2006/05/01 14:52:51 yihuan Exp $
 */

package wsmg;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URLDecoder;

import org.xmlpull.v1.XmlSerializer;
import org.xmlpull.v1.builder.XmlDocument;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;

import xsul.MLogger;
import xsul.XmlConstants;
import xsul.http_server.HttpServerException;
import xsul.http_server.HttpServerRequest;
import xsul.http_server.HttpServerResponse;
import xsul.processor.DynamicInfosetProcessorException;
import xsul.processor.soap_over_http.SoapHttpDynamicInfosetProcessor;
import xsul.soap.SoapUtil;
import xsul.soap11_util.Soap11Util;
import xsul.soap12_util.Soap12Util;
import xsul.util.XsulUtil;
import xsul.ws_addressing.WsaMessageInformationHeaders;

/**
 * Generic message consumer that will print received message ot STDOUT and make
 * them available to WWWW browsers.
 * 
 * @version $Revision: 1.9 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski
 *         </a>
 */
public class XmlConsumer extends SoapHttpDynamicInfosetProcessor {
	private final static MLogger logger = MLogger.getLogger();

	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	private final static String INDENT_PROP = "http://xmlpull.org/v1/doc/properties.html#serializer-indentation";

	private final static String XHTML_NS = null;

	private final static String UTF8 = "UTF-8";

	public final static XmlNamespace WSNT_NS = builder.newNamespace("wsnt",
			"http://www.ibm.com/xmlns/stdwip/web-services/WS-BaseNotification");

	private URI serviceLocation = null;

	// The list will save all the received messages. It causes memory leak.
	// private List list = new ArrayList();

	private static XmlConsumer xc;

	private NotificationHandler handler = null;

	private int listenPort = 0;

	/*
	 * private class NotificatonEntry { NotificatonEntry() { timestamp = new
	 * Date(); }
	 * 
	 * Date timestamp;
	 * 
	 * XmlElement envelope;
	 * 
	 * XmlElement message; }
	 */

	public static String getLocation() {
		return xc != null ? xc.getServer().getLocation() : null;
	}

	public static void shutdownMe() {
		if (xc != null) {
			System.out.println("XML Consumer shutdown requested");
			xc.shutdown();
		}
	}

	public static void main(String[] args) {
		xc = new XmlConsumer(args);
		xc.start();
	}

	public XmlConsumer(int port, NotificationHandler handler) {
		setSupportedSoapFragrances(new SoapUtil[] { Soap12Util.getInstance(),
				Soap11Util.getInstance() });
		setServerPort(port);
		this.handler = handler;
	}

	private XmlConsumer(String[] args) {
		setSupportedSoapFragrances(new SoapUtil[] { Soap12Util.getInstance(),
				Soap11Util.getInstance() });

		if (args.length > 0) {
			setServerPort(Integer.parseInt(args[0]));
		}
	}

	public void start() {
		try {
			super.start();
		} catch (DynamicInfosetProcessorException e) {
			throw new WsmgRuntimeException("Fail to start consumer service", e);
		} catch (IOException e) {
			throw new WsmgRuntimeException("Fail to start consumer service", e);
		}
		listenPort = getServer().getServerPort();
		serviceLocation = URI.create(getServer().getLocation());
		System.out.println("XC started on " + serviceLocation);
	}

	public int getListenPort() {
		return listenPort;
	}

	public String getConsumerLocation() {
		return serviceLocation.toString();
	}

	public XmlDocument processSoapEnvelope(XmlElement envelope,
			SoapUtil soapFragrance) throws DynamicInfosetProcessorException {
		return executeSoapEnvelope(envelope, soapFragrance);
	}

	/**
	 * 
	 */
	public XmlDocument executeSoapEnvelope(XmlElement envelope,
			SoapUtil soapFragrance)
	// throws DynamicInfosetProcessorException
	{
		// extract resource id - if any
		WsaMessageInformationHeaders headers = new WsaMessageInformationHeaders(
				envelope);
		// extract operation
		XmlElement message = soapFragrance.requiredBodyContent(envelope);
		/*
		 * //The list will save all the received messages. It causes memory
		 * leak. NotificatonEntry ne = new NotificatonEntry(); ne.envelope =
		 * envelope; ne.message = message; list.add(ne);
		 */

		// String messageName = message.getName();
		// XmlElement messageContent=message.requiredElement(WSNT_NS,"Message");
		// System.out.println("received:----\n" + XsulUtil.safeXmlToString(envelope)
		// + "---\n"
		// +"message:---\n" +XsulUtil.safeXmlToString(message)+ "---\n"
		// );
		handler.handleNotification(XsulUtil.safeXmlToString(envelope));
		return null;
	}

	public XmlElement processMessage(XmlElement message)
			throws DynamicInfosetProcessorException {
		throw new DynamicInfosetProcessorException(
				"internal error: this method should never be called");
	}

	private void doGet(String path, String query, HttpServerRequest req,
			HttpServerResponse res) {
		final String responseEnc = "UTF-8";
		res.setContentType("text/html; charset=\"" + responseEnc + "\"");
		XmlSerializer xs = null;

		try {
			xs = builder.getFactory().newSerializer();
			xs.setOutput(res.getOutputStream(), responseEnc);
			try {
				xs.setProperty(INDENT_PROP, "  ");
			} catch (IllegalArgumentException e) {
			} catch (IllegalStateException e) {
			}
			xs.startTag(XHTML_NS, "html");
			xs.startTag(XHTML_NS, "head");
			String title = "IU XSUL WSRF XML Consumer ("
					+ WsmgConstants.VERSION + ")";

			xs.startTag(XHTML_NS, "title").text(title)
					.endTag(XHTML_NS, "title");
			xs.endTag(XHTML_NS, "head");
			xs.startTag(XHTML_NS, "body");
			xs.attribute(null, "bgcolor", "white");
			xs.startTag(XHTML_NS, "h1").text(title).endTag(XHTML_NS, "h1");

			xs.startTag(XHTML_NS, "p");
			xs
					.text("This is WSRF/SOAP/HTTP service that acts as XML consumer ");
			xs.endTag(XHTML_NS, "p");

			/*
			 * The list will save all the received messages. It causes memory
			 * leak. int s = list.size(); if (s == 0) { xs.startTag(XHTML_NS,
			 * "p"); xs.text("No messages were received"); xs.endTag(XHTML_NS,
			 * "p"); } else { int first = (s - 10) > 0 ? s - 10 : 0; for (int i =
			 * s - 1; i >= first; --i) { NotificatonEntry ne =
			 * (NotificatonEntry) list.get(i); xs.startTag(XHTML_NS, "p"); //
			 * xs.text("Timestamp:"+ne.timestamp.toGMTString()); xs
			 * .text("Timestamp:" + DateFormat.getDateInstance().format(
			 * ne.timestamp)); xs.startTag(XHTML_NS, "br"); xs.endTag(XHTML_NS,
			 * "br");
			 * 
			 * xs.text("Message:"); xs.startTag(XHTML_NS, "br");
			 * xs.endTag(XHTML_NS, "br");
			 * 
			 * xs.startTag(XHTML_NS, "pre");
			 * xs.text(XsulUtil.safeXmlToString(ne.message)); xs.endTag(XHTML_NS,
			 * "pre");
			 * 
			 * xs.text("Envelope:"); xs.startTag(XHTML_NS, "br");
			 * xs.endTag(XHTML_NS, "br");
			 * 
			 * //xs.startTag(XHTML_NS, "pre");
			 * xs.text(XsulUtil.safeXmlToString(ne.envelope)); //xs.endTag(XHTML_NS,
			 * "pre");
			 * 
			 * xs.startTag(XHTML_NS, "hr"); xs.endTag(XHTML_NS, "hr");
			 * 
			 * xs.endTag(XHTML_NS, "p");
			 *  } }
			 */

			xs.endTag(XHTML_NS, "body");
			xs.endTag(XHTML_NS, "html");

		} catch (IOException e) {
			throw new HttpServerException("could not show content of '" + path
					+ "'", e);
		} catch (org.xmlpull.v1.XmlPullParserException e) {
			throw new HttpServerException("could not show content of '" + path
					+ "'", e);
			// } catch (XmlPullParserException e) {
			// throw new HttpServerException("could not show content of
			// '"+path+"'", e);
		} finally {
			// try {
			if (xs != null) {
				try {
					xs.flush();
				} catch (IOException e) {
				}
			}
			// out.close();
			// } catch (IOException e) {}
		}
	}

	public void service(HttpServerRequest req, HttpServerResponse res)
			throws HttpServerException {
		String method = req.getMethod();
		if ("GET".equals(method) || "POST".equals(method)) {
			String path = req.getPath();
			String query = null;
			final String responseEnc = UTF8;
			if (path == null) {
				path = "";
			} else {
				if (path.startsWith("/")) {
					path = path.substring(1);
				}
				try {
					path = URLDecoder.decode(path, responseEnc);
				} catch (UnsupportedEncodingException e) {
				}
				int qPos = path.indexOf('?');
				if (qPos != -1) {
					query = path.substring(qPos + 1);
					path = path.substring(0, qPos);
				}
				if (path.endsWith("/")) {
					path = path.substring(0, path.length() - 1);
				}
			}

			if ("GET".equals(method)) {
				doGet(path, query, req, res);
			} else if ("POST".equals(method)) {
				// doPost(path, query, req, resp);
				super.service(req, res);

			}
		} else {
			throw new HttpServerException("unsupported method " + method);
		}

	}

}
/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
