/**
 * BitSetTest.java
 * @author Yi Huang (yihuan@cs.indiana.edu)
 * Created:Mar 27, 2006 7:02:05 PM
 */

package wsmg.adapter.XPath.util;

import java.util.BitSet;

//Got code from "The Java Developers Almanac 1.4"

public class BitSetUtil {
    // Returns a bitset containing the values in bytes.
    // The byte-ordering of bytes must be big-endian which means the most significant bit is in element 0.
    public static BitSet fromByteArray(byte[] bytes) {
        BitSet bits = new BitSet();
        for (int i=0; i<bytes.length*8; i++) {
            if ((bytes[bytes.length-i/8-1]&(1<<(i%8))) > 0) {
                bits.set(i);
            }
        }
        return bits;
    }
    
    // Returns a byte array of at least length 1.
    // The most significant bit in the result is guaranteed not to be a 1
    // (since BitSet does not support sign extension).
    // The byte-ordering of the result is big-endian which means the most significant bit is in element 0.
    // The bit at index 0 of the bit set is assumed to be the least significant bit.
    public static byte[] toByteArray(BitSet bits) {
        byte[] bytes = new byte[bits.length()/8+1];
        for (int i=0; i<bits.length(); i++) {
            if (bits.get(i)) {
                bytes[bytes.length-i/8-1] |= 1<<(i%8);
            }
        }
        return bytes;
    }

public static void main(String[] args){
//	 Create the bitset
    BitSet bits = new BitSet();
    
    // Set a bit on
    bits.set(2);                        // 100 = decimal 4
    bits.set(127);   
    long start=System.nanoTime();
    byte[] byteArray=toByteArray(bits);
    long end0=System.nanoTime();
    String base64=org.apache.axis2.util.Base64.encode(byteArray);
    long end1=System.nanoTime();
    byte[] byteArrayNew=org.apache.axis2.util.Base64.decode(base64);
    BitSet bitsNew=fromByteArray(byteArrayNew);
    long end2=System.nanoTime();
    System.out.println("Base64String="+base64);
    System.out.println("bitSet="+bitsNew.toString());
    System.out.println("encodingTime="+(end1-start)+" decodingTime="+(end2-end1)+" encodingTime0="+(end0-start));
    // Retrieving the value of a bit
    boolean b = bits.get(0);            // false
    b = bits.get(2);                    // true
    
    // Clear a bit
    bits.clear(1);
    
    // Setting a range of bits
    BitSet bits2 = new BitSet();
    bits2.set(1, 4);                    // 1110
    bits2.set(127);
    
	long start2=System.nanoTime();
    
    // And'ing two bitsets    
    bits.and(bits2);                    // 0100
	long end=System.nanoTime();
	long total=(end-start2); 
	System.out.println("Total="+total);
    // Xor'ing two bitsets
    bits.xor(bits2);                    // 1010
    
    // Flip all bits in the bitset
    bits.flip(0, bits.length());        // 0101
    
    // Andnot'ing two bitsets
    bits.andNot(bits2);                 // 0001
    
    // Or'ing two bitsets
    bits.or(bits2);                     // 1111
    
//    byte[] byteArray=toByteArray(bits);
//    String base64=org.apache.axis2.util.Base64.encode(byteArray);
//    System.out.println("Base64String="+base64);

	
}
}


/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */
