package wsmg.client;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import org.exolab.jms.util.CommandLine;

import wsmg.WseClientAPI;

/**
 * @author Chathura Herath (cherath@cs.indiana.edu)
 */

public class Publisher {

    private String broker;

    private String topic;

    private String message;

    private long repetitions;

    private long sleepTime;

    public Publisher(String broker, String topic, String message,
            long repetitions, long sleepTime) {

        this.broker = broker;
        this.topic = topic;
        this.message = message;
        this.repetitions = repetitions;
        this.sleepTime = sleepTime;
    }

    public static void main(String[] args) {
        CommandLine cmdline = new CommandLine(args);

        String broker = cmdline.value("b", "tyr11.cs.indiana.edu");
        String topic = cmdline.value("t");
        String message = cmdline.value("m");
        String messageFile = cmdline.value("f");
        long repetitions = Long.parseLong(cmdline.value("r", "1"));
        long sleepTime = Long.parseLong(cmdline.value("d", "1000"));

        if (topic == null ) {

            printUsage();
            System.exit(1);
        }

        Publisher publisher = null;
        if (message != null) {
            publisher = new Publisher(broker, topic, message, repetitions,
                    sleepTime);
        } else if (messageFile != null) {
            InputStream in;
            try {
                in = new FileInputStream(new File(messageFile));

                int byteRead = -1;
                StringBuffer buff = new StringBuffer();
                while (true) {
                    byteRead = in.read();
                    if (byteRead == -1) {
                        break;
                    }
                    buff.append((char) byteRead);
                }
                message = buff.toString();
            } catch (IOException e) {
                System.err.println("Error reading the input file. Stacktrace:");
                e.printStackTrace();
                System.exit(1);
            }
            publisher = new Publisher(broker, topic, message, repetitions,
                    sleepTime);
            
        } else {
            printUsage();
            System.exit(1);
        }
        publisher.publish();

    }

    public void publish() {
        WseClientAPI client = new WseClientAPI();
        for (int i=0; i<this.repetitions; ++i){
            client.publish(this.broker, "localTesthost",this.topic, this.message);
            try {
                Thread.sleep(this.sleepTime);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
    }

    public static void printUsage() {
        System.err
                .println("Usage:\n"
                        + "java Publisher -b <brokerlocation> -t <topic> -m <xml publish message>\n"
                        + "java Publisher -b <brokerlocation> -t <topic> -f <xml message file>\n"
                        + "Advance Usage:\n"
                        + "java Publisher -b <brokerlocation> -t <topic> -m <xml publish message> -r <repetitions> -d <delay between repetitions in miliseconds>\n"
                        + "java Publisher -b <brokerlocation> -t <topic> -f <xml message file> -r <repetitions> -d <delay between repetitions in miliseconds>\n"
                        + "Eg:\n"
                        + "java Publisher -b tyr11.cs.indiana.edu -t testTopic -f c:\\test.xml -r 100 -d 1000\n" +
                                "java Publisher -b tyr11.cs.indiana.edu -t testTopic -m \"<test> test message</test>\" -r 100 -d 1000");

    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

