package wsmg.client;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.net.InetAddress;
import java.net.URI;
import java.net.UnknownHostException;

import org.exolab.jms.util.CommandLine;

import wsmg.NotificationHandler;
import wsmg.WseClientAPI;
import wsmg.pull.MessagePuller;
import xsul.ws_addressing.WsaEndpointReference;

/**
 * @author Chathura Herath (cherath@cs.indiana.edu)
 */

public class Subscriber implements NotificationHandler {

    private String broker;

    private String topic;

    private String consumer;

    private String messageBox;

    private PrintStream out;

    public Subscriber(String broker, String topic, String consumer,
            String messageBox, String file) {

        this.broker = broker;
        this.topic = topic;
        this.consumer = consumer;
        this.messageBox = messageBox;
        if (file != null) {
            try {
                this.out = new PrintStream(new File(file));
            } catch (FileNotFoundException e) {
                this.out = System.out;
            }
        } else {
            this.out = System.out;
        }
    }

    public static void main(String[] args) {
        CommandLine cmdline = new CommandLine(args);

        String broker = cmdline.value("b", "tyr11.cs.indiana.edu");
        String topic = cmdline.value("t");
        String consumer = cmdline.value("c");
        String messageBox = cmdline.value("m");
        String file = cmdline.value("f");
        if (topic == null) {
            printUsage();
            System.exit(1);
        }

        if (consumer != null && (messageBox != null || file != null)) {
            System.err
                    .println("If a third party consumer location is specified usage of messgebox option and file option is not allowed\n");
            printUsage();
            System.exit(1);
        }

        Subscriber subscriber = new Subscriber(broker, topic, consumer,
                messageBox, file);
        subscriber.subscribeAndListen();

    }

    public void subscribeAndListen() {
        WseClientAPI client = new WseClientAPI();
        String subscriptionId = null;
        if (this.messageBox == null) {
            if (this.consumer == null) {
                int port = client.startConsumerService(0, this);

                InetAddress localhost;
                try {
                    localhost = InetAddress.getLocalHost();
                } catch (UnknownHostException e) {
                    throw new RuntimeException(e);
                }
                subscriptionId = client.subscribe(this.broker, localhost
                        .getHostAddress()
                        + ":" + port, this.topic);
            } else {
                subscriptionId = client.subscribe(this.broker, this.consumer,
                        this.topic);
                ;
            }
        } else {
            WsaEndpointReference msgBoxAddr = client
                    .createPullMsgBox(this.messageBox);
            URI address = msgBoxAddr.getAddress();
            String messageBoxAddress = address.toString();

            subscriptionId = client.subscribe(this.broker, messageBoxAddress,
                    topic);
            MessagePuller messagePuller = client.startPullingEventsFromMsgBox(
                    msgBoxAddr, this, 500L);
        }

    }

    public static void printUsage() {
        System.err
                .println("Usage:\n"
                        + "java Subscriber -b <brokerlocation> -t <topic> \n"
                        + "Advance Usage:\n"
                        + "java Subscriber -b <brokerlocation> -t <topic> -m <message box location> -f <file path to write the output>\n"
                        + "java Subscriber -b <brokerlocation> -t <topic> -c <event consumer location>\n"
                        + "Eg:\n"
                        + "java Subscriber -b tyr11.cs.indiana.edu -t testTopic\n"
                        + "java Subscriber -b tyr11.cs.indiana.edu -t testTopic -m http://tyr11.cs.indiana.edu:13333/MsgBox\n"
                        + "java Subscriber -b tyr11.cs.indiana.edu -t testTopic -m http://tyr11.cs.indiana.edu:13333/MsgBox -f c:\\out.txt");

    }

    public void handleNotification(String message) {
        this.out.println(message);

    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

