package wsmg.db;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.BatchUpdateException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import wsmg.config.WSMGParameter;

import edu.indiana.extreme.q.impl.KeyValueWrapper;

/**
 * @author Chathura Herath (cherath@cs.indiana.edu)
 */

public class PersistantDistrebutedQueueDB implements PersistentDistributedQueueDBInterface {

	private static final int FINAL_WAIT_IN_MILI = 5000;

    private JdbcStorage db = new JdbcStorage(WSMGParameter.DB_CONFIG_NAME);

	private static PersistantDistrebutedQueueDB instance = null;

	private static final String TABLE_NAME = "disQ";

	private static final String TABLE_NAME_MAXID = "MaxIDTable";

	private static final String TABLE_NAME_MINID = "MinIDTable";

	// private static final String SQL_RETRIVE_MIN_US

	private static final int STATUS_OPEN = 0;

	// private static final int STATUS_IN_PROGRESS = 1;

	// private static final int STATUS_DONE = 2;

	/*
	 * SQL Queries
	 */
	// private static final String SQL_MAX_ID = "SELECT MAX(id) from "
	// + TABLE_NAME;
	//
	// private static final String SQL_MIN_ID = "SELECT MIN(id) FROM "
	// + TABLE_NAME + " WHERE status=" + STATUS_OPEN;
	//
	// private static final String SQL_UPDATE_FOR_RESERVING_RECODR = "UPDATE "
	// + TABLE_NAME + " SET status=" + STATUS_IN_PROGRESS + " WHERE id =";
	private static final String SQL_INSERT_STATEMENT = "INSERT INTO "
			+ TABLE_NAME + " (id, trackId, message, status) " + "VALUES (?,?,?,"
			+ STATUS_OPEN + ")";

	private static String SQL_DELETE_STATEMENT = "DELETE FROM " + TABLE_NAME
			+ " WHERE id=";

	private static String SQL_SELECT_STATEMENT = "SELECT id,message FROM " + TABLE_NAME
			+ " WHERE id=";

	private static String SQL_MAX_ID_SEPERATE_TABLE = "SELECT maxID FROM "
			+ TABLE_NAME_MAXID;

	private static String SQL_MIN_ID_SEPERATE_TABLE = "SELECT minID FROM "
			+ TABLE_NAME_MINID;

	private static String SQL_MAX_ID_INSERT = "INSERT INTO " + TABLE_NAME_MAXID
			+ " (maxID) VALUES (1)";

	private static String SQL_MIN_ID_INSERT = "INSERT INTO " + TABLE_NAME_MINID
			+ " (minID) VALUES (1)";

	private static String SQL_MAX_ID_INCREMENT = "UPDATE " + TABLE_NAME_MAXID
			+ " SET maxID = maxID+1 WHERE maxID =";

	private static String SQL_MIN_ID_INCREMENT = "UPDATE " + TABLE_NAME_MINID
			+ " SET minID = minID+1 WHERE minID =";

//	public static PersistantDistrebutedQueueDBImpl_Yi getInstance() {
//		if (instance == null) {
//			instance = new PersistantDistrebutedQueueDBImpl_Yi();
//		}
//		return instance;
//	}

	public void save(Object object, String trackId) throws SQLException, IOException {

		// Get the Max ID cache and update and unlock the table

		Connection connection = db.connect();

		PreparedStatement stmt;
		stmt = connection.prepareStatement("lock tables " + TABLE_NAME_MAXID
				+ " write");
		stmt.executeQuery();
		stmt.close();
	//	System.out.println("locked maxId table");

		stmt = connection.prepareStatement(SQL_MAX_ID_SEPERATE_TABLE);
		ResultSet result = stmt.executeQuery();

		int nextkey;

		if (result.next()) {
			nextkey = result.getInt(1);
			result.close();
			stmt.close();
			stmt = connection.prepareStatement(SQL_MAX_ID_INCREMENT
					+ (nextkey ));
			stmt.executeUpdate();
			stmt.close();
		} else {
			nextkey = 1;
			result.close();
			stmt.close();
			stmt = connection.prepareStatement(SQL_MAX_ID_INSERT);
			stmt.executeUpdate();
			stmt.close();
		}
		stmt = connection.prepareStatement("unlock tables");
		stmt.executeQuery();
		stmt.close();
	//	System.out.println("unlocked maxId table");
		stmt = connection.prepareStatement(SQL_INSERT_STATEMENT);

		stmt.setInt(1, nextkey);
		stmt.setString(2, trackId);
		byte[] buffer;
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		ObjectOutputStream out = new ObjectOutputStream(output);
		out.writeObject(object);
		buffer = output.toByteArray();
		ByteArrayInputStream in = new ByteArrayInputStream(buffer);
		stmt.setBinaryStream(3, in, buffer.length);
		db.executeUpdate(stmt);
		db.closeConnection(connection);
	}

	public KeyValueWrapper retrive() throws SQLException, IOException {
		Object obj = null;

		// Get the smallest id
		Connection connection=db.connect(); 
		boolean loop = true;

		int nextkey = -1;
		int maxid = -2;
		PreparedStatement stmt = null;
		ResultSet result = null;
        boolean connectionClosed=false;  
        long wait = 1000;
		while (loop) {
      //      System.out.println("Checking Queue DB");  
			stmt = connection.prepareStatement("lock tables "
					+ TABLE_NAME_MAXID + " write"+ "," +TABLE_NAME_MINID + " write");
			stmt.executeQuery();
			stmt.close();
//			System.out.println("locked maxId and minId table");
        //    System.out.println("looping in retrive");
			
			stmt = connection.prepareStatement(SQL_MIN_ID_SEPERATE_TABLE);
			result = stmt.executeQuery();	

			if (result.next()) {
				nextkey = result.getInt(1);
				result.close();
				stmt.close();				
				
			} else {
				throw new RuntimeException("Queue init has failed earlier");				
			}
			
			stmt = connection.prepareStatement(SQL_MAX_ID_SEPERATE_TABLE);
			result = stmt.executeQuery();

			if (result.next()) {
				maxid = result.getInt(1);
				result.close();
				stmt.close();				
				
			} else {
				throw new RuntimeException("Queue init has failed earlier");				
			}
			
//			stmt = connection.prepareStatement(SQL_MAX_ID_SEPERATE_TABLE);
//			result = stmt.executeQuery();	
//
//			if (result.next()) {
//				nextkey = result.getInt(1);
//				result.close();
//				stmt.close();				
//				
//			} else {
//				throw new RuntimeException("Queue init has failed earlier");				
//			}
			
			stmt = connection.prepareStatement("unlock tables");
			stmt.executeQuery();
			stmt.close();
//			System.out.println("unlocked tables (maxId and minId)1");
			if(maxid > nextkey){
				loop = false;
				stmt = connection.prepareStatement(SQL_MIN_ID_INCREMENT
						+ (nextkey ));
				stmt.executeUpdate();
				stmt.close();
				stmt = connection.prepareStatement("unlock tables");
				stmt.executeQuery();
				stmt.close();		
//				System.out.println("unlocked tables (maxId and minId) 2");
				
			}else{
				try {
					stmt = connection.prepareStatement("unlock tables");
//					System.out.println("unlocked tables (maxId and minId) 3");
					stmt.executeQuery();
					stmt.close();
					db.closeConnection(connection);
					connectionClosed=true;
                    wait = Math.min((wait+1000), FINAL_WAIT_IN_MILI);
                 //   System.out.println("Wait="+wait);
					Thread.sleep(wait);
				} catch (InterruptedException e) {					
					e.printStackTrace();
				}
			}
			if(connectionClosed){
				   connection = db.connect();
				   connectionClosed=false;  
				}			
		}



		stmt = connection.prepareStatement(SQL_SELECT_STATEMENT + nextkey);
		result = stmt.executeQuery();
		//FIXME: THis loop caused out-of-memory
		while(!result.next()){
            //FIXME @author Chathura Remove this while loop and change the order of db update if possible in the save.
            result.close();
            stmt.close();
//            System.out.println("Looping in 1");
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
               
                e.printStackTrace();
            }
			stmt = connection.prepareStatement(SQL_SELECT_STATEMENT + nextkey);
			result = stmt.executeQuery();
			
		}
		int id = result.getInt(1);
		InputStream in = result.getAsciiStream(2);
		ObjectInputStream s = new ObjectInputStream(in);
		try {
			obj = s.readObject();
		} catch (ClassNotFoundException e) {

			e.printStackTrace();
		}
		result.close();
		stmt.close();

		db.closeConnection(connection);

		return new KeyValueWrapper(id, obj);

	}

	public void done(int key) throws SQLException {
		String query = null;
		PreparedStatement stmt = null;
		Connection connection = db.connect();

		query = SQL_DELETE_STATEMENT + key;
//		System.out.println("Deleting key="+key);
		stmt = connection.prepareStatement(query);
		db.executeUpdate(stmt);
		stmt.close();

		db.closeConnection(connection);
	}

	public void init() throws SQLException{
		Connection connection = db.connect();

		PreparedStatement stmt;
		stmt = connection.prepareStatement("lock tables "
				+ TABLE_NAME_MAXID + " write"+ "," +TABLE_NAME_MINID + " write");
		stmt.executeQuery();
		stmt.close();
//		System.out.println("locked tables (maxId and minId)4");
		stmt = connection.prepareStatement(SQL_MAX_ID_SEPERATE_TABLE);
		ResultSet result = stmt.executeQuery();
		if (!result.next()){
			result.close();
			stmt.close();
			stmt = connection.prepareStatement(SQL_MAX_ID_INSERT);
			stmt.executeUpdate();
			stmt.close();
		}	
		
		stmt = connection.prepareStatement(SQL_MIN_ID_SEPERATE_TABLE);
		result = stmt.executeQuery();		

		if (!result.next()) {
			result.close();
			stmt.close();
			stmt = connection.prepareStatement(SQL_MIN_ID_INSERT);
			stmt.executeUpdate();
			stmt.close();
		}
		
		stmt = connection.prepareStatement("unlock tables");
		stmt.executeQuery();
		stmt.close();
		db.closeConnection(connection);
//		System.out.println("unlocked tables (maxId and minId)4");
	}
	
	public void cleanDB() throws SQLException{
		Connection con = db.connect();
		Statement stmt=con.createStatement();
		int[] aiupdateCounts=new int[0];
		boolean bError=false;
		try
		{

		    con.setAutoCommit( false );
		  //  ...
		    bError = false;
		    stmt.clearBatch();
            int totalStatement=0;
		    // add SQL statements
            stmt.addBatch("lock tables disQ write, MaxIDTable write, MinIDTable write;");
//            System.out.println("locked tables (maxId and minId) 5");
		    stmt.addBatch( "Delete from disQ;" );
		    stmt.addBatch( "Delete from MaxIDTable;");
		    stmt.addBatch( "Delete from MinIDTable;" );
		    stmt.addBatch("unlock tables;");
//		    System.out.println("unlocked tables (maxId and minId) 5");
		    totalStatement=5;
		    
		    aiupdateCounts=new int[totalStatement]; 

		    // execute the statements
		    aiupdateCounts = stmt.executeBatch();

		    
		    
		} // end try

//		 catch blocks
	//	...
		 catch( BatchUpdateException bue )
		  {
		    bError = true;
		    aiupdateCounts = bue.getUpdateCounts();
			System.err.println("SQLException: " + bue.getMessage());
			System.err.println("SQLState:  " + bue.getSQLState());
			System.err.println("Message:  " + bue.getMessage());
			System.err.println("Vendor:  " + bue.getErrorCode());
			System.err.print("Update counts:  ");
			int [] updateCounts = bue.getUpdateCounts();
			
			for (int i = 0; i < updateCounts.length; i++) {
				System.err.print(updateCounts[i] + "   ");
			}

		    SQLException SQLe = bue;
		    while( SQLe != null)
		    {
		      // do exception stuff

		      SQLe = SQLe.getNextException();
			  SQLe.printStackTrace(); 
		    }
		  } // end BatchUpdateException catch
		  catch( SQLException SQLe )
		  {
		 //   ...
			  SQLe.printStackTrace();   
		  } // end SQLException catch
		finally
		{
			db.closeConnection(con);
			// determine operation result
		    for (int i = 0; i < aiupdateCounts.length; i++)
		    {
		      int iProcessed = aiupdateCounts[i];
//			    The int values that can be returned in the update counts array are:
//		      		    	-3--Operation error. A driver has the option to stop at the first error and throw a BatchUpdateException or to report the error and continue. This value is only seen in the latter case. 
//		      		    	-2--The operation was successful, but the number of rows affected is unknown. 
//		      		    	Zero--DDL statement or no rows affected by the operation. 
//		      		    	Greater than zero--Operation was successful, number of rows affected by the operation. 		      
		      if( iProcessed > 0 ||
		          iProcessed == -2
		        )
		      {
		        // statement was successful
		     //   ...
		      }
		      else
		      {
		        // error on statement
		    	System.out.println("Batch update successful.");  
		        bError = true;
		        break;
		      }
		    } // end for

		    if( bError ) 
		    { 
		      con.rollback(); 
		    }
		    else 
		    { 
		      con.commit(); 
		    }
		    con.setAutoCommit(true);
		} // end finally
        System.out.println("Queue is cleaned.");
	}

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
