package wsmg.db;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.BatchUpdateException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import wsmg.config.WSMGParameter;

import edu.indiana.extreme.q.impl.KeyValueWrapper;

/**
 * @author Chathura Herath (cherath@cs.indiana.edu)
 */

public class PersistantDistrebutedQueueDB_Yi implements PersistentDistributedQueueDBInterface {

	private static final int FINAL_WAIT_IN_MILI = 5000;

    private JdbcStorage db = new JdbcStorage(WSMGParameter.DB_CONFIG_NAME);

//	private static PersistentDistributedQueueDBInterface instance = null;

	private static final String TABLE_NAME = "disQ";

	private static final String TABLE_NAME_MAXID = "MaxIDTable";

	private static final String TABLE_NAME_MINID = "MinIDTable";

	// private static final String SQL_RETRIVE_MIN_US

	private static final int STATUS_OPEN = 0;
	private static final boolean debug=false;
	// private static final int STATUS_IN_PROGRESS = 1;

	// private static final int STATUS_DONE = 2;

	/*
	 * SQL Queries
	 */
	// private static final String SQL_MAX_ID = "SELECT MAX(id) from "
	// + TABLE_NAME;
	//
	// private static final String SQL_MIN_ID = "SELECT MIN(id) FROM "
	// + TABLE_NAME + " WHERE status=" + STATUS_OPEN;
	//
	// private static final String SQL_UPDATE_FOR_RESERVING_RECODR = "UPDATE "
	// + TABLE_NAME + " SET status=" + STATUS_IN_PROGRESS + " WHERE id =";
	private static final String SQL_INSERT_STATEMENT = "INSERT INTO "
			+ TABLE_NAME + " (trackId, message, status) " + "VALUES (?,?,"
			+ STATUS_OPEN + ")";

	private static String SQL_DELETE_STATEMENT = "DELETE FROM " + TABLE_NAME
			+ " WHERE id=";

	private static String SQL_SELECT_STATEMENT = "SELECT id,message FROM " + TABLE_NAME
			+ " WHERE id=";
	
	private static String SQL_MIN_ID_STATEMENT = "SELECT min(id) FROM " + TABLE_NAME;

	private static String SQL_MAX_ID_SEPERATE_TABLE = "SELECT maxID FROM "
			+ TABLE_NAME_MAXID;

	private static String SQL_MIN_ID_SEPERATE_TABLE = "SELECT minID FROM "
			+ TABLE_NAME_MINID;

	private static String SQL_MAX_ID_INSERT = "INSERT INTO " + TABLE_NAME_MAXID
			+ " (maxID) VALUES (1)";

	private static String SQL_MIN_ID_INSERT = "INSERT INTO " + TABLE_NAME_MINID
			+ " (minID) VALUES (1)";

	private static String SQL_MAX_ID_INCREMENT = "UPDATE " + TABLE_NAME_MAXID
			+ " SET maxID = maxID+1 WHERE maxID =";

	private static String SQL_MIN_ID_INCREMENT = "UPDATE " + TABLE_NAME_MINID
			+ " SET minID = minID+1 WHERE minID =";

//	public static PersistentDistributedQueueDBInterface getInstance() {
//		if (instance == null) {
//			instance = new PersistantDistrebutedQueueDB();
//		}
//		return instance;
//	}

	/* (non-Javadoc)
	 * @see wsmg.db.PersistentDistributedQueueDBInterface#save(java.lang.Object)
	 */
	public void save(Object object, String trackId) throws SQLException, IOException {

		Connection connection = db.connect();

		PreparedStatement stmt;
		if(debug)
       System.out.println("Going to insert record trackID="+trackId);
		stmt = connection.prepareStatement(SQL_INSERT_STATEMENT);
		stmt.setString(1, trackId);
		byte[] buffer;
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		ObjectOutputStream out = new ObjectOutputStream(output);
		out.writeObject(object);
		buffer = output.toByteArray();
		ByteArrayInputStream in = new ByteArrayInputStream(buffer);
		stmt.setBinaryStream(2, in, buffer.length);
		int returnValue=db.executeUpdate(stmt);
		stmt.close();
		if(debug)
		System.out.println("Inserted record trackId="+trackId+" return value="+returnValue);
		db.closeConnection(connection);	
		
	}

	/* (non-Javadoc)
	 * @see wsmg.db.PersistentDistributedQueueDBInterface#retrive()
	 */
	public KeyValueWrapper retrive() throws SQLException, IOException {
		Object obj = null;

		Connection connection= null;
		boolean loop = true;

		PreparedStatement stmt = null;
		ResultSet result = null;
        long wait = 1000;
        if(debug)
        System.out.println("Trying to retrieve message ");
        int minId=-1;

		while (loop) {
			connection=db.connect();
			stmt = connection.prepareStatement("SELECT GET_LOCK('lock1',10)");
			result=stmt.executeQuery();		
			boolean gotLock=false;
			if (result.next()) {
				int locked = result.getInt(1);				
				if(locked==1){
					gotLock=true;
				}
			}
			result.close();
			stmt.close();
			if(!gotLock) {
				db.closeConnection(connection);
				continue;
			}

			minId=-1;

			stmt = connection.prepareStatement("Select min(id) from disQ");			
			result = stmt.executeQuery();	
			if (result.next()) {
				minId = result.getInt(1);		
			} 
			result.close();
			stmt.close();


			if(minId>0){
//				String query = "UPDATE  disQ SET status=1 where id="+minId + " and status=0";
//				System.out.println(query);
//				stmt = connection.prepareStatement(query);
//				//avoid another thread also got the message
//				//Who first update the message gets the message if several threads trying to access
//				int affectedRows = stmt.executeUpdate();
//				//=db.executeUpdate(stmt);
//				stmt.close();	
//				stmt = connection.prepareStatement("SELECT RELEASE_LOCK('lock1');");
//				result=stmt.executeQuery();
//				result.close();
//				stmt.close();
//				if(affectedRows==0){
//					System.out.println("***Cannot update_8. minId="+minId);
//					continue;
//				}else{
//					System.out.println("***Finished update_8. minId="+minId+" row="+affectedRows);
//				}
				
				
				if(debug)
		        System.out.println("Trying to retrieve message of ID="+minId);
				stmt = connection.prepareStatement(SQL_SELECT_STATEMENT + minId);
				result = stmt.executeQuery();
				if(!result.next()){  //Should not happen at all
					System.out.println("*****ERROR: Database Sync: Not got the message");
				}
				InputStream in = result.getAsciiStream(2);
				ObjectInputStream s = new ObjectInputStream(in);
				try {
					obj = s.readObject();
				} catch (ClassNotFoundException e) {

					e.printStackTrace();
				}catch (java.io.IOException e){
					e.printStackTrace();
				}
				result.close();
				stmt.close();
				if(debug)
				System.out.println("Got message of ID="+minId);
				String query = SQL_DELETE_STATEMENT + minId;
				stmt = connection.prepareStatement(query);
				int affectedRows=stmt.executeUpdate();
				if(affectedRows>0){
					if(debug)
				System.out.println("Deleted ID="+minId+" result="+affectedRows);
				loop = false;
				}else{
					if(debug)
					System.out.println("********not deleted_2, try again");
				}
				stmt.close();
				
				
				stmt = connection.prepareStatement("SELECT RELEASE_LOCK('lock1');");
				result=stmt.executeQuery();
				result.close();
				stmt.close();

				db.closeConnection(connection);
		
				
			}else{
				try {
					stmt = connection.prepareStatement("SELECT RELEASE_LOCK('lock1');");
					result=stmt.executeQuery();
					result.close();
					stmt.close();
					db.closeConnection(connection);
                    wait = Math.min((wait+1000), FINAL_WAIT_IN_MILI);
					Thread.sleep(wait);
				} catch (InterruptedException e) {					
					e.printStackTrace();
				}
			}

		} 

		return new KeyValueWrapper(minId, obj);

	}

	/* (non-Javadoc)
	 * @see wsmg.db.PersistentDistributedQueueDBInterface#done(int)
	 */
	public void done(int key) throws SQLException {
//		String query = null;
//		PreparedStatement stmt = null;
//		Connection connection = db.connect();
////		stmt = connection.prepareStatement("select * FROM disQ");
////		ResultSet resultSet = stmt.executeQuery();
////		System.out.println("Resultsize="+resultSet.getRow());
//
//		query = SQL_DELETE_STATEMENT + key;
//		stmt = connection.prepareStatement(query);
//		int result=db.executeUpdate(stmt);
//		System.out.println("Deleted ID="+key+" result="+result);
//		stmt.close();
//
//		db.closeConnection(connection);
	}

	/* (non-Javadoc)
	 * @see wsmg.db.PersistentDistributedQueueDBInterface#init()
	 */
	public void init() throws SQLException{

	}
	
	/* (non-Javadoc)
	 * @see wsmg.db.PersistentDistributedQueueDBInterface#cleanDB()
	 */
	public void cleanDB() throws SQLException{
		Connection con = db.connect();
		Statement stmt=con.createStatement();
		int[] aiupdateCounts=new int[0];
		boolean bError=false;
		try
		{

		    con.setAutoCommit( false );
		  //  ...
		    bError = false;
		    stmt.clearBatch();
            int totalStatement=0;
		    // add SQL statements
     //       stmt.addBatch("lock tables disQ write;");
		    stmt.addBatch( "Delete from disQ;" );
//		    stmt.addBatch( "Delete from MaxIDTable;");
//		    stmt.addBatch( "Delete from MinIDTable;" );
	//	    stmt.addBatch("unlock tables;");
		    totalStatement=1;
		    
		    aiupdateCounts=new int[totalStatement]; 

		    // execute the statements
		    aiupdateCounts = stmt.executeBatch();

		    
		    
		} // end try

//		 catch blocks
	//	...
		 catch( BatchUpdateException bue )
		  {
		    bError = true;
		    aiupdateCounts = bue.getUpdateCounts();
			System.err.println("SQLException: " + bue.getMessage());
			System.err.println("SQLState:  " + bue.getSQLState());
			System.err.println("Message:  " + bue.getMessage());
			System.err.println("Vendor:  " + bue.getErrorCode());
			System.err.print("Update counts:  ");
			int [] updateCounts = bue.getUpdateCounts();
			
			for (int i = 0; i < updateCounts.length; i++) {
				System.err.print(updateCounts[i] + "   ");
			}

		    SQLException SQLe = bue;
		    while( SQLe != null)
		    {
		      // do exception stuff

		      SQLe = SQLe.getNextException();
			  SQLe.printStackTrace(); 
		    }
		  } // end BatchUpdateException catch
		  catch( SQLException SQLe )
		  {
		 //   ...
			  SQLe.printStackTrace();   
		  } // end SQLException catch
		finally
		{
			db.closeConnection(con);
			// determine operation result
		    for (int i = 0; i < aiupdateCounts.length; i++)
		    {
		      int iProcessed = aiupdateCounts[i];
//			    The int values that can be returned in the update counts array are:
//		      		    	-3--Operation error. A driver has the option to stop at the first error and throw a BatchUpdateException or to report the error and continue. This value is only seen in the latter case. 
//		      		    	-2--The operation was successful, but the number of rows affected is unknown. 
//		      		    	Zero--DDL statement or no rows affected by the operation. 
//		      		    	Greater than zero--Operation was successful, number of rows affected by the operation. 		      
		      if( iProcessed > 0 ||
		          iProcessed == -2
		        )
		      {
		        // statement was successful
		     //   ...
		      }
		      else
		      {
		        // error on statement
		    	System.out.println("Batch update successful.");  
		        bError = true;
		        break;
		      }
		    } // end for

		    if( bError ) 
		    { 
		      con.rollback(); 
		    }
		    else 
		    { 
		      con.commit(); 
		    }
		    con.setAutoCommit(true);
		    stmt.close();
		} // end finally
        System.out.println("Queue is cleaned.");
	}

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
