package wsmg.db;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Queue;

import wsmg.config.WSMGParameter;

import edu.indiana.extreme.q.impl.KeyValueWrapper;

public class PersistantQueueDB {
	
	
	private JdbcStorage db = new JdbcStorage(WSMGParameter.DB_CONFIG_NAME);

	
	public static String TABLE_NAME = "PersistantQueue";

	public static String INSERT_STATEMENT = "INSERT INTO " + TABLE_NAME
			+ " (id, trackId, message) " + "VALUES (?,?,?)";

	private static String INITIAL_DELETE_STATEMENT = "DELETE FROM "
			+ TABLE_NAME + " WHERE id= '";

	public static String SQL_MAX_ID = " SELECT MAX(id) from " + TABLE_NAME;

	public static String SQL_MIN_ID = " SELECT MIN(id) from " + TABLE_NAME;

	private static String SQL_RETRIVE_MIN_RECORD = "SELECT * from "
			+ TABLE_NAME + " WHERE id = ";

	private static int nextkey = -1;

	private static int nextToDequeue = -1;

	private static PersistantQueueDB instance = null;

	public static PersistantQueueDB getInstance() throws SQLException {
		if (instance == null) {
			instance = new PersistantQueueDB();
		}
		return instance;
	}

	private PersistantQueueDB() throws SQLException {
		ResultSet result = db.query(SQL_MAX_ID);

		if (result.next()) {
			nextkey = result.getInt(1) + 1;
		} else {
			nextkey = 1;
		}
		result.close();

		result = db.query(SQL_MIN_ID);

		if (result.next()) {
			nextToDequeue = result.getInt(1);
		}
		result.close();

	}

	public void saveAll(List list) throws SQLException, IOException {

		Iterator iterator = list.iterator();
		PreparedStatement stmt = null;
		Connection connection = db.connect();
		while (iterator.hasNext()) {

			KeyValueWrapper wrapper = (KeyValueWrapper) (iterator.next());

			stmt = connection.prepareStatement(INSERT_STATEMENT);
			stmt.setInt(1, wrapper.getKey());
			stmt.setString(2, "BatchInsert");

			byte[] buffer;
			ByteArrayOutputStream output = new ByteArrayOutputStream();
			ObjectOutputStream out = new ObjectOutputStream(output);
			out.writeObject(wrapper.getValue());
			buffer = output.toByteArray();
			ByteArrayInputStream in = new ByteArrayInputStream(buffer);
			stmt.setBinaryStream(3, in, buffer.length);
			db.executeUpdate(stmt);
			stmt.close();

		}
		db.closeConnection(connection);

	}

	public void save(KeyValueWrapper wrapper, String trackId) throws SQLException, IOException {

		Connection connection = db.connect();
		PreparedStatement stmt = connection.prepareStatement(INSERT_STATEMENT);
		stmt.setInt(1, wrapper.getKey());
		stmt.setString(2, trackId);
		byte[] buffer;
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		ObjectOutputStream out = new ObjectOutputStream(output);
		out.writeObject(wrapper.getValue());
		buffer = output.toByteArray();
		ByteArrayInputStream in = new ByteArrayInputStream(buffer);
		stmt.setBinaryStream(3, in, buffer.length);
		db.executeUpdate(stmt);
		stmt.close();
		db.closeConnection(connection);

	}

	/**
	 * Retirves the object from the database but does not remove it from the
	 * database. Always returns the one with the least id.
	 * 
	 * @return key value pair. Null if none exist in the database.
	 * @throws SQLException
	 * @throws IOException
	 */
	public synchronized KeyValueWrapper retrive() throws SQLException,
			IOException {
		if (nextToDequeue != -1) {
			ResultSet result = db.query(SQL_RETRIVE_MIN_RECORD + nextToDequeue);
			Object obj = null;
			if (result.next()) {
				int id = result.getInt(1);
				InputStream in = result.getAsciiStream(2);
				ObjectInputStream s = new ObjectInputStream(in);
				try {
					obj = s.readObject();
				} catch (ClassNotFoundException e) {

					e.printStackTrace();
				}
				result.close();
				return new KeyValueWrapper(id, obj);

			} else {
				result.close();
				return null;
			}
		} else {
			return null;
		}
	}

	/**
	 * Deletes the elements in the Queue from the database. Queue COntains
	 * KeyValueWrapper Objects.
	 * 
	 * @param queue
	 * @throws SQLException
	 */
	public void delete(Queue queue) throws SQLException {

		String query = null;
		PreparedStatement stmt = null;
		Connection connection = db.connect();
		while (!queue.isEmpty()) {
			KeyValueWrapper obj = (KeyValueWrapper) queue.remove();
			query = INITIAL_DELETE_STATEMENT + obj.getKey() + "'";
			stmt = connection.prepareStatement(query);
			db.executeUpdate(stmt);
			stmt.close();

		}
		db.closeConnection(connection);

	}

	/**
	 * Deletes the givent object from the database.
	 * 
	 * @param obj object to be deleted.
	 * @throws SQLException
	 */
	public void delete(KeyValueWrapper obj) throws SQLException {

		String query = null;
		PreparedStatement stmt = null;
		Connection connection = db.connect();

		query = INITIAL_DELETE_STATEMENT + obj.getKey() + "'";
		stmt = connection.prepareStatement(query);
		db.executeUpdate(stmt);
		stmt.close();

		db.closeConnection(connection);

	}

	public synchronized int getNextkey() {
		nextkey++;
		return nextkey - 1;
	}

	public synchronized int getNextToDeQueue() {
		if (nextToDequeue != -1) {
			nextToDequeue++;
			return nextToDequeue - 1;
		}
		return -1;
	}

	public KeyValueWrapper getWrappedInstance(Object obj) {
		return new KeyValueWrapper(this.getNextkey(), obj);
	}

}
