package wsmg.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import wsmg.SubscriptionEntry;
import wsmg.config.WSMGParameter;
import wsmg.util.Counter;
import wsmg.util.TimerThread;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.ws_addressing.WsaEndpointReference;

//import wsmg.ws_addressing.*;

public class QueueDB {
	private final static MLogger logger = MLogger.getLogger();

	JdbcStorage db = new JdbcStorage(WSMGParameter.DB_CONFIG_NAME);

	// Connection conn = null;
	String queueName;

	private Counter storeToDBCounter = new Counter();

	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	private ConnectionPool connectionPool;

	public QueueDB(String queueName) {
		this.queueName = queueName;
		try {
			doDropTables();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (WSMGParameter.measureMessageRate) {
			TimerThread timerThread = new TimerThread(storeToDBCounter,
					" StoreSubScriptionToDBCounter");
			new Thread(timerThread).start();
		}
	}

	public SubscriptionEntry[] getAllMessage() {
		int totalRecord = 0;
		String query = "SELECT * FROM subscription";
		ResultSet rs = null;
		try {
			rs = db.query(query);
			if (rs != null) {
				// Get total number of rows

				// Point to the last row in resultset.
				rs.last();
				// Get the row position which is also the number of rows in the
				// ResultSet.
				totalRecord = rs.getRow();
				System.out.println("TotalRecord=" + totalRecord);
				// Create String array to return

				// Reposition at the beginning of the ResultSet to take up
				// rs.next() call.
				rs.beforeFirst();
				SubscriptionEntry[] subscriptionEntry = new SubscriptionEntry[totalRecord];
				int i = 0;
				while (rs.next()) {
					subscriptionEntry[i] = new SubscriptionEntry();
					subscriptionEntry[i].setSubscriptionId(rs
							.getString("SubscriptionId"));
					subscriptionEntry[i].setSubscribeXml(rs.getString("xml"));
					i++;
				}
				rs.close();
				return subscriptionEntry;
			}
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return null;
	}

	public int insert(String key, String subscribeXml, String topic,
			WsaEndpointReference consumerReference) {
		// XmlElement newsubscribeXmlElement=builder.parseFragmentFromReader(new
		// StringReader(subscribeXml));

		// String topic=
		String query = null;
		PreparedStatement stmt = null;
		String consumerAddress = null;
		consumerAddress = consumerReference.getAddress().toASCIIString();
		XmlElement consumerReferencePropertiesEl = consumerReference.element(
				null, "ReferenceProperties");
		String consumerReferenceProperties;
		if (consumerReferencePropertiesEl == null) {
			consumerReferenceProperties = "";
		} else {
			consumerReferenceProperties = ((XmlElement) (consumerReferencePropertiesEl
					.requiredElementContent().iterator().next()))
					.requiredTextContent();
		}
		query = "INSERT INTO subscription (SubscriptionId, xml, Topics, ConsumerAddress, ReferenceProperties, CreationTime) VALUES ";
		query += "('" + key + "','" + subscribeXml + "','" + topic + "','"
				+ consumerAddress + "','" + consumerReferenceProperties + "','";
		// query+="','"+subEntry.getTopics_Dialect()+"','"+subEntry.isUseNotify()+"','";
		java.util.Date today = new java.util.Date();
		java.sql.Timestamp now = new java.sql.Timestamp(today.getTime());
		query += now + "')";

		try {

			int result = 0;
			long startTime = System.currentTimeMillis();
			long total = 1000000000;

			for (int i = 0; i < total; i++) {
				Connection conn = db.connect();
				stmt = conn.prepareStatement(query);
				// stmt.setString(1, subEntry.getConsumerReference()); //use
				// setString to convert quotation mark automatically

//				logger.finest("Query=" + query);

				result = db.executeUpdate(stmt);

				storeToDBCounter.addCounter();
				// System.out.println(i);

				// }
				// long endTime=System.currentTimeMillis();
				// long duration=endTime-startTime;
				// System.out.println("*******TimeToPutInDB="+duration+" average
				// speed="+(total/(duration/1000)));
				db.closeConnection(conn);
			}
			// conn.close();
			return result;
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return 0;
	}

	public int delete(String subscriptionId) {
		String query;
		query = "DELETE FROM subscription WHERE SubscriptionId='"
				+ subscriptionId + "'";
		try {
			db.update(query);
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
		return 0;
	}

	public void doCreateTables() throws SQLException {
		Statement s = null;
		try {
			logger.info("creating tables");
			Connection conn = db.connect();
			s = conn.createStatement();
			String[] createStatments = getCreateSchemaStatments();
			for (int i = 0; i < createStatments.length; i++) {
				// This will fail usually since the tables will be
				// created already.
				try {
					boolean rc = s.execute(createStatments[i]);
				} catch (SQLException e) {
					logger
							.warning("Could not create JDBC tables; they could already exist."
									+ " Failure was: "
									+ createStatments[i]
									+ " Message: "
									+ e.getMessage()
									+ " SQLState: "
									+ e.getSQLState()
									+ " Vendor code: " + e.getErrorCode());
				}
			}
			// c.getConnection().commit();
			db.closeConnection(conn);
			logger.info("done creating tables");
		} finally {
			try {
				s.close();
			} catch (Throwable e) {
			}
		}
	}

	public void doDropTables() throws SQLException {
		Statement s = null;
		try {
			Connection conn = db.connect();
			s = conn.createStatement();
			String[] dropStatments = getDropSchemaStatments();
			for (int i = 0; i < dropStatments.length; i++) {
				// This will fail usually since the tables will be
				// created already.
				try {
					boolean rc = s.execute(dropStatments[i]);
				} catch (SQLException e) {
					logger
							.warning("Could not drop JDBC tables; they may not exist."
									+ " Failure was: "
									+ dropStatments[i]
									+ " Message: "
									+ e.getMessage()
									+ " SQLState: "
									+ e.getSQLState()
									+ " Vendor code: " + e.getErrorCode());
				}
			}
			db.closeConnection(conn);
			// c.getConnection().commit();
		} finally {
			try {
				s.close();
			} catch (Throwable e) {
			}
		}
	}

	public String[] getCreateSchemaStatments() {
		return new String[] {
		// "CREATE TABLE "+tablePrefix+messageTableName+"("
		// +"ID "+sequenceDataType+" NOT NULL"
		// +", CONTAINER "+containerNameDataType
		// +", MSGID_PROD "+msgIdDataType
		// +", MSGID_SEQ "+sequenceDataType
		// +", EXPIRATION "+longDataType
		// +", MSG "+(useExternalMessageReferences ? stringIdDataType :
		// binaryDataType)
		// +", PRIMARY KEY ( ID ) )",
		// "CREATE INDEX "+tablePrefix+messageTableName+"_MIDX ON
		// "+tablePrefix+messageTableName+" (MSGID_PROD,MSGID_SEQ)",
		// "CREATE INDEX "+tablePrefix+messageTableName+"_CIDX ON
		// "+tablePrefix+messageTableName+" (CONTAINER)",
		// "CREATE TABLE "+tablePrefix+durableSubAcksTableName+"("
		// +"CONTAINER "+containerNameDataType+" NOT NULL"
		// +", CLIENT_ID "+stringIdDataType+" NOT NULL"
		// +", SUB_NAME "+stringIdDataType+" NOT NULL"
		// +", SELECTOR "+stringIdDataType
		// +", LAST_ACKED_ID "+sequenceDataType
		// +", PRIMARY KEY ( CONTAINER, CLIENT_ID, SUB_NAME))",
		};
	}

	public String[] getDropSchemaStatments() {
		return new String[] {
		// "DROP TABLE "+tablePrefix+durableSubAcksTableName+"",
		// "DROP TABLE "+tablePrefix+messageTableName+"",
		};
	}

}
