/**
 * XmlElementUsageDemo.java
 * @author Yi Huang (yihuan@cs.indiana.edu)
 * Created:Nov 15, 2006 4:07:30 PM
 */

package wsmg.demo;

import java.io.StringReader;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;

import xsul.XmlConstants;
import xsul.util.XsulUtil;

public class XmlElementUsageDemo {
	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	private static XmlNamespace default_NS = builder.newNamespace(
        "", "http://www.default.com");
	private static XmlNamespace test_NS = builder.newNamespace(
	        "test", "http://www.test.com");
    
	public static String getADemoXmlMessage(){
		//create an XmlElement
		XmlElement myXmlEl = builder.newFragment(test_NS, "Root"); 
		myXmlEl.declareNamespace(default_NS);
		XmlElement elementA=myXmlEl.addElement("ElementA");
		elementA.addChild("contentStringA");  
		XmlElement elementB =myXmlEl.addElement("ElementB");
		elementB.addChild("contentBString"); 
		elementB.addAttribute("myAttr", "my attribute value");
		myXmlEl.addAttribute("rootAttr", "Root attr value");	
		
        //Convert an XmlElement to a String
		String messageContent =builder.serializeToString(myXmlEl);
		return messageContent;
	}

	public static void parseDemoXmlMessage(String messageContent){
		//parsing the String to Xmlelement
		XmlElement messageEl = builder.parseFragmentFromReader(new StringReader(messageContent));
		
		//get values you need
		System.out.println("parsed result:");
		XmlElement newElementA= messageEl.element(default_NS,"ElementA");
		if(newElementA==null){ //not element named "ElementA" found
			return;
		}
		String contentStringA=newElementA.requiredTextContent();
		System.out.println("contentStringA="+contentStringA);
		XmlElement newElementB =messageEl.element(null, "ElementB");
		String myAttr=newElementB.getAttributeValue(null,"myAttr");
		System.out.println("myAttr="+myAttr);		
	}
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {

		System.out.println("Creating an XML element:");
		String messageContent =getADemoXmlMessage();
		System.out.println(messageContent);
		
		System.out.println("Parsing a String:");
		parseDemoXmlMessage(messageContent);
		
	}

}


/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */
