package wsmg.demo.lead;

import java.io.StringReader;
import java.util.Date;
import java.util.Iterator;

import ncsa.common.NCSAException;
import ncsa.common.util.XmlUtils;
import ncsa.notification.types.NCSAEvent;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import xsul.XmlConstants;

//import java.text.SimpleDateFormat;
//import java.text.*;

/**
 * <p>
 * Title: Ws-Notification
 * </p>
 * 
 * <p>
 * Description:
 * </p>
 * 
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 * 
 * <p>
 * Company: Indiana University
 * </p>
 * 
 * @author yi Huang
 * @version 1.0
 */
public class LeadEvent {
	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	private XmlElement messageEl = null;

	private String status = null;

	private String source = null;

	// private String timeStamp=null;
	private String topic = null;

	private String message = null;

	private long creationTimestamp = 0;

	private String correlationId;

	public LeadEvent() {
	}

	public LeadEvent(String message) {
		messageEl = builder.parseFragmentFromReader(new StringReader(message));
		if (messageEl.getName().compareTo("NCSAEvent") == 0) {
			for (Iterator i = messageEl.elements(null, "properties").iterator(); i
					.hasNext();) {
				XmlElement prop = (XmlElement) i.next();
				String name = prop.requiredElement(null, "name")
						.requiredTextContent();
				// parse the message content
				if (name.compareToIgnoreCase("Status") == 0) {
					status = prop.requiredElement(null, "value")
							.requiredTextContent();
				} else if (name.compareToIgnoreCase("Source") == 0) {
					source = prop.requiredElement(null, "value")
							.requiredTextContent();
					/*
					 * else if(name.compareToIgnoreCase("TimeStamp")==0) {
					 * timeStamp = prop.requiredElement(null,
					 * "value").requiredTextContent(); }
					 */
				} else if (name.compareToIgnoreCase("Topic") == 0) {
					topic = prop.requiredElement(null, "value")
							.requiredTextContent();
				} else if (name.compareToIgnoreCase("CorrelationId") == 0) {
					correlationId = prop.requiredElement(null, "value")
							.requiredTextContent();
				}
			}
			// if (messageEl.element(null, "correlationId") != null) {
			// this.correlationId = messageEl.element(null,
			// "correlationId").requiredTextContent();
			// }
			if (messageEl.element(null, "message") != null) {
				this.message = messageEl.element(null, "message")
						.requiredTextContent();
			}
			String timeStamp = messageEl.getAttributeValue(null,
					"creation-timestamp");
			if (timeStamp != null) {
				creationTimestamp = Long.parseLong(timeStamp);
			}
		}
	}

	public String getMessage() {
		return message;
	}

	public String getSource() {
		return source;
	}

	public String getCorrelationId() {
		return correlationId;
	}

	public String getStatus() {
		return status;
	}

	// Return timeStamp in human-readable string
	public String getTimeStamp() {
		Date date = new Date(creationTimestamp);
		return date.toString();
	}

	// return millisecond since "the epoch", namely January 1, 1970, 00:00:00
	// GMT.
	public long getTimeStampInMillisecond() {
		return creationTimestamp;
	}

	public Date getTimeStampInDate() {
		/*
		 * if (timeStamp==null) return null; //Use the following string to
		 * handle Date.toString result. //I added Locale.US to provent it from
		 * setting Chinese as default language for days and months
		 * SimpleDateFormat format = new SimpleDateFormat("EEE MMM dd HH:mm:ss
		 * zzz yyyy",Locale.US); Date parsed=null; try { parsed =
		 * format.parse(timeStamp); } catch(ParseException pe) {
		 * System.out.println("ERROR: Cannot parse \"" + timeStamp + "\""); }
		 * return parsed;
		 */
		return new Date(creationTimestamp);
	}

	public String getTopic() {
		return topic;
	}

	public String createMessage(String status, String message, String source,
			String topic) {
		String notificationMessage = createMessage(status, message, source,
				topic, "");
		return notificationMessage;
	}

	public String createMessage(String status, String message, String source,
			String topic, String correlationId) {
		this.status = status;
		this.message = message;
		this.source = source;
		this.topic = topic;
		NCSAEvent event = new NCSAEvent();
		event.setMessage(message);
		event.addProperty("Status", status);
		// event.addProperty("Message",message);
		event.addProperty("source", source);
		event.addProperty("Topic", topic);
		event.addProperty("correlationId", correlationId);
		// Change to use timestamp in NCSAEvent
		// Calendar timestamp = new GregorianCalendar();
		// timestamp.setTime(new Date());
		// event.addProperty("TimeStamp",timestamp.getTime().toString());

		// send the notification as an xml string
		// messageBroker.sendNotification(notification.xmlText());
		String notificationMessage = null;
		try {
			notificationMessage = XmlUtils.prettyPrint(XmlUtils
					.marshalWithCastor(event));
		} catch (NCSAException ex) {
			ex.printStackTrace();
		}

		return notificationMessage;
	}

	public void setMessage(String message) {
		this.message = message;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public void setTopic(String topic) {
		this.topic = topic;
	}

	public void setCorrelationId(String correlationId) {
		this.correlationId = correlationId;
	}

}
