/*
 This file is licensed under the terms of the Globus Toolkit Public
 License, found at http://www.globus.org/toolkit/download/license.html.
 */
package wsmg.jms;

import java.io.Serializable;
import java.io.StringReader;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.TextMessage;

import org.activemq.command.ActiveMQObjectMessage;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import xsul.MLogger;
import xsul.XmlConstants;

public class DefaultJMSToWsntConverter implements JMSToWsntConverter {
	// static Log logger =
	// LogFactory.getLog(DefaultJMSToWsntConverter.class.getName());
	private final static MLogger logger = MLogger.getLogger();

	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	// private QName qName = new QName("http://www.ibm.com",
	// "ExtensibilityType");

	/*
	 * public QName wsntDataType() { return qName; }
	 */

	public Object convertToWsnt(Message jmsMessage) throws JMSAdapterException {
		try {

			if (jmsMessage == null) {
				throw new JMSAdapterException(
						"JMSAdapterException null message");

			}
			String xmlElementAsString = null;
			if (jmsMessage instanceof TextMessage) {
				logger.info("Message from JMS:" + jmsMessage);
				xmlElementAsString = ((TextMessage) jmsMessage).getText();
			}
			if (jmsMessage instanceof ObjectMessage) {
				Serializable s = ((ObjectMessage) jmsMessage).getObject();
				if (s instanceof TextMessage) {
					xmlElementAsString = ((TextMessage) s).getText();
				} else {
					logger.info("serializable.toString() = " + s.toString());
					return null; // IGNORE
				}

			}
			if (jmsMessage instanceof ActiveMQObjectMessage) {
				return null; // HORRIBLE
			}
			if (xmlElementAsString != null) {
				XmlElement xmlElement = builder
						.parseFragmentFromReader(new StringReader(
								xmlElementAsString));
				return xmlElement;
			}

			throw new JMSAdapterException("JMSAdapterException "
					+ jmsMessage.getClass() + " '" + jmsMessage + "'");
		} catch (JMSException e) {
			throw new JMSAdapterException("jmsStringError", e);
		}
	}
}
