/**
 * RunTimeStatistics.java
 * @author Yi Huang (yihuan@cs.indiana.edu)
 * Created:Jan 12, 2007 11:12:48 AM
 */

package wsmg.measure.profile;

import java.util.Calendar;
import java.util.Date;

import wsmg.WsmgVersion;
import wsmg.util.WseUtil;

public class RunTimeStatistics {
   public static long totalMessageSize=0;
   public static long totalReceivedNotification=0;
   public static long totalSentOutNotification=0; 
   public static long totalFailedNotification=0; 
   public static long totalSubscriptions=0;
   public static long totalSubscriptionsAtStartUp=0;
   public static long totalUnSubscriptions=0;
   public static long minMessageSize=Long.MAX_VALUE;
   public static long maxMessageSize=0;
   public static String startUpTime="";
   public static long totalSuccessfulDeliveryTime=0;
   public static long totalFailedDeliveryTime=0;
   public static long minSuccessfulDeliveryTime=Long.MAX_VALUE;
   public static long maxSuccessfulDeliveryTime=0;
   public static long minFailedDeliveryTime=Long.MAX_VALUE;
   public static long maxFailedDeliveryTime=0;
	
   public static void addNewNotificationMessageSize(int size){
	   if(size<minMessageSize){
		   minMessageSize=size;
	   }
	   if(size>maxMessageSize){
		   maxMessageSize=size;
	   }
	   totalMessageSize+=size;
	   totalReceivedNotification++;
   }
   
   public static void addNewSuccessfulDeliverTime(long deliveryTime){
	   if(deliveryTime<minSuccessfulDeliveryTime){
		   minSuccessfulDeliveryTime=deliveryTime;
	   }
	   if(deliveryTime>maxSuccessfulDeliveryTime){
		   maxSuccessfulDeliveryTime=deliveryTime;
	   }
	   totalSuccessfulDeliveryTime+=deliveryTime;
	   totalSentOutNotification++;
   }
   
   public static void addNewFailedDeliverTime(long deliveryTime){
	   if(deliveryTime<minFailedDeliveryTime){
		   minFailedDeliveryTime=deliveryTime;
	   }
	   if(deliveryTime>maxFailedDeliveryTime){
		   maxFailedDeliveryTime=deliveryTime;
	   }
	   totalFailedDeliveryTime+=deliveryTime;
	   totalFailedNotification++;
   }
   
   
   public static void setStartUpTime(){
       Date currentDate = new Date(); // Current date
       startUpTime = WseUtil.getXsdDateTime(currentDate);
   }
   
   public static String getHtmlString(){
	   String htmlString="";
	   htmlString+="<p>Total incoming message number: "+ totalReceivedNotification+"<br>\n";
	   htmlString+="Total successful outgoing message number: "+ totalSentOutNotification+"<br>\n";
	   htmlString+="Total unreachable outgoing message number: "+ totalFailedNotification+"<br>\n";
	   htmlString+="Total subscriptions requested: "+ totalSubscriptions +"(+"+totalSubscriptionsAtStartUp+" startUp)<br>\n";
	   htmlString+="Total Unsubscriptions requested: "+ totalUnSubscriptions+"<br>\n";
	   htmlString+="</p>\n";
	   int averageMessageSize=0;
	   if(totalReceivedNotification!=0){
		   averageMessageSize=(int)(totalMessageSize/totalReceivedNotification);
	   }
	   htmlString+="<p>Average message size: "+  averageMessageSize+" bytes<br>\n";
	   htmlString+="Max message size: "+ maxMessageSize+" bytes<br>\n";
	   htmlString+="Min message size: "+ minMessageSize +" bytes<br>\n";
	   htmlString+="</p>\n";
	   long averageSuccessfulDeliveryTime=0;
	   if(totalSuccessfulDeliveryTime!=0){
		   averageSuccessfulDeliveryTime=(long)(totalSuccessfulDeliveryTime/totalSentOutNotification);
	   }
	   htmlString+="<p>Average Successful Delivery Time: "+  averageSuccessfulDeliveryTime+" ms<br>\n";
	   htmlString+="Max Successful Delivery Time: "+ maxSuccessfulDeliveryTime+" ms<br>\n";
	   htmlString+="Min Successful Delivery Time: "+ minSuccessfulDeliveryTime +" ms<br>\n";
	   htmlString+="</p>\n";
	   long averageFailedDeliveryTime=0;
	   if(totalFailedDeliveryTime!=0){
		   averageFailedDeliveryTime=(long)(totalFailedDeliveryTime/totalFailedNotification);
	   }
	   htmlString+="<p>Average Unreachable Delivery Time: "+  averageFailedDeliveryTime+" ms<br>\n";
	   htmlString+="Max Unreachable Delivery Time: "+ maxFailedDeliveryTime+" ms<br>\n";
	   htmlString+="Min Unreachable Delivery Time: "+ minFailedDeliveryTime +" ms<br>\n";
	   htmlString+="</p>\n";
	   htmlString+="<p>Service started at: "+ startUpTime +"<br>\n";
	   htmlString+="Version: "+ WsmgVersion.getImplementationVersion() +"</p>\n";
	   return htmlString;
   }
   
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
//		setStartUpTime();
	}

}


/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (c) 2002-2004 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by the Indiana
 *      University Extreme! Lab.  For further information please visit
 *      http://www.extreme.indiana.edu/"
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "Indiana University" or "Indiana University
 *    Extreme! Lab" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims any
 * liability to any recipient for claims brought by any other entity
 * based on infringement of intellectual property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
 * OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
 * SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
 * DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
 * RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
 * AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
 * SOFTWARE.
 */
