package wsmg.processors;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.ObjectMessage;
import javax.jms.Session;

import org.activemq.ActiveMQConnectionFactory;

import wsmg.NotificationProducerForBroker;
import wsmg.WsmgRuntimeException;
import wsmg.config.WSMGParameter;
import wsmg.util.Counter;
import wsmg.util.TimerThread;

public class DeliveryThread implements Runnable, ExceptionListener {

	private NotificationProducerForBroker notificationProducerForBroker = new NotificationProducerForBroker();

	private MessageConsumer consumer = null;

	private Counter deliveryQueueCounter = new Counter();

	public DeliveryThread() {
		if (WSMGParameter.queueType == WSMGParameter.ACTIVE_MQ_QUEUE) {
			try {
				ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(
						WSMGParameter.OUTGOING_QUEUE_BROKER);
				// Create a Connection
				Connection connection = connectionFactory.createConnection();
				connection.start();

				connection.setExceptionListener(this);

				// Create a Session
				Session session = connection.createSession(false,
						Session.AUTO_ACKNOWLEDGE);

				// Create the destination (Topic or Queue)
				Destination destination = session.createQueue("OUTGOING");

				// Create a MessageConsumer from the Session to the Topic orO
				// Queue
				consumer = session.createConsumer(destination);

			} catch (Exception e) {
				System.out.println("Caught: " + e);
				e.printStackTrace();
			}
		}
		if (WSMGParameter.measureMessageRate) {
			TimerThread timerThread = new TimerThread(deliveryQueueCounter,
					" BrokerDeliveryQueueCounter");
			new Thread(timerThread).start();
		}

	}

	public void run() {

		boolean blocked = true;
		int dequeuedMessageCounter=0;
		while (true) {
			// Wait for a message
			Message message = null;

			// Test code pls ignore from reading.
			if (WSMGParameter.testOutGoingQueueMaxiumLength) {
				if (blocked) { // block for the first time
					try {
						Thread.sleep(1000000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					blocked = false;
				}
			}

			try {
				OutGoingMessage outGoingMessage = null;
				if (WSMGParameter.queueType == WSMGParameter.ACTIVE_MQ_QUEUE) {

					message = consumer.receive(); // indefinitely wait
					if (message instanceof ObjectMessage) {
						ObjectMessage objMessage = (ObjectMessage) message;
						try {
							outGoingMessage = (OutGoingMessage) objMessage
									.getObject();
						} catch (JMSException e) {
							throw new WsmgRuntimeException(
									"Cannot get the outgoing message JMS object",
									e);
						}
					} else {
						System.out
								.println("Received message from outgoing queue is not an object message.");
						return;
					}
				} else if (WSMGParameter.queueType == WSMGParameter.EXTREME_QUEUE
						|| WSMGParameter.queueType == WSMGParameter.EXTREME_DISTRIBUTED_QUEUE) {
				//	System.out.println("\nWaiting to dequeue message #"+dequeuedMessageCounter);
					outGoingMessage = (OutGoingMessage) WSMGParameter.OUT_GOING_QUEUE
							.blockingDequeue();
				//	System.out.println("\nFinished dequeued message #"+dequeuedMessageCounter);
					

				}
				if(WSMGParameter.showTrackId)
				System.out.println(outGoingMessage.getAdditionalMessageContent().getTrackId()+": dequeued from outgoing queue");
				deliveryQueueCounter.addCounter();
				if(!WSMGParameter.performRealDelivery) continue;
				notificationProducerForBroker.sendNotification(outGoingMessage);
			} catch (JMSException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			dequeuedMessageCounter++;
		}
	}

	public synchronized void onException(JMSException ex) {
		System.out.println("JMS Exception occured.  Shutting down client.");
	}
}
