package wsmg.processors;

import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.ObjectMessage;
import javax.jms.Session;

import org.activemq.ActiveMQConnectionFactory;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import wsmg.WsmgAdapter;
import wsmg.WsmgConfig;
import wsmg.config.WSMGParameter;
import wsmg.util.Counter;
import wsmg.util.TimerThread;
import xsul.XmlConstants;

public class PublisherThread implements Runnable, ExceptionListener {
	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	WsmgConfig wsmgConfig = null;

	MessageConsumer consumer = null;

	WsmgAdapter wsmgAdapter = null;

	private Counter arriveQueueCounter = new Counter();

	/**
	 * @param wsmgConfig
	 */
	public PublisherThread(WsmgConfig wsmgConfig) {
		super();
		// TODO Auto-generated constructor stub
		this.wsmgConfig = wsmgConfig;
		wsmgAdapter = wsmgConfig.getWsntAdapter();
		try {

			// Create a ConnectionFactory
			// ActiveMQConnectionFactory connectionFactory = new
			// ActiveMQConnectionFactory("tcp://localhost:61616");
			ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(
					WSMGParameter.INCOMING_QUEUE_BROKER);
			// Create a Connection
			Connection connection = connectionFactory.createConnection();
			connection.start();

			connection.setExceptionListener(this);

			// Create a Session
			Session session = connection.createSession(false,
					Session.AUTO_ACKNOWLEDGE);

			// Create the destination (Topic or Queue)
			Destination destination = session.createQueue("INCOMING");

			// Create a MessageConsumer from the Session to the Topic or Queue
			consumer = session.createConsumer(destination);
		} catch (Exception e) {
			System.out.println("Caught: " + e);
			e.printStackTrace();
		}
		if (WSMGParameter.measureMessageRate) {
			TimerThread timerThread = new TimerThread(arriveQueueCounter,
					" ArriveQueueCounter");
			new Thread(timerThread).start();
		}
	}

	public void run() {
		ReceivedMessage receivedMessage = null;
		String topicLocalString = null;
		AdditionalMessageContent additionalMessageContent = null;
		XmlElement notificationMessageEl = null;
		String wsntMessageConverterClassName = null;

		while (true) {
			// Wait for a message
			Message message = null;
			String notificationMessageStr = null;
			try {
				message = consumer.receive(); // indefinitely wait
				arriveQueueCounter.addCounter();
				if (message instanceof ObjectMessage) {
					ObjectMessage objMessage = (ObjectMessage) message;
					receivedMessage = (ReceivedMessage) objMessage.getObject();

					topicLocalString = receivedMessage.getTopicLocalString();
					additionalMessageContent = receivedMessage.getSoapHeader();
					if(WSMGParameter.showTrackId)
					System.out.println(additionalMessageContent .getTrackId()+": dequeued from incoming queue");
					// notificationMessageEl =
					// builder.parseFragmentFromReader(new StringReader(
					// receivedMessage.getNotificationMessage()));
					notificationMessageStr = receivedMessage
							.getNotificationMessage();

					wsntMessageConverterClassName = receivedMessage
							.getWsntMessageConverterClassName();
				} else {
					System.out
							.println("Received message from incoming queue is not an object message.");
					continue;
				}
			} catch (JMSException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			wsmgAdapter.handleNotify(topicLocalString, additionalMessageContent,
					notificationMessageStr, wsntMessageConverterClassName);

		}
	}

	public synchronized void onException(JMSException ex) {
		System.out.println("JMS Exception occured.  Shutting down client.");
	}
}
