package wsmg.processors.widget;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.TimeZone;

import javax.xml.namespace.QName;

import org.xmlpull.v1.builder.XmlAttribute;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;
import org.xmlpull.v1.builder.adapter.XmlElementAdapter;

import wsmg.SoapClientFault;
import wsmg.WsmgConstants;
import wsmg.util.DcDate;
import wsmg.widgetTest.CurrentDate;
import xsul.XmlConstants;
import xsul.XsulException;

/**
 * <p>
 * Title: Ws-Notification
 * </p>
 * 
 * <p>
 * Description:
 * </p>
 * 
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 * 
 * <p>
 * Company: Indiana University
 * </p>
 * 
 * @author yi Huang
 * @version 1.0
 */
public class WidgetState extends XmlElementAdapter {
	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	// private XmlElement state {
	// private DcDate termninationTime; //TODO XsdDateTime
	// private List fooStrings = new ArrayList(); //JDK15 List<String>
	public WidgetState() {
		super(builder.newFragment(WsmgConstants.WIDGET_NS, "Widget"));
		declareNamespace(WsmgConstants.WIDGET_NS);
		// XmlElement terminationTimeEl = addElement(WSRL_NS, TERMINATION_TIME);
		// terminationTimeEl.addAttribute(XmlConstants.XSI_NS, "nil", "true");
		// //true better?

		// to make IBM web client happy
		Calendar cal = new GregorianCalendar();
		cal.setTimeInMillis(System.currentTimeMillis() + 60 * 60 * 1000);
		cal.setTimeZone(TimeZone.getDefault());
		setTermninationTime(new DcDate(cal));
		XmlElement curTimeEl = addElement(WsmgConstants.WSRL_NS,
				WsmgConstants.CURRENT_TIME);
		curTimeEl.addChild(new CurrentDate(curTimeEl));
	}

	public void deleteElements(QName propertyQName) {
		if (!propertyQName.getNamespaceURI().equals(
				WsmgConstants.WIDGET_NS.getNamespaceName())) {
			throw new SoapClientFault("only properties in "
					+ WsmgConstants.WIDGET_NS.getNamespaceName()
					+ " can be modified not " + propertyQName.getNamespaceURI());
		}
		List elementsToDelete = new ArrayList();
		XmlNamespace ns = builder.newNamespace(propertyQName.getNamespaceURI());
		String n = propertyQName.getLocalPart();
		for (Iterator i = elements(ns, n).iterator(); i.hasNext();) {
			XmlElement childToDelete = (XmlElement) i.next();
			elementsToDelete.add(childToDelete);
		}
		for (Iterator i = elementsToDelete.iterator(); i.hasNext();) {
			XmlElement childToDelete = (XmlElement) i.next();
			removeChild(childToDelete);
		}
	}

	public void insertChildren(List elements) {
		for (Iterator i = elements.iterator(); i.hasNext();) {
			XmlElement childToInsert = (XmlElement) i.next();
			String childNs = childToInsert.getNamespace().getNamespaceName();
			if (!childNs.equals(WsmgConstants.WIDGET_NS.getNamespaceName())) {
				throw new SoapClientFault("only properties in "
						+ WsmgConstants.WIDGET_NS.getNamespaceName()
						+ " can be modified not " + childNs);
			}
			addElement(childToInsert);
		}
	}

	public synchronized List getProperties(QName propertyQName) {
		List list = new ArrayList();
		XmlNamespace ns = builder.newNamespace(propertyQName.getNamespaceURI());
		for (Iterator it = elements(ns, propertyQName.getLocalPart())
				.iterator(); it.hasNext();) {
			XmlElement el = (XmlElement) it.next();
			try {
				list.add(el.clone());
			} catch (CloneNotSupportedException e) {
				throw new XsulException("could not clone property element "
						+ el, e);
			}
		}
		return list;
	}

	public synchronized DcDate getTermninationTime() {
		XmlElement el = element(WsmgConstants.WSRL_NS,
				WsmgConstants.TERMINATION_TIME, true);
		if (el.hasChildren()) {
			DcDate termninationTime = (DcDate) el.children().next();
			return termninationTime;
		} else {
			XmlAttribute xsiNilAttr = el.attribute(XmlConstants.XSI_NS, "nil");
			if (xsiNilAttr == null) {
				el.addAttribute(XmlConstants.XSI_NS, "nil", "true");
			}
			return null;
		}
	}

	public synchronized void setTermninationTime(DcDate dd) {
		XmlElement el = element(WsmgConstants.WSRL_NS,
				WsmgConstants.TERMINATION_TIME, true);
		XmlAttribute xsiNilAttr = el.attribute(XmlConstants.XSI_NS, "nil");
		if (xsiNilAttr != null) {
			el.removeAttribute(xsiNilAttr);
		}
		el.removeAllChildren();
		if (dd == null) {
			el.addAttribute(XmlConstants.XSI_NS, "nil", "true");
		} else {
			el.addChild(dd);
		}
	}
	// synchronized List getFooStrings() {
	// //TODO: clone it?
	// return fooStrings;
	// }
}
