package wsmg.pull;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import wsmg.NotificationHandler;
import wsmg.msg_box.MsgBoxClient;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.ws_addressing.WsaEndpointReference;

/**
 * <p>
 * Title: Ws-Notification
 * </p>
 *
 * <p>
 * Description:
 * </p>
 *
 * <p>
 * Copyright: Copyright (c) 2004
 * </p>
 *
 * <p>
 * Company: Indiana University
 * </p>
 *
 * @author yi Huang
 * @version 1.0
 */
public class MessagePuller {
    final XmlInfosetBuilder builder = XmlConstants.BUILDER;

    private final static MLogger logger = MLogger.getLogger();

    MsgBoxClient msgBoxService = null;

    WsaEndpointReference msgBoxAddr = null;

    NotificationHandler handler = null;

    long interval = 100;

    boolean stopPulling = false;

    public MessagePuller() {
    }

    public MessagePuller(MsgBoxClient msgBoxService,
            WsaEndpointReference msgBoxAddr, NotificationHandler handler,
            long interval) {
        this.msgBoxService = msgBoxService;
        this.msgBoxAddr = msgBoxAddr;
        this.handler = handler;
        this.interval = interval;
    }

    public MessagePuller(MsgBoxClient msgBoxService,
            WsaEndpointReference msgBoxAddr, NotificationHandler handler) {
        this.msgBoxService = msgBoxService;
        this.msgBoxAddr = msgBoxAddr;
        this.handler = handler;
        interval = 1000;
    }

    public void startPulling() {
        Puller puller = new Puller();
        new Thread(puller).start();
    }

    public void stopPulling() {
        stopPulling = true;
    }

    protected class Puller implements Runnable {

        public void run() {

            while (!stopPulling) {
                XmlElement msg[] = msgBoxService.takeMessages(msgBoxAddr);
                try {
                    if(msg == null || msg.length == 0) {
                        // sleep only when nothing was found
                        Thread.currentThread().sleep(interval);
                        logger.fine(".");
                    }
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }

                if (msg.length > 0) {
                    for (int i = 0; i < msg.length; i++) {
                        XmlElement notification = msg[i];
                        handler.handleNotification(builder
                                .serializeToString(notification));
                        logger.info("Message No." + i + "="
                                + builder.serializeToString(notification));
                    }
                }
            }
            return;
        }
    }

}
