/* -*-             c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
// for more information and license info please visit http://storyblog.dev.java.net/
package wsmg.util;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.xmlpull.v1.builder.XmlInfosetBuilder;

/**
 * Tests handling of dc:date.
 * 
 * @version $Revision: 1.2 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class TestDcDate extends TestCase {
	private final static XmlInfosetBuilder builder = XmlInfosetBuilder
			.newInstance();

	public static void main(String[] args) {
		junit.textui.TestRunner.run(new TestSuite(TestDcDate.class));
	}

	public TestDcDate(String name) {
		super(name);
	}

	public void testDcDate() throws Exception {
		DcDate d;

		// d = new DcDate("1997-07-16T19:20:30.45+01:00");

		d = new DcDate(TimeZone.getDefault());

		String dc = d.getDcDate();

		// System.out.println(getClass()+" dc="+dc);

		DcDate d2 = new DcDate(dc);
		Calendar loc = new GregorianCalendar();
		assertEquals(d.getYear(), loc.get(Calendar.YEAR));
		assertEquals(d.getMonth(), loc.get(Calendar.MONTH) + 1);
		assertEquals(d.getDay(), loc.get(Calendar.DAY_OF_MONTH));
		assertEquals(d.getHour(), loc.get(Calendar.HOUR_OF_DAY));
		assertEquals(d.getMinute(), loc.get(Calendar.MINUTE));

		int tzOffset = TimeZone.getDefault().getRawOffset() / (60 * 1000);
		int tzOffsetHours = tzOffset / 60;
		assertEquals(tzOffset, d.getTimeZoneOffset());

		DcDate dZ = new DcDate(TimeZone.getTimeZone("GMT"));

		// System.out.println(getClass()+" dZ="+dZ);

		assertEquals(0, dZ.getTimeZoneOffset());
		// assertEquals(d.getDay() + tzOffset, dZ.getDay()); //may fail!
		int localHour = dZ.getHour() + tzOffsetHours;
		if (localHour > 24) {
			localHour -= 24;
		}
		if (localHour < 0) {
			localHour += 24;
		}
		assertEquals(d.getHour(), localHour);

		//
		d = new DcDate("1997");
		assertEquals(1997, d.getYear());
		//
		d = new DcDate("1997-07");
		assertEquals(1997, d.getYear());
		assertEquals(7, d.getMonth());
		//
		d = new DcDate("1997-07-16");
		assertEquals(1997, d.getYear());
		assertEquals(7, d.getMonth());
		assertEquals(16, d.getDay());

		// 1997-07-16T19:20+01:00
		d = new DcDate("1997-07-16T19:20+01:00");
		assertEquals(1997, d.getYear());
		assertEquals(7, d.getMonth());
		assertEquals(16, d.getDay());
		assertEquals(19, d.getHour());
		assertEquals(20, d.getMinute());
		assertEquals(1 * 60, d.getTimeZoneOffset());
		// 1997-07-16T19:20:30+01:00
		d = new DcDate("1997-07-16T19:20:30+01:00");
		// 1997-07-16T19:20:30.45+01:00
		d = new DcDate("1997-07-16T19:20:30.45+01:00");
		assertEquals("45", d.getDecimalFraction());
		// Ex: 2004-04-13T11:53:15.4362784-04:00
		d = new DcDate("2004-04-13T11:53:15.4362784-04:00");
		assertEquals(13, d.getDay());
		assertEquals("4362784", d.getDecimalFraction());

	}

}
