/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: Client.java,v 1.5 2006/05/01 14:52:54 yihuan Exp $
 */

package wsmg.widgetTest;



import java.net.URI;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.TimeZone;

import javax.xml.namespace.QName;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import wsmg.Server;
import wsmg.WsmgConstants;
import wsmg.XmlConsumer;
import wsmg.stub.BrokerStub;
import wsmg.stub.NotificationProducerStub;
import wsmg.stub.SubscriptionStub;
import wsmg.stub.WidgetStub;
import wsmg.stub.WsrfResourceStub;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.XsulException;
import xsul.util.XsulUtil;
import xsul.ws_addressing.WsAddressing;
import xsul.ws_addressing.WsaEndpointReference;

/**
 * Implementaiton of Widget client for <a
 * href="http://www-106.ibm.com/developerworks/offers/WS-Specworkshops/ws-rf200404.html">WSRF
 * Interop</a>
 * 
 * @version $Revision: 1.5 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class Client {
	private final static MLogger logger = MLogger.getLogger();

	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	private static final int DEFAULT_SCENARIO = 0; // 10; //6; //-8;

	public static final String WELL_KNOWN_QUERY = "boolean(/*/*[namespace-uri()='"
			+ WsmgConstants.WSRL_NS.getNamespaceName()
			+ "' "
			+ "and local-name()='" + WsmgConstants.TERMINATION_TIME + "'])";

	// boolean(/*/*[namespace-uri()=&apos;http://www.ibm.com/xmlns/stdwip/web-services/WS-ResourceLifetime&apos;
	// and local-name()=&apos;TerminationTime&apos;])

	private Client() {
	}

	public static void main(String[] args) throws Exception {
		new Client().run(args);
	}

	public static void verbose(String msg) {
		if (true) {
			System.err.println(msg);
		}
	}

	private void run(String[] args) throws Exception {

		int scenario = DEFAULT_SCENARIO;
		// setTo s12:mustUnderstand="1">
		if (args.length > 0) {
			scenario = Integer.parseInt(args[0]);
		}

		String widgetServiceLocation = Server.getLocation();

		int port = 11534;
		if (widgetServiceLocation == null) {
			if (args.length > 1) {
				String s = args[1];
				if (s.startsWith("http")) {
					widgetServiceLocation = s;
					// } else {
					// port = Integer.parseInt(s);
				}
			}
		}
		System.out.println("***WidgetServiceLocation:" + widgetServiceLocation);
		if (scenario != 0) {
			runScenario(scenario, widgetServiceLocation);
		} else {
			System.err.println("running all scenarios");
			for (int i = 1; i <= 8; i++) {
				runScenario(i, widgetServiceLocation);
			}
			runScenario(-8, widgetServiceLocation);
			runScenario(-9, widgetServiceLocation);
			for (int i = 8; i <= 10; i++) {
				runScenario(i, widgetServiceLocation);
			}
		}
		System.err.println("Client finished OK");

	}

	private void runScenario(int scenario, String widgetServiceLocation)
			throws Exception {
		// if(widgetServiceLocation == null) {
		// widgetServiceLocation = "http://localhost:"+port;
		// }
		System.err.println("scenario " + scenario
				+ " with widget service located at " + widgetServiceLocation);

		WidgetStub ws = null;
		if (scenario < 8) {
			if (widgetServiceLocation != null) {
				System.err.println("scenario " + scenario + " connecting to "
						+ widgetServiceLocation);
				WidgetCreator widgetService = new WidgetCreator(new URI(
						widgetServiceLocation));
				ws = widgetService.createWidget();
			} else {
				if (scenario < 8) {
					throw new XsulException(
							"widget location is required for scenarios < 8");
				}
				System.err.println("no widget service location specified");
			}
		}

		QName terminationTimeQ = new QName(WsmgConstants.WSRL_NS
				.getNamespaceName(), "TerminationTime", WsmgConstants.WSRL_NS
				.getPrefix());
		QName currentTimeQ = new QName(
				WsmgConstants.WSRL_NS.getNamespaceName(), "CurrentTime",
				WsmgConstants.WSRL_NS.getPrefix());
		QName widgetFooQ = new QName(
				WsmgConstants.WIDGET_NS.getNamespaceName(), "foo",
				WsmgConstants.WIDGET_NS.getPrefix());
		if (scenario != 6 && (scenario >= 3 && scenario < 8)) {
			Calendar cal = new GregorianCalendar();
			cal.setTimeInMillis(System.currentTimeMillis() + 6 * 60 * 1000);
			cal.setTimeZone(TimeZone.getDefault()); // TODO is it needed?
			ws.setTerminationTime(cal);
		}

		if (scenario == -8) {
			// checkTTIsNull(ws, terminationTimeQ);

			Calendar cal = new GregorianCalendar();
			cal.setTimeInMillis(System.currentTimeMillis() + 6 * 60 * 1000);
			cal.setTimeZone(TimeZone.getDefault()); // TODO is it needed?
			ws.setTerminationTime(cal);

			ws.setTerminationTime(null);
			checkTTIsNull(ws, terminationTimeQ);
		}

		if (scenario == 4 || scenario == 5) {
			List list = ws.getResourceProperty(terminationTimeQ);
			verbose("getResourceProperty = " + XsulUtil.safeXmlToString(list));
		}

		if (scenario == 5) {
			List list = ws.getMultipleResourceProperties(new QName[] {
					terminationTimeQ, currentTimeQ });
			verbose("getMultipleResourceProperties = "
					+ XsulUtil.safeXmlToString(list));

		}

		if (scenario == 6) {
			List results = ws.queryResourcePropertiesByXpath(WELL_KNOWN_QUERY);
			verbose("queryResourcePropertiesByXpath = "
					+ XsulUtil.safeXmlToString(results));
			results = ws.queryResourcePropertiesByXpath("/*");
			verbose("queryResourcePropertiesByXpath = "
					+ XsulUtil.safeXmlToString(results));
		}

		if (scenario == 7) {
			List list = ws.getResourceProperty(widgetFooQ);
			assert list.size() == 0;

			XmlElement insertRequest = builder.newFragment(
					WsmgConstants.WSRP_NS, "Insert");
			XmlElement s1 = insertRequest.addElement(WsmgConstants.WIDGET_NS,
					"foo");
			s1.addChild("string1");
			ws.setResourceProperties(new XmlElement[] { insertRequest });
			// verbose("setResourceProperties = "+XsulUtil.safeXmlToString(e));

			list = ws.getResourceProperty(widgetFooQ);
			assert list.size() == 1;
			checkFooEl((XmlElement) list.get(0), "string1");

			XmlElement updateRequest = builder.newFragment(
					WsmgConstants.WSRP_NS, "Update");
			XmlElement s2 = updateRequest.addElement(WsmgConstants.WIDGET_NS,
					"foo");
			s2.addChild("string2");
			XmlElement s3 = updateRequest.addElement(WsmgConstants.WIDGET_NS,
					"foo");
			s3.addChild("string3");
			ws.setResourceProperties(new XmlElement[] { updateRequest });

			list = ws.getResourceProperty(widgetFooQ);
			assert list.size() == 2;
			checkFooEl((XmlElement) list.get(0), "string2");
			checkFooEl((XmlElement) list.get(1), "string3");

			// TODO refactor into WsrpDeleteRequest(WIDGET_NS, "foo");
			XmlElement deleteRequest = builder.newFragment(
					WsmgConstants.WSRP_NS, "Delete");
			deleteRequest.declareNamespace(WsmgConstants.WIDGET_NS);
			deleteRequest.addAttribute("resourceProperty",
					WsmgConstants.WIDGET_NS.getPrefix() + ":foo");
			ws.setResourceProperties(new XmlElement[] { deleteRequest });

			list = ws.getResourceProperty(widgetFooQ);
			assert list.size() == 0;
		}

		if (scenario == 8 || scenario == 9) {
			String consumerLoc = System.getProperty("wsrf.consumer");
			if (consumerLoc == null) {
				consumerLoc = XmlConsumer.getLocation();
			}
			if (consumerLoc == null) {
				throw new IllegalStateException(
						"consumer location required to run scenario >= 8");
			}

			String producerLoc = System.getProperty("wsrf.producer");
			WsaEndpointReference producerReference = null;
			NotificationProducerStub np = null;
			if (producerLoc == null && ws != null) {
				producerReference = ws.getResourceEpr();
				producerLoc = producerReference.getAddress().toString();
				np = new NotificationProducerStub(producerReference);
				System.err.println("using WidgetService Instance as producer "
						+ producerReference);
			}
			if (producerReference == null && producerLoc == null) {
				producerLoc = widgetServiceLocation; // use widget service as
														// default producer
				System.err.println("using WidgetService Instance as producer "
						+ producerLoc);
			}
			if (producerLoc == null) {
				throw new IllegalStateException(
						"producer location required to run scenario >= 8");
			}
			if (producerReference == null) {
				producerReference = new WsaEndpointReference(new URI(
						producerLoc));
				np = new NotificationProducerStub(producerReference);
				System.err.println("using (Broker) producer " + producerLoc);
			}

			// if(scenario == 10) {
			// BrokerStub br = new BrokerStub(producerReference);
			// WsrfResourceStub publisherRegistrationRes =
			// br.registerPublisher(new WsaEndpointReference(new
			// URI(widgetServiceLocation)));
			// }

			WsaEndpointReference consumerReference = new WsaEndpointReference(
					new URI(consumerLoc));
			XmlElement topicExpEl = builder.newFragment(WsmgConstants.WSNT_NS,
					"TopicExpression");
			topicExpEl.addAttribute("dialect",
					WsmgConstants.TOPIC_EXPRESSION_SIMPLE_DIALECT);
			topicExpEl.declareNamespace(WsmgConstants.WIDGET_NS);
			topicExpEl.addChild(WsmgConstants.WIDGET_NS.getPrefix()
					+ ":TestTopic");
			SubscriptionStub subscriptionRes = np.subscribe(consumerReference,
					topicExpEl, true); // boolean value is used to set
										// UseNotify

			XmlElement topicEl = (XmlElement) topicExpEl.clone();
			topicEl.setName("Topic");

			generateN(ws, np);
			XmlElement curMsg = np.getCurrentMessage(topicEl);
			verbose("getCurrentMessage=" + XsulUtil.safeXmlToString(curMsg)
					+ "\n\n\n");

			if (scenario == 9) {

				generateN(ws, np);
				curMsg = np.getCurrentMessage(topicEl);
				verbose("getCurrentMessage=" + XsulUtil.safeXmlToString(curMsg)
						+ "\n\n\n");

				subscriptionRes.pauseSubscription();
				generateN(ws, np);
				curMsg = np.getCurrentMessage(topicEl);
				verbose("getCurrentMessage=" + XsulUtil.safeXmlToString(curMsg)
						+ "\n\n\n");

				subscriptionRes.resumeSubscription();
				generateN(ws, np);
				curMsg = np.getCurrentMessage(topicEl);
				verbose("getCurrentMessage=" + XsulUtil.safeXmlToString(curMsg)
						+ "\n\n\n");
			}

			subscriptionRes.destroy();
		}

		if (scenario == 10) {
			String brokerLoc = System.getProperty("wsrf.broker");
			if (brokerLoc == null) {
				String producerLoc = System.getProperty("wsrf.producer");
				if (producerLoc != null) {
					brokerLoc = producerLoc;
					System.err.println("using WS-NT producer as broker "
							+ producerLoc);
				}
			}
			if (brokerLoc == null) {
				throw new RuntimeException(
						"WS-NT broker or producer location is required for scenario "
								+ scenario);
			}
			WsaEndpointReference brokerReference = new WsaEndpointReference(
					new URI(brokerLoc));
			BrokerStub br = new BrokerStub(brokerReference);
			WsrfResourceStub publisherRegistrationRes = br
					.registerPublisher(new WsaEndpointReference(new URI(
							widgetServiceLocation)));
		}

		if (scenario == 2) {
			ws.destroy();
		}
		System.err.println("scenario " + scenario + " finished OK");

	}

	private void checkTTIsNull(WidgetStub ws, QName terminationTimeQ) {
		List list = ws.getResourceProperty(terminationTimeQ);
		if (list.size() == 0) {
			return; // ALEK: some impls will have no termination time intiailly
					// ...
		}
		verbose("getResourceProperty = " + XsulUtil.safeXmlToString(list));
		assert list.size() == 1;
		XmlElement ttEl = (XmlElement) list.get(0);
		assert terminationTimeQ.getLocalPart().equals(ttEl.getName());
		assert terminationTimeQ.getNamespaceURI().equals(
				ttEl.getNamespace().getNamespaceName());
		String xsiNil = ttEl.getAttributeValue(XmlConstants.XSI_NS
				.getNamespaceName(), "nil");
		assert xsiNil != null;
		assert xsiNil.equals("1") || xsiNil.equals("true");
	}

	private void generateN(WidgetStub ws, NotificationProducerStub np) {
		if (ws == null) { // special case when broker is tested as notficaiton
							// producer
			System.out.println("Ws is null");
			np.generateNotification(); // "notification from IU XSUL");
		} else {
			System.out.println("Ws is NOT null");
			ws.generateNotification(); // "notification from IU XSUL");
		}
	}

	private void checkFooEl(XmlElement e, String s) {
		assert "foo".equals(e.getName());
		assert WsmgConstants.WIDGET_NS.equals(e.getNamespace());
		assert e.hasChildren();
		assert s.equals(e.requiredTextContent());
	}

	/*
	 * public static class SubscriptionStub extends WsrfResourceStub {
	 * 
	 * SubscriptionStub(WsaEndpointReference widgetEpr) { super(widgetEpr); }
	 * 
	 * public void pauseSubscription() { invoker.setMessageId(null); XmlElement
	 * message = builder.newFragment(WsmgConstants.WSNT_NS,
	 * "PauseSubscription"); //XmlElement message =
	 * builder.newFragment(WidgetService.WSNT_NS, "PauseSubcription"); //NOTE
	 * TYPO!!! invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName())); XmlElement
	 * responseMessage = invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage.getRoot()) + "\n\n\n"); }
	 * 
	 * public void resumeSubscription() { invoker.setMessageId(null); XmlElement
	 * message = builder.newFragment(WsmgConstants.WSNT_NS,
	 * "ResumeSubscription");
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName())); XmlElement
	 * responseMessage = invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage.getRoot()) + "\n\n\n"); } }
	 * 
	 * public static class BrokerStub extends WidgetStub {
	 * 
	 * BrokerStub(WsaEndpointReference widgetEpr) { super(widgetEpr); }
	 * 
	 * public WsrfResourceStub registerPublisher(WsaEndpointReference
	 * wsaEndpointReference) { invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSBR_NS, "RegisterPublisher");
	 * 
	 * WsaEndpointReference publisherRef; try { publisherRef =
	 * (WsaEndpointReference) wsaEndpointReference.clone(); } catch
	 * (CloneNotSupportedException e) { throw new XsulException("internal error:
	 * could not clone EPR"); }
	 * publisherRef.setNamespace(WsmgConstants.WSBR_NS);
	 * publisherRef.setName("PublisherReference");
	 * message.addElement(publisherRef);
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName())); XmlElement
	 * responseMessage = invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage) + "\n\n\n"); XmlElement
	 * publisherRegistrationRef = responseMessage.requiredElement(
	 * WsmgConstants.WSBR_NS, "PublisherRegistrationReference"); return new
	 * WsrfResourceStub(new WsaEndpointReference( publisherRegistrationRef)); } }
	 */

	/**
	 * This stub is NOT multi-thread safe!
	 */
	/*
	 * public static class NotificationProducerStub extends WidgetStub {
	 * 
	 * NotificationProducerStub(WsaEndpointReference widgetEpr) {
	 * super(widgetEpr); }
	 * 
	 * public XmlElement getCurrentMessage(XmlElement topicExp) {
	 * invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSNT_NS, "GetCurrentMessage");
	 * message.addChild(topicExp);
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName())); XmlElement
	 * responseMessage = invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage.getRoot()) + "\n\n\n"); if
	 * (responseMessage.hasChildren() == false) { //INTEROP can it be empoty?
	 * return null; } else { return (XmlElement)
	 * responseMessage.requiredElementContent().iterator(). next(); } }
	 * 
	 * public SubscriptionStub subscribe(WsaEndpointReference consumerReference,
	 * XmlElement topicExpression, boolean useNotfy) {
	 * invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSNT_NS, "Subscribe");
	 * 
	 * WsaEndpointReference cr; try { cr = (WsaEndpointReference)
	 * consumerReference.clone(); } catch (CloneNotSupportedException e) { throw
	 * new XsulException("internal error: could not clone EPR"); }
	 * cr.setNamespace(WsmgConstants.WSNT_NS); cr.setName("ConsumerReference");
	 * message.addElement(cr); // <wsnt:TopicExpression
	 * dialect="http://www.ibm.com/xmlns/stdwip/web-services/WS-Topics/TopicExpressiom/simple">widget:TestTopic</wsnt:TopicExpression>
	 * message.addElement(topicExpression);
	 * message.addElement(WsmgConstants.WSNT_NS, "UseNotify").addChild( useNotfy ?
	 * "true" : "false");
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName())); XmlElement
	 * responseMessage = invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage) + "\n\n\n"); XmlElement sr =
	 * responseMessage.requiredElement(WsmgConstants.WSNT_NS,
	 * "SubscriptionReference"); return new SubscriptionStub(new
	 * WsaEndpointReference(sr)); } }
	 */

	/**
	 * This stub is NOT multi-thread safe!
	 */
	/*
	 * public static class WidgetStub extends WsrfResourceStub {
	 * 
	 * protected WidgetStub(WsaEndpointReference widgetEpr) { super(widgetEpr); }
	 * 
	 * public void generateNotification() { invoker.setMessageId(null); //String
	 * msg = XmlElement message = builder.newFragment(WsmgConstants.WIDGET_NS,
	 * "generateNotification");
	 * //invoker.setDefaultAction(URI.create(message.getNamespace().getNamespaceName()+"/"+message.getName()));
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/generateEvent")); XmlElement responseMessage =
	 * invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage) + "\n\n\n"); }
	 *  }
	 */

	private static class WidgetCreator extends WsrfResourceStub {

		// private URI location;

		WidgetCreator(URI location) {
			// this.location = location;
			super(new WsaEndpointReference(location));
		}

		private WidgetStub createWidget() {
			// WsaInvoker invoker = new WsaInvoker();
			// invoker.setSoapFragrance(Soap12XsulUtil.getInstance());
			// invoker.setSoapFragrance(Soap11XsulUtil.getInstance());
			// invoker.setTargetEPR(epr);
			invoker.setMessageId(null);
			// invoker.setIndent(null);
			XmlElement message = builder.newFragment(WsmgConstants.WIDGET_NS,
					"createWidget");
			invoker.setDefaultAction(URI.create(message.getNamespace()
					.getNamespaceName()
					+ "/" + message.getName()));

			XmlElement responseMessage = invoker.invokeMessage(message);

			XmlElement e = responseMessage.requiredElement(
					WsAddressing.NS_2004_08, WsaEndpointReference.NAME);
			WsaEndpointReference widgetEpr = new WsaEndpointReference(e);
			verbose("\n\nWIDGET EPR=" + widgetEpr + "\n\n\n");

			return new WidgetStub(widgetEpr);
		}

	}
	/*
	 * public static class WsrfResourceStub { protected WsaInvoker invoker; // =
	 * new WsaInvoker(); private WsaEndpointReference resourceEpr;
	 * 
	 * WsrfResourceStub(WsaEndpointReference resourceEpr) { this.resourceEpr =
	 * resourceEpr; logger.finest("resourceEprInWsrfResourceStub
	 * Constructor"+builder.serializeToString(resourceEpr)); invoker = new
	 * WsaInvoker(); invoker.setTargetEPR(resourceEpr);
	 * invoker.setSoapFragrance(Soap11XsulUtil.getInstance()); invoker.setIndent("
	 * "); // to improve interop do not use KA
	 * invoker.setUseHttpKeepAlive(false);
	 *  }
	 * 
	 * public WsaEndpointReference getResourceEpr() { return resourceEpr; }
	 * 
	 * public void destroy() { invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSRL_NS, "Destroy");
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName())); XmlElement
	 * responseMessage = invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage.getRoot()) + "\n\n\n"); }
	 * 
	 * public void setTerminationTime(Calendar cal) {
	 * invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSRL_NS, "SetTerminationTime");
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName()));
	 * 
	 * XmlElement child = message.addElement(WsmgConstants.WSRL_NS,
	 * "RequestedTerminationTime"); if (cal == null) {
	 * child.addAttribute(XmlConstants.XSI_NS, "nil", "true"); } else {
	 * child.addChild(new DcDate(cal)); } XmlElement responseMessage =
	 * invoker.invokeMessage(message); verbose("\n\nRESPONSE=" +
	 * XsulUtil.safeXmlToString(responseMessage.getRoot()) + "\n\n\n"); }
	 * 
	 * public List getResourceProperty(QName qn) { //List<XmlElement>
	 * XmlNamespace ns = builder.newNamespace(qn.getPrefix(),
	 * qn.getNamespaceURI()); String name = qn.getLocalPart();
	 * invoker.setMessageId(null);
	 * 
	 * XmlElement message = builder.newFragment(WsmgConstants.WSRP_NS,
	 * "GetResourceProperty");
	 * invoker.setDefaultAction(URI.create(message.getNamespaceName() + "/" +
	 * message.getName()));
	 * 
	 * //message.addChild(new QNameElText(message, WidgetService.WIDGET_NS,
	 * "TerminationTime")); //message.declareNamespace(WidgetService.WIDGET_NS);
	 * message.addChild(new QNameElText(message, ns, name));
	 * //message.declareNamespace(WidgetService.WSRL_NS); if (ns.getPrefix() !=
	 * null) { message.declareNamespace(ns); } //child.addChild(new
	 * DcDate(TimeZone.getDefault())); XmlElement responseMessage =
	 * invoker.invokeMessage(message);
	 * 
	 * verbose("\n\nRESPONSE=" + XsulUtil.safeXmlToString(responseMessage.getRoot()) +
	 * "\n\n\n"); List list = elementsAsList(responseMessage); return list; }
	 * 
	 * public List getMultipleResourceProperties(QName[] qnamez) { //TODO JDK15
	 * List<XmlElement> invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSRP_NS,
	 * "GetMultipleResourceProperties");
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName()));
	 * 
	 * //message.addChild(new QNameElText(message, WidgetService.WIDGET_NS,
	 * "TerminationTime")); //message.declareNamespace(WidgetService.WIDGET_NS);
	 * //wsrp:ResourceProperty
	 * 
	 * for (int i = 0; i < qnamez.length; i++) { QName qn = qnamez[i];
	 * XmlNamespace ns = builder.newNamespace(qn.getPrefix(),
	 * qn.getNamespaceURI()); XmlElement child =
	 * message.addElement(WsmgConstants.WSRP_NS, "ResourceProperty");
	 * child.addChild(new QNameElText(message, ns, qn.getLocalPart())); if
	 * (qn.getPrefix() != null) { child.declareNamespace(ns); } }
	 * 
	 * XmlElement responseMessage = invoker.invokeMessage(message);
	 * 
	 * verbose("\n\nRESPONSE=" + XsulUtil.safeXmlToString(responseMessage.getRoot()) +
	 * "\n\n\n");
	 * 
	 * List list = elementsAsList(responseMessage); return list; }
	 * 
	 * public List queryResourcePropertiesByXpath(String query) { if (query ==
	 * null) { throw new IllegalArgumentException(); }
	 * invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSRP_NS, "QueryResourceProperties");
	 * invoker.setDefaultAction(URI.create(message.getNamespaceName() + "/" +
	 * message.getName()));
	 * 
	 * XmlElement queryExpressionEl = message.addElement(WsmgConstants.WSRP_NS,
	 * "QueryExpression"); queryExpressionEl.addAttribute("dialect",
	 * WsmgConstants.XPATH1_DIALECT); queryExpressionEl.addChild(query);
	 * 
	 * XmlElement responseMessage = invoker.invokeMessage(message);
	 * 
	 * verbose("\n\nRESPONSE=" + XsulUtil.safeXmlToString(responseMessage.getRoot()) +
	 * "\n\n\n");
	 * 
	 * List list = childrenAsList(responseMessage); return list; }
	 * 
	 * public void setResourceProperties(XmlElement[] requests) { if
	 * (requests.length == 0) { throw new IllegalArgumentException("at least one
	 * request is required"); }
	 * 
	 * invoker.setMessageId(null); XmlElement message =
	 * builder.newFragment(WsmgConstants.WSRP_NS, "SetResourceProperties");
	 * invoker.setDefaultAction(URI.create(message.getNamespace().
	 * getNamespaceName() + "/" + message.getName())); for (int i = 0; i <
	 * requests.length; i++) { message.addElement(requests[i]); }
	 * //message.addChild(new QNameElText(message, WidgetService.WIDGET_NS,
	 * "TerminationTime")); //message.declareNamespace(WidgetService.WIDGET_NS);
	 * //wsrp:ResourceProperty
	 * //message.declareNamespace(WidgetService.WSRL_NS);
	 * 
	 * XmlElement responseMessage = invoker.invokeMessage(message);
	 * 
	 * verbose("\n\nRESPONSE=" + XsulUtil.safeXmlToString(responseMessage.getRoot()) +
	 * "\n\n\n"); }
	 * 
	 * private List elementsAsList(XmlElement responseMessage) { List list = new
	 * ArrayList(); for (Iterator it =
	 * responseMessage.requiredElementContent().iterator(); it.hasNext(); ) {
	 * XmlElement el = (XmlElement) it.next(); list.add(el); } return list; }
	 * 
	 * private List childrenAsList(XmlElement responseMessage) { List list = new
	 * ArrayList(); for (Iterator it = responseMessage.children(); it.hasNext(); ) {
	 * Object child = it.next(); list.add(child); } return list; }
	 *  }
	 */

}
/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
