package wsmg.wsrm;

import java.io.ByteArrayInputStream;

import javax.xml.namespace.QName;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.impl.llom.factory.OMXMLBuilderFactory;
import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.sandesha2.client.SandeshaClientConstants;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import wsmg.NotificationException;
import wsmg.WsmgConstants;
import xsul.XmlConstants;

/**
 * 
 * The adaptor class that will interface the WSRM part (which is based on
 * Sandesha2) and the Notification brocker.
 * 
 * @author Chathura Herath
 * 
 */
public class WSRMAdaptor implements RMNotificationSender {

    
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    /* (non-Javadoc)
     * @see wsmg.wsrm.RMNotificationSender#send(org.xmlpull.v1.builder.XmlElement, java.lang.String)
     */
    public void send(XmlElement element, String toEPR) throws AxisFault {
        OMElement body = this.getNotificationMessage(element);
        body.build();
        ServiceClient sender = getConfiguredMessageSender(toEPR);
        sender.fireAndForget( body);
        

        
    }
    
   

    private OMElement getNotificationMessage(XmlElement element) {
        OMElement omElement = null;
        XMLStreamReader parser = null;
        try {
            String string = builder.serializeToString(element);
           
            parser = XMLInputFactory.newInstance().createXMLStreamReader(
                    new ByteArrayInputStream(string.getBytes()));
        } catch (XMLStreamException e) {
            throw new NotificationException(
                    "Cannot create message for sandesha", e);
        } catch (FactoryConfigurationError e) {
            throw new NotificationException(e);
        }
        StAXOMBuilder builder = OMXMLBuilderFactory.createStAXOMBuilder(
                OMAbstractFactory.getOMFactory(), parser);
        omElement = builder.getDocumentElement();
        return omElement;
    }

    private ServiceClient getConfiguredMessageSender(String eventSinkLoc)
            throws AxisFault {
        ConfigurationContext configContext = ConfigurationContextFactory.createConfigurationContextFromFileSystem(WsmgConstants.AXIS2_MAR_LOCATION,WsmgConstants.AXIS2_MAR_LOCATION+"/axis2.xml");
        ServiceClient sender = new ServiceClient(configContext, null);
        Options clientOptions = new Options();
        clientOptions.setProperty(Options.COPY_PROPERTIES, new Boolean(true));
        sender.setOverrideOptions(clientOptions);
        clientOptions
                .setSoapVersionURI(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI);
        // clientOptions.setProperty(Sandesha2ClientAPI.AcksTo,acksToEPR);
        //sender.engageModule(new QName(WsmgConstants.SANDESHA_VERSIONED_MODULE_NAME));
        clientOptions.setTo(new EndpointReference(eventSinkLoc));
        clientOptions.setProperty(SandeshaClientConstants.SEQUENCE_KEY, "sequence1");
        clientOptions.setAction("notify");
        clientOptions.setProperty(SandeshaClientConstants.LAST_MESSAGE, "true");
        return sender;

    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

