package wsmg.wsrm.axis2;

import javax.xml.namespace.QName;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.InOnlyAxisOperation;
import org.apache.axis2.description.InOutAxisOperation;
import org.apache.axis2.transport.http.SimpleHTTPServer;
import org.apache.axis2.transport.http.server.SimpleHttpServer;

import wsmg.WsmgConstants;
import wsmg.WsmgRuntimeException;
import wsmg.util.WsmgUtil;

/**
 * @author Chathura Herath (cherath@cs.indiana.edu)
 */

public class DedicatedSimpleSandeshaServer {
    private static String SANDESHA2_HOME = "F:\\Sandesha2-0.9-src";

    private static String AXIS2_SERVER_PATH = SANDESHA2_HOME
            + "\\target\\repos\\server\\";

    private SandeshaConfiguration config;

    private String operationCOntextPath;

    private boolean serverSide;

    private SimpleHTTPServer server;

    /**
     * Initializes the server but will not start it. The SandeshaConfiguration
     * would contain the port in which the server will start and a handler
     * object. If this is a
     * 
     * Server-side - config.handler will be of the type <code>WSMGService</code>
     * and should have used the public SandeshaConfiguration(int port,
     * WSMGService broker) Constructor
     * 
     * 
     * Cilent-side -config.handler will be of the type
     * <code>NotificationHandler</code> and should have used public
     * SandeshaConfiguration(int port, NotificationHandler handler) Constructor
     * 
     * @param config
     */
    public DedicatedSimpleSandeshaServer(SandeshaConfiguration config) {
        this.config = config;
    }

    public void start() {
        try {
            ConfigurationContext systemContext = ConfigurationContextFactory.createConfigurationContextFromFileSystem(WsmgConstants.AXIS2_MAR_LOCATION,WsmgConstants.AXIS2_MAR_LOCATION+"/axis2.xml");

            
            server = new SimpleHTTPServer(systemContext, config.getPort());
            

            systemContext.setProperty(
                    WsmgConstants.INVOCATION_HANDLER_KEY_FOR_SANDESHA2, config
                            .getHandler());

//            systemContext.getAxisConfiguration().engageModule(
//                    new QName("sandesha2-SNAPSHOT"));

            System.out.println(systemContext.getAxisConfiguration()
                    .getEngagedModules());
            server.start();
        } catch (AxisFault e) {
            throw new WsmgRuntimeException("Unble to startup RM service", e);
        }

    }
    
    
    public void shutDown(){
        if(this.server != null && this.server.isRunning()){
            this.server.stop();
        }
    }

    public void deployServersideServices() {
        AxisService service = new AxisService();
        service.setName("wsrmbrocker");

        service.setClassLoader(Thread.currentThread().getContextClassLoader());

        AxisOperation axisOp = new InOnlyAxisOperation(new QName("publish"));

        this.operationCOntextPath = "wsrmbrocker/publish";
        axisOp.setMessageReceiver(new WSRMBrockerMessageReceiver());
        service.addOperation(axisOp);
        axisOp.setSoapAction("http://www.extreme.indiana.edu/WseNotification");
        
        try {
            service.engageModule(new AxisModule(new QName(
                    WsmgConstants.SANDESHA_VERSIONED_MODULE_NAME)), this.server.getConfigurationContext()
                    .getAxisConfiguration());

            this.server.getConfigurationContext().getAxisConfiguration().addService(
                    service);
        } catch (AxisFault e) {
            throw new WsmgRuntimeException(
                    "Failed to start Sandesha serverpoint", e);
        }
        System.out
                .println("WSRM enabled WS-Eventing/Notification Broker (Axis2/Sandesh2) started at "
                        + getServerLocation());

    }

    public void deployClientService() {
        AxisService service = new AxisService();
        service.setName("wsrmclient");

        service.setClassLoader(Thread.currentThread().getContextClassLoader());

        AxisOperation axisOp = new InOutAxisOperation(new QName("notify"));

        axisOp.setMessageReceiver(new WSRMBrockerCallbackMessageReceiver());
        service.addOperation(axisOp);

        this.operationCOntextPath = "wsrmclient/notify";
        axisOp.setSoapAction("http://www.extreme.indiana.edu/WseNotification");
        

        try {
            service.engageModule(new AxisModule(new QName(
                    WsmgConstants.SANDESHA_VERSIONED_MODULE_NAME)), this.server.getConfigurationContext()
                    .getAxisConfiguration());

            this.server.getConfigurationContext().getAxisConfiguration().addService(
                    service);
        } catch (AxisFault e) {
            throw new WsmgRuntimeException(
                    "Failed to start Sandesha serverpoint", e);
        }
        System.out
                .println("WSRM enabled WS-Eventing/Notification Broker (Axis2/Sandesh2) started at "
                        + getServerLocation());

    }

    public String getServerLocation() {
        return "http://" + WsmgUtil.getHostIP() + ":" + config.getPort()
                + "/axis2/services/" + operationCOntextPath;
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

