package wsmg.wsrm.axis2;

import java.io.StringReader;
import java.net.URI;
import java.net.URISyntaxException;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.receivers.AbstractMessageReceiver;
import org.xmlpull.v1.builder.XmlBuilderException;
import org.xmlpull.v1.builder.XmlDocument;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import wsmg.ProcessingContext;
import wsmg.SoapClientFault;
import wsmg.SoapFault;
import wsmg.SoapServerFault;
import wsmg.WSMGService;
import wsmg.WsmgConfig;
import wsmg.WsmgConstants;
import wsmg.WsmgRuntimeException;
import wsmg.util.WsmgUtil;
import xsul.XmlConstants;
import xsul.processor.DynamicInfosetProcessorException;
import xsul.soap.SoapUtil;
import xsul.soap11_util.Soap11Util;
import xsul.soap12_util.Soap12Util;
import xsul.util.FastUUIDGen;
import xsul.ws_addressing.WsAddressing;
import xsul.ws_addressing.WsaEndpointReference;
import xsul.ws_addressing.WsaMessageInformationHeaders;
import xsul.ws_addressing.WsaRelatesTo;

public class WSRMBrockerMessageReceiver extends AbstractMessageReceiver {

    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    private final static SoapUtil[] soapFragrances = new SoapUtil[] {
            Soap11Util.getInstance(), Soap12Util.getInstance() };

    public void receive(MessageContext messageCtx) throws AxisFault {
        SOAPEnvelope envelope = messageCtx.getEnvelope();
        String message = envelope.toString();
        XmlElement root = builder.parseFragmentFromReader(new StringReader(
                message));

        org.apache.axis2.description.AxisOperation op = messageCtx
                .getOperationContext().getAxisOperation();

        SoapUtil soapFragrance = SoapUtil.selectSoapFragrance(root,
                soapFragrances);
        processSoapEnvelope(root, soapFragrance, messageCtx);
        return;

    }

    public XmlDocument processSoapEnvelope(XmlElement envelope,

    SoapUtil soapFragrance, MessageContext msgContext)
            throws DynamicInfosetProcessorException {
        XmlDocument respDoc = null;

        // extract resource id - if any
        WsaMessageInformationHeaders headers = new WsaMessageInformationHeaders(
                envelope);
        // extract operation
        ProcessingContext ctx = new ProcessingContext();
        try {
            ctx.setSoapBody(soapFragrance.requiredBodyContent(envelope));
        } catch (Exception e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }

        System.err.println("Context" + ctx);
        ctx.setEnvelope(envelope);
        WSMGService broker = (WSMGService) msgContext
                .getConfigurationContext()
                .getProperty(WsmgConstants.INVOCATION_HANDLER_KEY_FOR_SANDESHA2);
        if (broker == null) {
            broker = init(msgContext);
        }
        System.err.println("Broker shouldnt be null :" + broker);

        broker.processHeaders(ctx);

        URI respAction = WsAddressing.URI_ROLE_ANONYMOUS;
        try {
            try {
                broker.executeSoapEnvelope(ctx);
            } catch (XmlBuilderException ex) {
                throw new SoapClientFault("problem with parsing XML: " + ex, ex);
            }
            if (ctx.getRespMessage() != null) {
                respAction = URI.create(ctx.getRespMessage().getNamespace()
                        .getNamespaceName()
                        + "/" + ctx.getRespMessage().getName());
            }
        } catch (SoapServerFault sse) {
            ctx.setRespMessage(soapFragrance.generateSoapServerFault(sse
                    .getMessage(), sse));
        } catch (SoapClientFault sce) {
            ctx.setRespMessage(soapFragrance.generateSoapClientFault(sce
                    .getMessage(), sce));
        } catch (SoapFault sf) {
            // String ns = sf.getFaultCodeValueNs() != null ?
            // sf.getFaultCodeValueNs().getNamespaceName() : null;
            ctx.setRespMessage(soapFragrance.generateSoapFault(sf
                    .getFaultCodeValueNs(), sf.getFaultCodeValueName(), sf
                    .getMessage(), sf));
        }
        // prepare response document with all fancy WS-Addressing
        respDoc = soapFragrance.wrapBodyContent(ctx.getRespMessage());
        if (respDoc != null) {
            XmlElement respEnv = respDoc.getDocumentElement();

            respEnv.declareNamespace(WsmgConstants.WIDGET_NS); // widget prefix

            WsaMessageInformationHeaders respHeaders = new WsaMessageInformationHeaders(
                    respDoc);

            respHeaders.setAction(respAction);
            respHeaders.setMessageId(URI.create("uuid:"+FastUUIDGen.nextUUID()));
            respHeaders.setTo(WsAddressing.URI_ROLE_ANONYMOUS);
            URI epr = null;
            try {
                epr = new URI("http://localhost:8888/");
            } catch (URISyntaxException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            respHeaders.setFrom(new WsaEndpointReference(epr));
            URI messageId = headers.getMessageId();
            if (messageId != null) {
                respHeaders.addRelatesTo(new WsaRelatesTo(messageId));
            }

        }

        return respDoc;
    }

    private synchronized WSMGService init(MessageContext msgContext) {

        System.err.println("It was null so enterred the backup");
        WsmgConfig wsmgConfig = new WsmgConfig();
        wsmgConfig
                .setBrokerWebServiceStack(WsmgConstants.BROKER_WEB_SERVICE_STACK_TOMCAT_XSUL);
        try {
            wsmgConfig.setServiceLocation(new URI("http://"
                    + WsmgUtil.getHostIP() + ":8080/axis2s"));
        } catch (URISyntaxException e) {
            throw new WsmgRuntimeException(e);
        }
        wsmgConfig
                .setBrokerWebServiceStack(WsmgConstants.BROKER_WEB_SERVICE_STACK_TOMCAT_XSUL);
        WSMGService broker = new WSMGService(wsmgConfig);

        broker = new WSMGService(wsmgConfig);
        broker.start();
        msgContext.getConfigurationContext().setProperty(
                WsmgConstants.INVOCATION_HANDLER_KEY_FOR_SANDESHA2, broker);
        return broker;
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 *
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

