/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2002-2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: VerySimpleClient.java,v 1.2 2006/01/04 17:47:25 cherath Exp $
 */

package wsmg.xdir;

import org.gjt.xpp.XmlNode;

import xdrs_util.XmlNodeUtil;
import xsul.MLogger;

// import soaprmi.RemoteException;
// import xsul.XsulException;

/**
 * Example client of directory service (portlet would be another example of
 * client)
 * 
 * @version $Revision: 1.2 $ $Date: 2006/01/04 17:47:25 $ (GMT)
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 * @author Gopi
 */
public class VerySimpleClient extends BaseClient {
	// http://rainier.extreme.indiana.edu:2020
	// http://linbox1.extreme.indiana.edu:8047
	// --port 2020 --cert hostcert.pem --key hostkey.pem
	// --port 8115
	// --port 7020 --cert hostcert.pem --key hostkey.pem
	private final static MLogger logger = MLogger.getLogger();

	public static void main(String[] args) {
		(new VerySimpleClient()).run(args);
	}

	public String getPrefix() {
		return "XDRS VerySimple Client: ";
	}

	public void doWork() throws Exception {

		// XmlNode result = null;

		{
			String path = "";
			msg("Listing directory content of " + path);
			XmlNode list[] = directoryService.getChildrenWithoutHtml(path);
			if (list != null && list.length > 0) {
				for (int j = 0; j < list.length; j++) {
					XmlNode child = list[j];
					String name = XmlNodeUtil.getNodeName(child);
					msg("Directory has node: " + name);
					msg("'" + name + "' node XML content="
							+ XmlNodeUtil.convertXmlTreeToString(child));
				}
			} else {
				msg("no nodes in " + path);
			}
		}
		XmlNode dirNodeTest = XmlNodeUtil.newDirNode("Yi Huang Test 1");
		msg("Storing dir node "
				+ XmlNodeUtil.convertXmlTreeToString(dirNodeTest));
		directoryService.addNode("", dirNodeTest);

		boolean adding = false;
		if (adding) {
			// create first directoy under root
			XmlNode dirNode = XmlNodeUtil.newDirNode("helloDir1");
			msg("Storing dir node "
					+ XmlNodeUtil.convertXmlTreeToString(dirNode));
			directoryService.addNode("", dirNode);

			dirNode = XmlNodeUtil.newDirNode("helloDir2");
			msg("Storing dir node "
					+ XmlNodeUtil.convertXmlTreeToString(dirNode));
			directoryService.addNode("", dirNode);

			dirNode = XmlNodeUtil.newDirNode("helloDir3");
			msg("Storing dir node "
					+ XmlNodeUtil.convertXmlTreeToString(dirNode));
			directoryService.addNode("", dirNode);

			dirNode = XmlNodeUtil.newDirNode("helloDir4");
			msg("Storing dir node "
					+ XmlNodeUtil.convertXmlTreeToString(dirNode));
			directoryService.addNode("", dirNode);

			// Create a subdirectory within the directory
			dirNode = XmlNodeUtil.newDirNode("helloDirSubNode");
			msg("Storing dir node "
					+ XmlNodeUtil.convertXmlTreeToString(dirNode));
			directoryService.addNode("helloDir3", dirNode);

			// add a leaf node to the directory
			XmlNode leafNode = XmlNodeUtil.newLeafNode("foo");
			XmlNodeUtil.setNodeHtml(leafNode, "<p>Hello in Html</p>");
			final String CREATED = "created";
			final String AUTHOR_NS = "http://tempuri.org/author";
			XmlNodeUtil.setNodeMetadataItemFromXmlString(leafNode, "<m:"
					+ CREATED + " xmlns:m='" + AUTHOR_NS + "'>alek</m:"
					+ CREATED + ">");

			msg("Storing node " + XmlNodeUtil.convertXmlTreeToString(leafNode));
			directoryService.addNode("helloDir3", leafNode);
		}

		if (false) {
			String path = "new2";

			XmlNode newNode = XmlNodeUtil.newDirNode(path);
			msg("Storing dir node "
					+ XmlNodeUtil.convertXmlTreeToString(newNode));
			directoryService.addNode("", newNode);

			XmlNode alekNode = XmlNodeUtil.newLeafNode("testAlek2");
			msg("Storing leaf node "
					+ XmlNodeUtil.convertXmlTreeToString(alekNode));
			directoryService.addNode(path, alekNode);

			XmlNode list[] = directoryService.getChildrenWithoutHtml(path);

			msg("Listing directory content of " + path);
			if (list != null && list.length > 0) {
				for (int j = 0; j < list.length; j++) {
					XmlNode child = list[j];
					String name = XmlNodeUtil.getNodeName(child);
					msg("Directory has node: " + name);
					msg("'" + name + "' node XML content="
							+ XmlNodeUtil.convertXmlTreeToString(child));
				}
			} else {
				msg("no nodes in " + path);
			}
		}

	}

}
/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2002-2004 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
