package wsmg.xdir;

import java.io.StringReader;

import javax.xml.namespace.QName;

import org.gjt.xpp.XmlNode;
import org.gjt.xpp.XmlPullParserException;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import org.xmlpull.v1.builder.XmlNamespace;

import soaprmi.RemoteException;
import wsmg.NotificationHandler;
import xdrs.XDirectoryService;
import xdrs_util.XmlNodeUtil;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.XsulException;
import xsul.soap.SoapUtil;
import xsul.soap11_util.Soap11Util;
import xsul.soap12_util.Soap12Util;
import xsul.util.XsulUtil;

public class XDirNotificationHandler implements NotificationHandler {
	private final static MLogger logger = MLogger.getLogger();

	XDirectoryService directoryService;

	private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

	public final static XmlNamespace WSNT_NS = builder.newNamespace("wsnt",
			"http://www.ibm.com/xmlns/stdwip/web-services/WS-BaseNotification");

	public final static String SIMPLE_TOPIC_DIALECT = "http://www.ibm.com/xmlns/stdwip/web-services/WS-Topics/TopicExpression/simple";

	private final static SoapUtil[] soapFragrances = new SoapUtil[] {
			Soap12Util.getInstance(), Soap11Util.getInstance() };

	public XDirNotificationHandler(XDirectoryService directoryService) {
		this.directoryService = directoryService;
	}

	public void handleNotification(String message) {
		System.out
				.println("*******XDirectory Notification handler Received message********");
		System.out.println(message);
		System.out.println("*******GET here.0");

		// parse the received String message and create XmlElement object
		System.out.println("*******GET here.0.1");
		XmlElement messageEl = builder
				.parseFragmentFromReader(new StringReader(message));
		System.out.println("*******GET here.0.2");
		System.out.println(XsulUtil.safeXmlToString(messageEl));
		System.out.println(messageEl.getName() + " of "
				+ messageEl.getNamespaceName());
		// System.out.println(messageEl.getBaseUri());
		System.out.println("*******GET here.0.3");

		SoapUtil soapFragrance = null;
		for (int i = 0; i < soapFragrances.length; i++) {
			if (soapFragrances[i].isSoapEnvelopeSupported(messageEl)) {
				soapFragrance = soapFragrances[i];
				break;
			}
		}

		if (soapFragrance == null) {
			throw new RuntimeException("only SOAP 1.1 or 1.2 is supported");
		}

		XmlElement bodyContent = soapFragrance.requiredBodyContent(messageEl);
		System.out.println("*******GET here.1");
		System.out.println(messageEl.toString());
		logger.finest("before extracting Topic from " + bodyContent.toString());

		// builder.
		// Find the topic of the message
		XmlElement topicExpressionEl = bodyContent
		// .requiredElement(WSNT_NS,"Notify")
				.requiredElement(WSNT_NS, "NotificationMessage")
				.requiredElement(WSNT_NS, "Topic");
		System.out.println("*******GET here..1.1");
		if (topicExpressionEl == null) {
			System.out.println("It is null");
		}
		System.out.println("TopicExpression=" + topicExpressionEl.getName());
		System.out.println("*******GET here..1.11");
		String topicExpression = topicExpressionEl.requiredTextContent();
		System.out.println("*******GET here.1.2");
		String dialectString = topicExpressionEl.attribute("dialect")
				.getValue();

		System.out.println("*******GET here.2");
		String topicLocalString = null;
		if (dialectString.equals(SIMPLE_TOPIC_DIALECT)) {
			QName topicExpressionQName = XsulUtil.toQName(topicExpressionEl,
					topicExpression);
			topicLocalString = topicExpressionQName.getLocalPart();
		} else {
			try {
				throw new soaprmi.soaprpc.SoapClientFault("Unkown dialect: "
						+ dialectString);
			} catch (soaprmi.soaprpc.SoapClientFault ex) {
				ex.printStackTrace();
			}
		}

		System.out.println("*******GET here.3");
		java.util.Date today = new java.util.Date();
		long timeStamp = today.getTime();

		// add a leaf node to the directory
		XmlNode leafNode = null;
		try {
			leafNode = XmlNodeUtil.newLeafNode(topicLocalString + timeStamp);
			// XmlNodeUtil.setNodeHtml(leafNode, "<p>Hello in Html</p>");
			XmlNodeUtil.setNodeHtml(leafNode, message);
			final String CREATED = "created";
			final String AUTHOR_NS = "http://tempuri.org/author";
			XmlNodeUtil.setNodeMetadataItemFromXmlString(leafNode, "<m:"
					+ CREATED + " xmlns:m='" + AUTHOR_NS + "'>YiHuang</m:"
					+ CREATED + ">");
			System.out.println("Storing node "
					+ XmlNodeUtil.convertXmlTreeToString(leafNode));
		} catch (XmlPullParserException ex1) {
			ex1.printStackTrace();
		}

		try {
			directoryService.addNode("helloDir4", leafNode);
			directoryService.setPermissions("helloDir4", "Yi Huang", "extreme",
					"extreme", true);

			// directoryService.addNode("Gopi's Dir", leafNode);
		} catch (XsulException ex2) {
			ex2.printStackTrace();
		} catch (RemoteException ex2) {
			ex2.printStackTrace();
		}

	}

}

/*
 * XmlElement topicEl = messageEl.requiredElement(WSNT_NS, "Topic"); String
 * topicExpression = topicEl.requiredTextContent(); String dialectString =
 * topicEl.attribute("dialect").getValue(); String topicLocalString = null; if
 * (dialectString.equals(SIMPLE_TOPIC_DIALECT)) { QName topicExpressionQName =
 * Util.toQName(topicEl, topicExpression); topicLocalString =
 * topicExpressionQName.getLocalPart();
 * System.out.println("Topic="+topicLocalString); } else { try { throw new
 * SoapClientFault("Unkown dialect: " + dialectString); } catch (SoapClientFault
 * ex3) { System.out.println("Throws Exception"); ex3.printStackTrace(); } }
 */
