package wsmg.xdir;

import java.io.IOException;
import java.security.cert.X509Certificate;

import org.exolab.jms.util.CommandLine;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.GlobusCredentialException;
import org.globus.gsi.TrustedCertificates;

import soaprmi.RemoteException;
import wsmg.NotificationHandler;
import wsmg.WsntClientAPI;
import xdrs.XDirectoryService;
import xdrs_impl.Run;
import xdrs_util.XmlNodeUtil;
import xsul.MLogger;
import xsul.XsulException;
import xsul.processor.DynamicInfosetProcessorException;

//Used to receive WS-Notification message and send to X-Directory service
public class XdirClient {
	private final static MLogger logger = MLogger.getLogger();

	boolean QUIET = false;

	boolean secure = true;

	boolean xdrs_internal_test = System.getProperty("xdrs_internal_test") != null;

	String key = System.getProperty(Run.XDRS_KEY);

	String cert = System.getProperty(Run.XDRS_CERT);

	String internalServerPort;

	public XdirClient() {
	}

	public static void main(String[] args)
			throws DynamicInfosetProcessorException, IOException {
		CommandLine cmdline = new CommandLine(args);
		XDirectoryService directoryService = null;
		XdirClient xdirClient = new XdirClient();
		// default port 19999
		int port = Integer.parseInt(cmdline.value("port", "19999"));
		String location = cmdline
				.value("xdirLocation", "http://localhost:1847");
		WsntClientAPI client = new WsntClientAPI();
		// Create a handler to handle the notifications arrived
		try {
			directoryService = xdirClient.getXdrsStartpoint(location);
		} catch (RemoteException ex) {
			ex.printStackTrace();
		} catch (GlobusCredentialException ex) {
			ex.printStackTrace();
		} catch (XsulException ex) {
			ex.printStackTrace();
		}

		NotificationHandler handler = new XDirNotificationHandler(
				directoryService);
		client.startConsumerService(port, handler);
	}

	/**
	 * Create startpoint to XDRS that is configured according to command line
	 * options and system properties and print what is security context
	 * configuration
	 */
	private XDirectoryService getXdrsStartpoint(String location)
			throws XsulException, GlobusCredentialException, RemoteException {
		XDirectoryService directoryService;
		if (!secure) {
			// get access to directory service located at URL passed from
			// command line
			directoryService = XmlNodeUtil.getDirectoryServiceProxy(location);
		} else {

			GlobusCredential cred;
			if (key != null || cert != null) {
				msg("using certificate " + cert + " and key " + key);
				cred = new GlobusCredential(cert, key);
			} else {
				msg("using default CoG credential");
				cred = GlobusCredential.getDefaultCredential();
			}

			// msg("loading default globus credential (grid proxy) ...");
			// GlobusCredential cred = GlobusCredential.getDefaultCredential();
			logger.config("cred=" + cred);
			msg("using globus credential with DN=" + cred.getSubject());

			TrustedCertificates tc = TrustedCertificates
					.getDefaultTrustedCertificates();
			if (tc == null) {
				throw new XsulException(
						getPrefix()
								+ " can not be started as trusted certificates were not found");
			}

			X509Certificate[] trustedCerts = tc.getCertificates();
			msg("list of trusted of certifcates");
			for (int i = 0; i < trustedCerts.length; i++) {
				msg("trusted cert " + i + "=" + trustedCerts[i].getSubjectDN());
			}

			directoryService = XmlNodeUtil.getDirectoryServiceProxy(location,
					cred, trustedCerts);
			msg("*****using directory service " + location
					+ " with credential " + cred.getIssuer());
		}
		return directoryService;
	}

	private void msg(String msg) {
		if (!QUIET) {
			System.err.println(getPrefix() + msg);
		}
	}

	public String getPrefix() {
		return "VerySimple WS_Notification Client: ";
	}

}