/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: XSULDedicatedBrockerService.java,v 1.17 2007/01/12 18:20:55 yihuan Exp $
 */
package wsmg.xsul;

import java.io.IOException;
import java.net.URI;
import org.xmlpull.v1.XmlSerializer;
import org.xmlpull.v1.builder.XmlBuilderException;
import org.xmlpull.v1.builder.XmlDocument;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlNamespace;
import wsmg.ProcessingContext;
import wsmg.SoapClientFault;
import wsmg.SoapFault;
import wsmg.SoapServerFault;
import wsmg.WsmgConstants;
import wsmg.WsmgRuntimeException;
import wsmg.WsmgVersion;
import wsmg.config.WSMGParameter;
import wsmg.measure.profile.RunTimeStatistics;
import wsmg.util.Counter;
import wsmg.util.TimerThread;
import wsmg.util.WsmgUtil;
import xsul.MLogger;
import xsul.http_server.HttpServerException;
import xsul.http_server.HttpServerRequest;
import xsul.http_server.HttpServerResponse;
import xsul.monitoring.XsulMonitoringStats;
import xsul.monitoring.XsulMonitoringUtil;
import xsul.processor.DynamicInfosetProcessorException;
import xsul.processor.soap_over_http.SoapHttpDynamicInfosetProcessor;
import xsul.soap.SoapUtil;
import xsul.util.FastUUIDGen;
import xsul.ws_addressing.WsAddressing;
import xsul.ws_addressing.WsaEndpointReference;
import xsul.ws_addressing.WsaMessageInformationHeaders;
import xsul.ws_addressing.WsaRelatesTo;

// TODO CRITICAL: add throw new Soap*Fault

/**
 * Implementaiton of Widget service for <a
 * href="http://www-106.ibm.com/developerworks/offers/WS-Specworkshops/ws-rf200404.html">WSRF
 * Interop</a>
 *
 * @version $Revision: 1.17 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 * Restructured by
 * @author Chathura Herath cherath@cs.indiana.edu
 */
public class XSULDedicatedBrockerService extends
    SoapHttpDynamicInfosetProcessor {
    
    private final static MLogger logger = MLogger.getLogger();
    
    
    
    private URI serviceLocation;
    
    /***************************************************************************
     *
     * Debug variables
     **************************************************************************/
    
    private long numberOfConnections;
    
    
    
    private long startTime = System.currentTimeMillis();
    
    private Counter requestXmlMsgCount = new Counter();
    
    private XSULConfiguration xsulConfig;
    
    public XSULDedicatedBrockerService(XSULConfiguration xsulConfig){
        this.xsulConfig = xsulConfig;        
        this.setSupportedSoapFragrances(xsulConfig.getSoapFragrances());
        this.setServerPort(xsulConfig.getPort());
        
        if (WSMGParameter.measureMessageRate) {
            TimerThread timerThread = new TimerThread(requestXmlMsgCount,
                                                      " WebServer");
            new Thread(timerThread).start();
        }
        
    }
    
    public void start() {
        try {
            super.start();
        } catch (DynamicInfosetProcessorException e) {
            throw new WsmgRuntimeException(
                "Fail to start WS-Messenger service", e);
        } catch (IOException e) {
            throw new WsmgRuntimeException(
                "Fail to start WS-Messenger service", e);
        }
        serviceLocation = URI.create(getServer().getLocation()
                                         //                              + "/notification"
                                    );
        System.out
            .println("WS-Eventing/Notification Broker(XSUL) service (version:"
                         +WsmgVersion.SPEC_VERSION+" "+WSMGParameter.versionSetUpNote
                         +" XPathFilterEnabled="+WSMGParameter.useYFilter
                         +") started on "
                         + serviceLocation);
        
    }
    
    protected void serializeXmlResponse(XmlDocument xmlRes, XmlSerializer ser) {
        if (WsmgConstants.indent != null) {
            try {
                ser.setProperty(WsmgConstants.PROPERTY_SERIALIZER_INDENTATION,
                                "  ");
            } catch (IllegalStateException e) {
            } catch (IllegalArgumentException e) {
            }
        }
        //              logger.finest("xmlRes="
        //                              + WsmgConstants.builder.serializeToString(xmlRes));
        super.serializeXmlResponse(xmlRes, ser);
    }
    
    public XmlDocument processHttpXml(HttpServerRequest req, HttpServerResponse res, XmlDocument input)
        throws DynamicInfosetProcessorException
    {
        XmlElement envelope = input.getDocumentElement();
        SoapUtil soapUtil = guessSoapFragrance(envelope);
        //        if(soapUtil == null) {
        //            throw new DynamicInfosetProcessorException(
        //                "unrecognized type of SOAP Envelope "+builder.serializeToString(root));
        //        }
        
        if (soapUtil == null)
        {
            return processUnknownXml(input);
        }
        
        //XmlElement firstChild = soapUtil.requiredBodyContent(envelope);
        //XmlElement body = envelope.findElementByName(Soap11Util.NS_URI_SOAP11, "Body");
        //XmlElement firstChild = (XmlElement) body.requiredElementContent().iterator().next();
        //return soapUtil.wrapBodyContent(processMessage(firstChild));
        
        //public XmlDocument processSoapEnvelope(XmlElement envelope, SoapUtil soapFragrance)
        XmlDocument respDoc = null;
        
        if (logger.isInfoEnabled()) {
            //                      logger.info("*********Received message"
            //                                      + System.currentTimeMillis());
            //                      logger.finest("Received message:"
            //                                      + WsmgConstants.builder.serializeToString(envelope));
            logger.info("Received message:"
                            + WsmgConstants.builder.serializeToString(envelope));
        }
        
        requestXmlMsgCount.addCounter();
        
        // extract resource id - if any
        WsaMessageInformationHeaders headers = new WsaMessageInformationHeaders(
            envelope);
        // extract operation
        ProcessingContext ctx = new ProcessingContext();
        ctx.setSoapBody(soapUtil.requiredBodyContent(envelope));
        ctx.setEnvelope(envelope);
        WsmgUtil.setTopicFromRequestPath(ctx, req.getPathInfo());
        this.xsulConfig.getBroker().processHeaders(ctx);
        XmlElement env = ctx.getEnvelope();
        
        WsaMessageInformationHeaders reqHeaders = new WsaMessageInformationHeaders(
            env);
        XmlNamespace wsaVesion=reqHeaders.guessWsAddressingVersionUsedInHeaders();
        URI respAction = WsAddressing.URI_ROLE_ANONYMOUS;
        try {
            try {
                this.xsulConfig.getBroker().executeSoapEnvelope(ctx);
            } catch (XmlBuilderException ex) {
                throw new SoapClientFault("problem with parsing XML: " + ex, ex);
            }
            if (ctx.getRespMessage() != null) {
                respAction = URI.create(ctx.getRespMessage().getNamespace()
                                            .getNamespaceName()
                                            + "/" + ctx.getRespMessage().getName());
            }
        } catch (SoapServerFault sse) {
            ctx.setRespMessage(soapUtil.generateSoapServerFault(sse.getMessage(), sse));
        } catch (SoapClientFault sce) {
            ctx.setRespMessage(soapUtil.generateSoapClientFault(sce.getMessage(), sce));
        } catch (SoapFault sf) {
            // String ns = sf.getFaultCodeValueNs() != null ?
            // sf.getFaultCodeValueNs().getNamespaceName() : null;
            ctx.setRespMessage(soapUtil.generateSoapFault(sf.getFaultCodeValueNs(),
                                                          sf.getFaultCodeValueName(),
                                                          sf.getMessage(),
                                                          sf));
        }
        // prepare response document with all fancy WS-Addressing
        respDoc = soapUtil.wrapBodyContent(ctx.getRespMessage());
        if (respDoc != null) {
            XmlElement respEnv = respDoc.getDocumentElement();
            
            respEnv.declareNamespace(WsmgConstants.WIDGET_NS); // widget prefix
            
            WsaMessageInformationHeaders respHeaders = new
                WsaMessageInformationHeaders(wsaVesion,
                                             respDoc);
            
            respHeaders.setAction(respAction);
            respHeaders.setMessageId(URI.create("uuid:"+FastUUIDGen.nextUUID()));
            
            respHeaders.setTo(getAnonymousURI(wsaVesion));
            respHeaders.setFrom(new WsaEndpointReference(wsaVesion,serviceLocation));
            
            URI messageId = headers.getMessageId();
            if (messageId != null) {
                respHeaders.addRelatesTo(new WsaRelatesTo(wsaVesion,messageId));
            }
            
        }
        logger.finest("respDoc="
                          + WsmgConstants.builder.serializeToString(respDoc));
        
        return respDoc;
    }
    
    private URI getAnonymousURI(XmlNamespace wsaNs){
        if(wsaNs.getNamespaceName().equals(WsAddressing.NS_2004_03.getNamespaceName())){
            return WsAddressing.URI_ROLE_ANONYMOUS_2004_03;
        }else if(wsaNs.getNamespaceName().equals(WsAddressing.NS_2004_08.getNamespaceName())){
            return WsAddressing.URI_ROLE_ANONYMOUS_2004_08;
        }else{
            return WsAddressing.URI_ROLE_ANONYMOUS;
        }
    }
    
    public XmlElement processMessage(XmlElement message)
        throws DynamicInfosetProcessorException {
        throw new DynamicInfosetProcessorException(
            "internal error: this method should never be called");
    }
    
    public void service(HttpServerRequest req, HttpServerResponse res)
        throws HttpServerException {
        
        ++numberOfConnections;
        
        String method = req.getMethod();
        if ("GET".equals(method) || "POST".equals(method)) {
            if ("GET".equals(method)) {
                doGet(req, res);
            } else if ("POST".equals(method)) {
                // doPost(path, query, req, resp);
                super.service(req, res);
            }
        } else {
            throw new HttpServerException("unsupported method " + method);
        }
    }
    
    //    public static class PathAndQuery {
    //        String path;
    //        String query;
    //    }
    //
    //    public PathAndQuery getPathAndQuery(HttpServerRequest req) {
    //        PathAndQuery paq = new PathAndQuery();
    //        paq.path = req.getPath();
    //        paq.query = null;
    //        final String responseEnc = WsmgConstants.UTF8;
    //        if (paq.path == null) {
    //            paq.path = "";
    //        } else {
    //            if (paq.path.startsWith("/")) {
    //                paq.path = paq.path.substring(1);
    //            }
    //            try {
    //                paq.path = URLDecoder.decode(paq.path, responseEnc);
    //            } catch (UnsupportedEncodingException e) {
    //            }
    //            int qPos = paq.path.indexOf('?');
    //            if (qPos != -1) {
    //                paq.query = paq.path.substring(qPos + 1);
    //                paq.path = paq.path.substring(0, qPos);
    //            }
    //            if (paq.path.endsWith("/")) {
    //                paq.path = paq.path.substring(0, paq.path.length() - 1);
    //            }
    //        }
    //        return paq;
    //    }
    
    private void doGet(HttpServerRequest req,
                       HttpServerResponse res) {
        String path = req.getPathInfo();
        String query = req.getQueryString();
        final String responseEnc = "UTF-8";
        res.setContentType("text/html; charset=\"" + responseEnc + "\"");
        XmlSerializer xs = null;
        try {
            xs = WsmgConstants.builder.getFactory().newSerializer();
            xs.setOutput(res.getOutputStream(), responseEnc);
            xs.text(RunTimeStatistics.getHtmlString());
            XsulMonitoringStats s = XsulMonitoringUtil.createStats("WSNT",
                                                                   startTime, requestXmlMsgCount.getCounterValue(),
                                                                   numberOfConnections);
            XmlDocument xhtmlDoc = XsulMonitoringUtil.createXhtmlStatsDoc(s,
                                                                          responseEnc);

            WsmgConstants.builder.serialize(xhtmlDoc, xs);

            
            if (false) {
                xs.startTag(WsmgConstants.XHTML_NS, "html");
                xs.startTag(WsmgConstants.XHTML_NS, "head");
                String title = "IU XSUL WSRF/Eventing Broker Service ("
                    + WsmgConstants.VERSION + ")";
                xs.startTag(WsmgConstants.XHTML_NS, "title").text(title)
                    .endTag(WsmgConstants.XHTML_NS, "title");
                xs.endTag(WsmgConstants.XHTML_NS, "head");
                xs.startTag(WsmgConstants.XHTML_NS, "body");
                xs.attribute(null, "bgcolor", "white");
                xs.startTag(WsmgConstants.XHTML_NS, "h1").text(title).endTag(
                    WsmgConstants.XHTML_NS, "h1");
                xs.startTag(WsmgConstants.XHTML_NS, "p");
                xs.text("This is WSRF/SOAP/HTTP service that supports ");
                xs.startTag(WsmgConstants.XHTML_NS, "a");  
                xs
                    .attribute(
                    null,
                    "href",
                    "http://www-106.ibm.com/developerworks/offers/WS-Specworkshops/ws-rf200404.html");
                xs.text("WSRF Interop scenario version 0.13");
                xs.endTag(WsmgConstants.XHTML_NS, "a");
                xs.text(".");
                xs.endTag(WsmgConstants.XHTML_NS, "p");
                xs.startTag(WsmgConstants.XHTML_NS, "p");
                xs.text("All scenarios are implemented.");
                xs
                    .text("Constact this endpoint using your WSRF/SOAP/HTTP implementation (endpoints responds to HTTP POST).");
                xs.endTag(WsmgConstants.XHTML_NS, "p");
                xs.startTag(WsmgConstants.XHTML_NS, "p");
                xs.text("For testing purposes ");
                xs.startTag(WsmgConstants.XHTML_NS, "a");
                xs.attribute(null, "href",
                             "http://www.extreme.indiana.edu:11535/");
                xs
                    .text("endpoint that accepts XML notifications is also available");
                xs.endTag(WsmgConstants.XHTML_NS, "a");
                xs.text(". ");
                xs
                    .text("Test endpoint has also WWW/HTTP interface and shows XML content of last few notificaitons.");
                xs.text(".");
                xs.endTag(WsmgConstants.XHTML_NS, "p");
                xs.startTag(WsmgConstants.XHTML_NS, "p");
                xs.text("Changes:");
                xs.startTag(WsmgConstants.XHTML_NS, "ul");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.A: should work OK with IBM web client");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.9: after interop");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.8: notification testing in progress ...");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.7: interop related fixes applied");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.6: fixed various bugs, typos, etc.");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.5: added scenatio 9 and 10");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.4: added scenatio 8");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                
                xs
                    .text("0.13.3: added scenatio 6 (6 suports only pre-defined scenario query and '/*')");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs.text("0.13.2: added scenatio 7, ");
                xs.text("improved handling of namespace prefixes in GMPR");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.startTag(WsmgConstants.XHTML_NS, "li");
                xs
                    .text("0.13.1: added scenario 5, improved date support for fractional digits");
                xs.endTag(WsmgConstants.XHTML_NS, "li");
                xs.endTag(WsmgConstants.XHTML_NS, "ul");
                xs.startTag(WsmgConstants.XHTML_NS, "hr");
                xs.endTag(WsmgConstants.XHTML_NS, "hr");
                xs.endTag(WsmgConstants.XHTML_NS, "p");
                xs.startTag(WsmgConstants.XHTML_NS, "p");
                xs.text("Last Modified "); 
                xs.startTag(WsmgConstants.XHTML_NS, "i");
                xs.text("$Date: 2007/01/12 18:20:55 $");
                xs.endTag(WsmgConstants.XHTML_NS, "i");
                xs.startTag(WsmgConstants.XHTML_NS, "br");
                xs.endTag(WsmgConstants.XHTML_NS, "br");
                xs.text("Service maintained by ");
                xs.startTag(WsmgConstants.XHTML_NS, "a");
                xs.attribute(null, "href",
                             "http://www.extreme.indiana.edu/~aslom/");
                xs.text("Aleksander Slominski");
                xs.endTag(WsmgConstants.XHTML_NS, "a");
                xs.endTag(WsmgConstants.XHTML_NS, "p");
                xs.endTag(WsmgConstants.XHTML_NS, "body");
                xs.endTag(WsmgConstants.XHTML_NS, "html");
            }
        } catch (IOException e) {
            throw new HttpServerException("could not show content of '" + path
                                              + "'", e);
        } catch (org.xmlpull.v1.XmlPullParserException e) {
            throw new HttpServerException("could not show content of '" + path
                                              + "'", e);
            // } catch (XmlPullParserException e) {
            // throw new HttpServerException("could not show content of
            // '"+path+"'", e);
        } finally {
            // try {
            if (xs != null) {
                try {
                    xs.flush();
                } catch (IOException e) {
                }
            }
            // out.close();
            // } catch (IOException e) {}
        }
    }
    
    public URI getServiceLocation() {
        return serviceLocation;
    }
    
    
    
    
}
/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 *
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */





