package wsmg.xsul.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URI;
import java.net.URISyntaxException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.xmlpull.v1.builder.XmlBuilderException;
import org.xmlpull.v1.builder.XmlDocument;
import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;
import wsmg.ProcessingContext;
import wsmg.SoapClientFault;
import wsmg.SoapFault;
import wsmg.SoapServerFault;
import wsmg.WSMGService;
import wsmg.WsmgConfig;
import wsmg.WsmgConstants;
import wsmg.measure.profile.RunTimeStatistics;
import wsmg.util.WsmgUtil;
import xsul.MLogger;
import xsul.XmlConstants;
import xsul.processor.DynamicInfosetProcessorException;
import xsul.soap.SoapUtil;
import xsul.soap11_util.Soap11Util;
import xsul.soap12_util.Soap12Util;
import xsul.util.FastUUIDGen;
import xsul.ws_addressing.WsAddressing;
import xsul.ws_addressing.WsaEndpointReference;
import xsul.ws_addressing.WsaMessageInformationHeaders;
import xsul.ws_addressing.WsaRelatesTo;

/**
 * @author Chathura Herath (cherath@cs.indiana.edu)
 */

public class BrockerServlet extends HttpServlet {
    private final static MLogger logger = MLogger.getLogger();
    
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    
    private final static boolean DEBUG = true;
    
    private final static SoapUtil[] soapFragrances = new SoapUtil[] {
        Soap11Util.getInstance(), Soap12Util.getInstance() };
    
    private static boolean INITIALIZED = false;
    
    private static URI SERVICE_LOCATION;
    
    //HACK -- little chance it is done twice
    protected static WSMGService broker;
    
    public void initialise(String epr) throws ServletException {
        if(broker != null)
            return;
        
        WsmgConfig wsmgConfig = new WsmgConfig();
        try {
            wsmgConfig.setServiceLocation(new URI(epr));
        } catch (URISyntaxException e) {
            throw new ServletException(e);
        }
        wsmgConfig.setBrokerWebServiceStack(WsmgConstants.BROKER_WEB_SERVICE_STACK_TOMCAT_XSUL);
        
        
        broker = new WSMGService(wsmgConfig);
        broker.start();
        INITIALIZED = true;
        if(DEBUG){
            System.err.println("Brocker Initialized");
        }
        
    }
    
    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws IOException, ServletException {
        // use this to add ?WSDL and whatnot
        Writer out = response.getWriter();
        response.setContentType("text/html");
        out.write("<html><head><title>WS-Messenger</title></head>");
        out.write("<body bgcolor='white'><h1>WS-Messenger is running</h1>");
        out.write(RunTimeStatistics.getHtmlString());
        out.write("</body>");
    }
    
    /** Execute SOAP RPC call thatis HTTP POST dispatching to message processor */
    public void doPost(HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException {
        if (!INITIALIZED) {
            String epr = "http://" + req.getServerName() + ":"
                + req.getServerPort() + req.getServletPath();
            
            try {
                SERVICE_LOCATION = new URI(epr);
            } catch (URISyntaxException e) {
                throw new ServletException("Intialization Error:Invalid URI", e);
            }
            initialise(epr);
        }
        
        
        
        Reader reader = null;
        try {
            reader = req.getReader();
        } catch (IOException ioe) {
            System.err.println(getClass() + " got " + ioe);
            ioe.printStackTrace();
            throw ioe;
        }
        res.setContentType("text/xml");
        
        // nice ot see what is going on - in production DEBUG should be false
        if (DEBUG) {
            System.err.println(getClass() + " new request");
            int length = req.getContentLength();
            char[] cbuf = new char[length];
            int toRead = length;
            while (toRead > 0) {
                int received = reader.read(cbuf, length - toRead, toRead);
                toRead -= received;
            }
            System.err.println(getClass() + " request =" + new String(cbuf));
            
            BufferedReader sreader = new BufferedReader(new StringReader(
                                                            new String(cbuf)));
            reader = sreader;
        }
        
        if (DEBUG)
            System.err.println(getClass() + " getting writer");
        Writer rwriter = res.getWriter();
        Writer writer = rwriter;
        if (DEBUG) {
            StringWriter swriter = new StringWriter();
            writer = swriter;
        }
        
        if (DEBUG)
            System.err.println(getClass() + " dispatching");
        
        
        
        XmlDocument inputXml = builder.parseReader(reader);
        
        
        XmlElement root = inputXml.getDocumentElement();
        SoapUtil soapFragrance = SoapUtil.selectSoapFragrance(root,
                                                              soapFragrances);
        
        
        if(DEBUG)
            System.err.println("Identified the soap Fragrance"+soapFragrance.getSoapVersion());
        
        XmlDocument outputXml = processSoapEnvelope(req, root, soapFragrance);
        
        
        
        if (DEBUG)
            System.err.println(getClass() + " leaving doPost()");
        
        builder.serializeToWriter(outputXml, writer);
        
        if (DEBUG) {
            String s = writer.toString();
            System.err.println(getClass() + " response = " + s);
            rwriter.write(s);
            rwriter.close();
        }
        writer.close();
    }
    
    public XmlDocument processSoapEnvelope(HttpServletRequest req, XmlElement envelope, SoapUtil soapFragrance)
        throws DynamicInfosetProcessorException
    {
        XmlDocument respDoc = null;
        
        
        // extract resource id - if any
        WsaMessageInformationHeaders headers = new WsaMessageInformationHeaders(
            envelope);
        // extract operation
        ProcessingContext ctx = new ProcessingContext();
        ctx.setSoapBody(soapFragrance.requiredBodyContent(envelope));
        
        ctx.setEnvelope(envelope);
        
        WsmgUtil.setTopicFromRequestPath(ctx, req.getPathInfo());
        
        broker.processHeaders(ctx);
        
        URI respAction = WsAddressing.URI_ROLE_ANONYMOUS;
        try {
            try {
                broker.executeSoapEnvelope(ctx);
            } catch (XmlBuilderException ex) {
                throw new SoapClientFault("problem with parsing XML: " + ex, ex);
            }
            if (ctx.getRespMessage() != null) {
                respAction = URI.create(ctx.getRespMessage().getNamespace()
                                            .getNamespaceName()
                                            + "/" + ctx.getRespMessage().getName());
            }
        } catch (SoapServerFault sse) {
            ctx.setRespMessage(soapFragrance.generateSoapServerFault(sse
                                                                         .getMessage(), sse));
        } catch (SoapClientFault sce) {
            ctx.setRespMessage(soapFragrance.generateSoapClientFault(sce
                                                                         .getMessage(), sce));
        } catch (SoapFault sf) {
            // String ns = sf.getFaultCodeValueNs() != null ?
            // sf.getFaultCodeValueNs().getNamespaceName() : null;
            ctx.setRespMessage(soapFragrance.generateSoapFault(sf
                                                                   .getFaultCodeValueNs(), sf.getFaultCodeValueName(), sf
                                                                   .getMessage(), sf));
        }
        // prepare response document with all fancy WS-Addressing
        respDoc = soapFragrance.wrapBodyContent(ctx.getRespMessage());
        if (respDoc != null) {
            XmlElement respEnv = respDoc.getDocumentElement();
            
            respEnv.declareNamespace(WsmgConstants.WIDGET_NS); // widget prefix
            
            WsaMessageInformationHeaders respHeaders = new WsaMessageInformationHeaders(
                respDoc);
            
            respHeaders.setAction(respAction);
            respHeaders.setMessageId(URI.create("uuid:"+FastUUIDGen.nextUUID()));
            respHeaders.setTo(WsAddressing.URI_ROLE_ANONYMOUS);
            respHeaders.setFrom(new WsaEndpointReference(SERVICE_LOCATION));
            URI messageId = headers.getMessageId();
            if (messageId != null) {
                respHeaders.addRelatesTo(new WsaRelatesTo(messageId));
            }
            
        }
        
        return respDoc;
    }
    
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 *
 * Copyright (C) 2004 The Trustees of Indiana University. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 *
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 *
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

